@@ -1,11 +1,11 @@
 /* Copyright 2003-2004 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: container.h,v 1.14 2005/07/22 21:12:10 nickm Exp $ */
+/* $Id: container.h,v 1.15 2005/09/09 23:12:54 nickm Exp $ */
 
 #ifndef __CONTAINER_H
 #define __CONTAINER_H
-#define CONTAINER_H_ID "$Id: container.h,v 1.14 2005/07/22 21:12:10 nickm Exp $"
+#define CONTAINER_H_ID "$Id: container.h,v 1.15 2005/09/09 23:12:54 nickm Exp $"
 
 /** Generic resizeable array. */
 typedef struct smartlist_t smartlist_t;
@@ -50,6 +50,10 @@ int smartlist_len(const smartlist_t *sl);
 void smartlist_del(smartlist_t *sl, int idx);
 void smartlist_del_keeporder(smartlist_t *sl, int idx);
 void smartlist_insert(smartlist_t *sl, int idx, void *val);
+void smartlist_sort(smartlist_t *sl,
+                    int (*compare)(const void **a, const void **b));
+void *smartlist_bsearch(smartlist_t *sl, const void *key,
+                        int (*compare)(const void *key, const void **member));
 
 #define SPLIT_SKIP_SPACE   0x01
 #define SPLIT_IGNORE_BLANK 0x02
