@@ -1,8 +1,8 @@
 /* Copyright 2001,2002,2003 Roger Dingledine, Matej Pfajfar.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: crypto.c,v 1.151 2005/08/07 20:36:14 nickm Exp $ */
-const char crypto_c_id[] = "$Id: crypto.c,v 1.151 2005/08/07 20:36:14 nickm Exp $";
+/* $Id: crypto.c,v 1.155 2005/09/09 22:07:15 nickm Exp $ */
+const char crypto_c_id[] = "$Id: crypto.c,v 1.155 2005/09/09 22:07:15 nickm Exp $";
 
 /**
  * \file crypto.c
@@ -22,7 +22,6 @@ const char crypto_c_id[] = "$Id: crypto.c,v 1.151 2005/08/07 20:36:14 nickm Exp
 
 #include <string.h>
 
-#include <openssl/engine.h>
 #include <openssl/err.h>
 #include <openssl/rsa.h>
 #include <openssl/pem.h>
@@ -65,6 +64,12 @@ const char crypto_c_id[] = "$Id: crypto.c,v 1.151 2005/08/07 20:36:14 nickm Exp
 #define OPENSSL_095
 #endif
 
+#if OPENSSL_VERSION_NUMBER < 0x00907000l
+#define NO_ENGINES
+#else
+#include <openssl/engine.h>
+#endif
+
 /* Certain functions that return a success code in OpenSSL 0.9.6 return void
  * (and don't indicate errors) in OpenSSL version 0.9.5.
  *
@@ -161,6 +166,7 @@ crypto_log_errors(int severity, const char *doing)
   }
 }
 
+#ifndef NO_ENGINES
 static void
 log_engine(const char *fn, ENGINE *e)
 {
@@ -174,6 +180,7 @@ log_engine(const char *fn, ENGINE *e)
     log(LOG_INFO, "Using default implementation for %s", fn);
   }
 }
+#endif
 
 /** Initialize the crypto library.  Return 0 on success, -1 on failure.
  */
@@ -185,6 +192,7 @@ crypto_global_init(int useAccel)
     OpenSSL_add_all_algorithms();
     _crypto_global_initialized = 1;
     setup_openssl_threading();
+#ifndef NO_ENGINES
     if (useAccel) {
       if (useAccel < 0)
         log_fn(LOG_WARN, "Initializing OpenSSL via tor_tls_init().");
@@ -201,6 +209,7 @@ crypto_global_init(int useAccel)
       log_engine("3DES", ENGINE_get_cipher_engine(NID_des_ede3_ecb));
       log_engine("AES", ENGINE_get_cipher_engine(NID_aes_128_ecb));
     }
+#endif
   }
   return 0;
 }
@@ -210,7 +219,9 @@ crypto_global_init(int useAccel)
 int crypto_global_cleanup()
 {
   ERR_free_strings();
+#ifndef NO_ENGINES
   ENGINE_cleanup();
+#endif
 #ifdef TOR_IS_MULTITHREADED
   if (_n_openssl_mutexes) {
     int n = _n_openssl_mutexes;
@@ -422,7 +433,7 @@ int crypto_pk_read_private_key_from_filename(crypto_pk_env_t *env, const char *k
   /* Read the file into a string. */
   contents = read_file_to_str(keyfile, 0);
   if (!contents) {
-    log_fn(LOG_WARN, "Error reading private key from %s", keyfile);
+    log_fn(LOG_WARN, "Error reading private key from \"%s\"", keyfile);
     return -1;
   }
 
@@ -1374,6 +1385,7 @@ int crypto_dh_get_public(crypto_dh_env_t *dh, char *pubkey, size_t pubkey_len)
 
 /** Check for bad diffie-hellman public keys (g^x).  Return 0 if the key is
  * okay, or -1 if it's bad.
+ * See http://www.cl.cam.ac.uk/ftp/users/rja14/psandqs.ps.gz for some tips.
  */
 static int
 tor_check_dh_key(BIGNUM *bn)
@@ -1395,7 +1407,7 @@ tor_check_dh_key(BIGNUM *bn)
     log_fn(LOG_WARN, "Rejecting DH key < 0");
     return -1;
   }
-  if (BN_cmp(bn, dh_param_p)>=0){
+  if (BN_cmp(bn, dh_param_p)>=0) {
     log_fn(LOG_WARN, "Rejecting DH key >= p");
     return -1;
   }
@@ -1549,7 +1561,7 @@ int crypto_seed_rng(void)
   for (i = 0; filenames[i]; ++i) {
     fd = open(filenames[i], O_RDONLY, 0);
     if (fd<0) continue;
-    log_fn(LOG_INFO, "Seeding RNG from %s", filenames[i]);
+    log_fn(LOG_INFO, "Seeding RNG from \"%s\"", filenames[i]);
     n = read_all(fd, buf, sizeof(buf), 0);
     close(fd);
     if (n != sizeof(buf)) {
