@@ -1,8 +1,8 @@
 /* Copyright 2003 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: tortls.c,v 1.99 2005/06/21 01:07:32 nickm Exp $ */
-const char tortls_c_id[] = "$Id: tortls.c,v 1.99 2005/06/21 01:07:32 nickm Exp $";
+/* $Id: tortls.c,v 1.101 2005/08/26 18:40:44 arma Exp $ */
+const char tortls_c_id[] = "$Id: tortls.c,v 1.101 2005/08/26 18:40:44 arma Exp $";
 
 /**
  * \file tortls.c
@@ -15,6 +15,7 @@ const char tortls_c_id[] = "$Id: tortls.c,v 1.99 2005/06/21 01:07:32 nickm Exp $
  * functions and variables.)
  */
 
+#include "orconfig.h"
 #include "./crypto.h"
 #include "./tortls.h"
 #include "./util.h"
@@ -651,7 +652,7 @@ tor_tls_get_peer_cert_nickname(tor_tls *tls, char *buf, size_t buflen)
   if (lenout == -1)
     goto error;
   if (((int)strspn(buf, LEGAL_NICKNAME_CHARACTERS)) < lenout) {
-    log_fn(LOG_WARN, "Peer certificate nickname '%s' has illegal characters.",
+    log_fn(LOG_WARN, "Peer certificate nickname \"%s\" has illegal characters.",
            buf);
     if (strchr(buf, '.'))
       log_fn(LOG_WARN, "  (Maybe it is not really running Tor at its advertised OR port.)");
