@@ -1,8 +1,8 @@
 /* Copyright 2003 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: util.c,v 1.214 2005/08/05 01:51:19 nickm Exp $ */
-const char util_c_id[] = "$Id: util.c,v 1.214 2005/08/05 01:51:19 nickm Exp $";
+/* $Id: util.c,v 1.222 2005/09/10 02:34:32 nickm Exp $ */
+const char util_c_id[] = "$Id: util.c,v 1.222 2005/09/10 02:34:32 nickm Exp $";
 
 /**
  * \file util.c
@@ -19,10 +19,13 @@ const char util_c_id[] = "$Id: util.c,v 1.214 2005/08/05 01:51:19 nickm Exp $";
 #include "log.h"
 #include "crypto.h"
 #include "torint.h"
+#include "container.h"
 
 #ifdef MS_WINDOWS
 #include <io.h>
 #include <direct.h>
+#else
+#include <dirent.h>
 #endif
 
 #ifdef HAVE_CTYPE_H
@@ -77,6 +80,9 @@ const char util_c_id[] = "$Id: util.c,v 1.214 2005/08/05 01:51:19 nickm Exp $";
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 
 #ifndef O_BINARY
 #define O_BINARY 0
@@ -293,6 +299,16 @@ void tor_strlower(char *s)
   }
 }
 
+/** Convert all alphabetic characters in the nul-terminated string <b>s</b> to
+ * lowercase. */
+void tor_strupper(char *s)
+{
+  while (*s) {
+    *s = toupper(*s);
+    ++s;
+  }
+}
+
 /* Compares the first strlen(s2) characters of s1 with s2.  Returns as for
  * strcmp.
  */
@@ -886,22 +902,22 @@ int write_bytes_to_file(const char *fname, const char *str, size_t len,
   }
   if ((fd = open(tempname, O_WRONLY|O_CREAT|O_TRUNC|(bin?O_BINARY:O_TEXT), 0600))
       < 0) {
-    log(LOG_WARN, "Couldn't open %s for writing: %s", tempname,
+    log(LOG_WARN, "Couldn't open \"%s\" for writing: %s", tempname,
         strerror(errno));
     goto err;
   }
   result = write_all(fd, str, len, 0);
   if (result < 0 || (size_t)result != len) {
-    log(LOG_WARN, "Error writing to %s: %s", tempname, strerror(errno));
+    log(LOG_WARN, "Error writing to \"%s\": %s", tempname, strerror(errno));
     close(fd);
     goto err;
   }
   if (close(fd)) {
-    log(LOG_WARN,"Error flushing to %s: %s", tempname, strerror(errno));
+    log(LOG_WARN,"Error flushing to \"%s\": %s", tempname, strerror(errno));
     goto err;
   }
   if (replace_file(tempname, fname)) {
-    log(LOG_WARN, "Error replacing %s: %s", fname, strerror(errno));
+    log(LOG_WARN, "Error replacing \"%s\": %s", fname, strerror(errno));
     goto err;
   }
   tor_free(tempname);
@@ -934,13 +950,13 @@ char *read_file_to_str(const char *filename, int bin) {
   r = stat(f, &statbuf);
   tor_free(f);
   if (r < 0) {
-    log_fn(LOG_INFO,"Could not stat %s.",filename);
+    log_fn(LOG_INFO,"Could not stat \"%s\".",filename);
     return NULL;
   }
 
   fd = open(filename,O_RDONLY|(bin?O_BINARY:O_TEXT),0);
   if (fd<0) {
-    log_fn(LOG_WARN,"Could not open %s.",filename);
+    log_fn(LOG_WARN,"Could not open \"%s\".",filename);
     return NULL;
   }
 
@@ -948,7 +964,7 @@ char *read_file_to_str(const char *filename, int bin) {
 
   r = read_all(fd,string,statbuf.st_size,0);
   if (r<0) {
-    log_fn(LOG_WARN,"Error reading from file '%s': %s", filename,
+    log_fn(LOG_WARN,"Error reading from file \"%s\": %s", filename,
            strerror(errno));
     tor_free(string);
     close(fd);
@@ -959,7 +975,7 @@ char *read_file_to_str(const char *filename, int bin) {
   if (bin && r != statbuf.st_size) {
     /* If we're in binary mode, then we'd better have an exact match for
      * size.  Otherwise, win32 encoding may throw us off, and that's okay. */
-    log_fn(LOG_WARN,"Could read only %d of %ld bytes of file '%s'.",
+    log_fn(LOG_WARN,"Could read only %d of %ld bytes of file \"%s\".",
            r, (long)statbuf.st_size,filename);
     tor_free(string);
     close(fd);
@@ -1075,7 +1091,7 @@ char *expand_filename(const char *filename)
       else
         username = tor_strdup(filename+1);
       if (!(home = get_user_homedir(username))) {
-        log_fn(LOG_WARN,"Couldn't get homedir for %s",username);
+        log_fn(LOG_WARN,"Couldn't get homedir for \"%s\"",username);
         tor_free(username);
         return NULL;
       }
@@ -1103,6 +1119,54 @@ char *expand_filename(const char *filename)
   }
 }
 
+/** Return a new list containing the filenames in the directory <b>dirname</b>.
+ * Return NULL on error or if <b>dirname</b> is not a directory.
+ */
+smartlist_t *
+tor_listdir(const char *dirname)
+{
+  smartlist_t *result;
+#ifdef MS_WINDOWS
+  char *pattern;
+  HANDLE handle;
+  WIN32_FIND_DATA findData;
+  size_t pattern_len = strlen(dirname)+16;
+  pattern = tor_malloc(pattern_len);
+  tor_snprintf(pattern, pattern_len, "%s\\*", dirname);
+  if (!(handle = FindFirstFile(pattern, &findData))) {
+    tor_free(pattern);
+    return NULL;
+  }
+  result = smartlist_create();
+  while (1) {
+    smartlist_add(result, tor_strdup(findData.cFileName));
+    if (!FindNextFile(handle, &findData)) {
+      if (GetLastError() != ERROR_NO_MORE_FILES) {
+        log_fn(LOG_WARN, "Error reading directory.");
+      }
+      break;
+    }
+  }
+  FindClose(handle);
+  tor_free(pattern);
+#else
+  DIR *d;
+  struct dirent *de;
+  if (!(d = opendir(dirname)))
+    return NULL;
+
+  result = smartlist_create();
+  while ((de = readdir(d))) {
+    if (!strcmp(de->d_name, ".") ||
+        !strcmp(de->d_name, ".."))
+      continue;
+    smartlist_add(result, tor_strdup(de->d_name));
+  }
+  closedir(d);
+#endif
+  return result;
+}
+
 /* =====
  * Net helpers
  * ===== */
@@ -1232,7 +1296,7 @@ parse_addr_and_port_range(const char *s, uint32_t *addr_out,
   } else if (tor_inet_aton(address, &in) != 0) {
     *addr_out = ntohl(in.s_addr);
   } else {
-    log_fn(LOG_WARN, "Malformed IP %s in address pattern; rejecting.",address);
+    log_fn(LOG_WARN, "Malformed IP \"%s\" in address pattern; rejecting.",address);
     goto err;
   }
 
@@ -1254,7 +1318,7 @@ parse_addr_and_port_range(const char *s, uint32_t *addr_out,
     } else if (tor_inet_aton(mask, &in) != 0) {
       *mask_out = ntohl(in.s_addr);
     } else {
-      log_fn(LOG_WARN, "Malformed mask %s on address range; rejecting.",
+      log_fn(LOG_WARN, "Malformed mask \"%s\" on address range; rejecting.",
              mask);
       goto err;
     }
@@ -1273,11 +1337,11 @@ parse_addr_and_port_range(const char *s, uint32_t *addr_out,
       *port_max_out = (uint16_t) tor_parse_long(port, 10, 1, 65535, NULL,
                                                 &endptr);
       if (*endptr || !*port_max_out) {
-      log_fn(LOG_WARN, "Malformed port %s on address range rejecting.",
+      log_fn(LOG_WARN, "Malformed port \"%s\" on address range rejecting.",
              port);
       }
     } else if (*endptr || !*port_min_out) {
-      log_fn(LOG_WARN, "Malformed port %s on address range; rejecting.",
+      log_fn(LOG_WARN, "Malformed port \"%s\" on address range; rejecting.",
              port);
       goto err;
     } else {
@@ -1311,6 +1375,20 @@ tor_inet_ntoa(struct in_addr *in, char *buf, size_t buf_len)
                       (int)(uint8_t)((a    )&0xff));
 }
 
+/** Given a host-order <b>addr</b>, call tor_inet_ntoa() on it
+ * and return a strdup of the resulting address.
+ */
+char *
+tor_dup_addr(uint32_t addr)
+{
+  char buf[INET_NTOA_BUF_LEN];
+  struct in_addr in;
+
+  in.s_addr = htonl(addr);
+  tor_inet_ntoa(&in, buf, sizeof(buf));
+  return tor_strdup(buf);
+}
+
 /* Return true iff <b>name</b> looks like it might be a hostname or IP
  * address of some kind. */
 int
@@ -1459,7 +1537,7 @@ void finish_daemon(const char *desired_cwd)
     desired_cwd = "/";
    /* Don't hold the wrong FS mounted */
   if (chdir(desired_cwd) < 0) {
-    log_fn(LOG_ERR,"chdir to %s failed. Exiting.",desired_cwd);
+    log_fn(LOG_ERR,"chdir to \"%s\" failed. Exiting.",desired_cwd);
     exit(1);
   }
 
@@ -1497,7 +1575,7 @@ void write_pidfile(char *filename) {
   FILE *pidfile;
 
   if ((pidfile = fopen(filename, "w")) == NULL) {
-    log_fn(LOG_WARN, "Unable to open %s for writing: %s", filename,
+    log_fn(LOG_WARN, "Unable to open \"%s\" for writing: %s", filename,
            strerror(errno));
   } else {
     fprintf(pidfile, "%d\n", (int)getpid());
