@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: buffers.c,v 1.168 2005/08/04 20:12:44 nickm Exp $ */
-const char buffers_c_id[] = "$Id: buffers.c,v 1.168 2005/08/04 20:12:44 nickm Exp $";
+/* $Id: buffers.c,v 1.169 2005/08/30 07:01:30 arma Exp $ */
+const char buffers_c_id[] = "$Id: buffers.c,v 1.169 2005/08/30 07:01:30 arma Exp $";
 
 /**
  * \file buffers.c
@@ -1065,7 +1065,7 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
 "client accordingly.\n"
 "</p>\n"
 "<p>\n"
-"See <a href=\"http://tor.eff.org/doc/tor-doc.html#installing\">http://tor.eff.org/doc/tor-doc.html#installing</a> for more information.\n"
+"See <a href=\"http://tor.eff.org/documentation.html\">http://tor.eff.org/documentation.html</a> for more information.\n"
 "<!-- Plus this comment, to make the body response more than 512 bytes, so IE will be willing to display it. Comment comment comment comment comment comment comment comment comment comment comment comment.-->\n"
 "</p>\n"
 "</body>\n"
