@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitbuild.c,v 1.132 2005/08/07 21:24:00 nickm Exp $ */
-const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.132 2005/08/07 21:24:00 nickm Exp $";
+/* $Id: circuitbuild.c,v 1.141 2005/09/10 01:01:40 arma Exp $ */
+const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.141 2005/09/10 01:01:40 arma Exp $";
 
 /**
  * \file circuitbuild.c
@@ -531,12 +531,21 @@ circuit_send_next_onion_skin(circuit_t *circ)
       if (!has_completed_circuit) {
         or_options_t *options = get_options();
         has_completed_circuit=1;
-        log(LOG_NOTICE,"Tor has successfully opened a circuit. Looks like it's working.");
         /* XXX009 Log a count of known routers here */
-        if (server_mode(options) && !check_whether_orport_reachable())
-          log(LOG_NOTICE,"Now checking whether ORPort %s%s reachable... (this may take several minutes)",
-                 options->DirPort ? "and DirPort " : "",
-                 options->DirPort ? "are" : "is");
+        log(LOG_NOTICE,"Tor has successfully opened a circuit. Looks like it's working.");
+        if (server_mode(options) && !check_whether_orport_reachable()) {
+          char dirbuf[128];
+          routerinfo_t *me = router_get_my_routerinfo();
+          if (me) {
+            if (me->dir_port)
+              tor_snprintf(dirbuf, sizeof(dirbuf), " and DirPort %s:%d",
+                           me->address, me->dir_port);
+            log(LOG_NOTICE,"Now checking whether ORPort %s:%d%s %s reachable... (this may take several minutes)",
+                   me->address, me->or_port,
+                   me->dir_port ? dirbuf : "",
+                   me->dir_port ? "are" : "is");
+          }
+        }
       }
       circuit_rep_hist_note_result(circ);
       circuit_has_opened(circ); /* do other actions as necessary */
@@ -885,7 +894,8 @@ onionskin_answer(circuit_t *circ, uint8_t cell_type, char *payload, char *keys)
  * is feasible, except if it's less than 2, in which case return -1.
  */
 static int
-new_route_len(double cw, uint8_t purpose, smartlist_t *routers)
+new_route_len(double cw, uint8_t purpose, extend_info_t *exit,
+              smartlist_t *routers)
 {
   int num_acceptable_routers;
   int routelen;
@@ -897,23 +907,11 @@ new_route_len(double cw, uint8_t purpose, smartlist_t *routers)
 #ifdef TOR_PERF
   routelen = 2;
 #else
-  if (purpose == CIRCUIT_PURPOSE_C_GENERAL)
-    routelen = 3;
-  else if (purpose == CIRCUIT_PURPOSE_TESTING)
-    routelen = 3;
-  else if (purpose == CIRCUIT_PURPOSE_C_INTRODUCING)
-    routelen = 4;
-  else if (purpose == CIRCUIT_PURPOSE_C_ESTABLISH_REND)
-    routelen = 3;
-  else if (purpose == CIRCUIT_PURPOSE_S_ESTABLISH_INTRO)
-    routelen = 3;
-  else if (purpose == CIRCUIT_PURPOSE_S_CONNECT_REND)
-    routelen = 4;
-  else {
-    log_fn(LOG_WARN,"Bug: unhandled purpose %d", purpose);
-    tor_fragile_assert();
-    return -1;
-  }
+  routelen = 3;
+  if (exit &&
+      purpose != CIRCUIT_PURPOSE_TESTING &&
+      purpose != CIRCUIT_PURPOSE_S_ESTABLISH_INTRO)
+    routelen++;
 #endif
   log_fn(LOG_DEBUG,"Chosen route length %d (%d routers available).",routelen,
          smartlist_len(routers));
@@ -1159,6 +1157,12 @@ choose_good_exit_server_general(routerlist_t *dir, int need_uptime,
     smartlist_t *needed_ports = circuit_get_unhandled_ports(time(NULL));
 
     if (best_support == -1) {
+      if (need_uptime || need_capacity) {
+        log_fn(LOG_INFO, "We couldn't find any live%s%s routers; falling back to list of all routers.",
+               need_capacity?", fast":"",
+               need_uptime?", stable":"");
+        return choose_good_exit_server_general(dir, 0, 0);
+      }
       log(LOG_NOTICE, "All routers are down or middleman -- choosing a doomed exit at random.");
     }
     for (try = 0; try < 2; try++) {
@@ -1168,7 +1172,7 @@ choose_good_exit_server_general(routerlist_t *dir, int need_uptime,
         router = smartlist_get(dir->routers, i);
         if (n_supported[i] != -1 &&
             (try || router_handles_some_port(router, needed_ports))) {
-          log_fn(LOG_DEBUG,"Try %d: '%s' is a possibility.", try, router->nickname);
+//          log_fn(LOG_DEBUG,"Try %d: '%s' is a possibility.", try, router->nickname);
           smartlist_add(sl, router);
         }
       }
@@ -1244,7 +1248,8 @@ onion_pick_cpath_exit(circuit_t *circ, extend_info_t *exit)
     log_fn(LOG_WARN,"router_get_routerlist returned empty list; closing circ.");
     return -1;
   }
-  r = new_route_len(get_options()->PathlenCoinWeight, circ->purpose, rl->routers);
+  r = new_route_len(get_options()->PathlenCoinWeight, circ->purpose,
+                    exit, rl->routers);
   if (r < 1) /* must be at least 1 */
     return -1;
   state->desired_path_len = r;
@@ -1416,7 +1421,7 @@ choose_good_entry_server(cpath_build_state_t *state)
     smartlist_add(excluded, r);
     routerlist_add_family(excluded, r);
   }
-  if (options->FascistFirewall) {
+  if (firewall_is_fascist()) {
     /* exclude all ORs that listen on the wrong port */
     routerlist_t *rl;
     int i;
@@ -1427,7 +1432,7 @@ choose_good_entry_server(cpath_build_state_t *state)
 
     for (i=0; i < smartlist_len(rl->routers); i++) {
       r = smartlist_get(rl->routers, i);
-      if (!fascist_firewall_allows_address(options,r->addr,r->or_port))
+      if (!fascist_firewall_allows_address(r->addr,r->or_port))
         smartlist_add(excluded, r);
     }
   }
@@ -1672,7 +1677,7 @@ remove_dead_helpers(void)
 
   for (i = 0; i < smartlist_len(helper_nodes); ) {
     helper_node_t *helper = smartlist_get(helper_nodes, i);
-    char *why = NULL;
+    const char *why = NULL;
     time_t since = 0;
     if (helper->unlisted_since + HELPER_ALLOW_UNLISTED > now) {
       why = "unlisted";
@@ -1986,3 +1991,4 @@ helper_nodes_getinfo_helper(const char *question, char **answer)
   }
   return 0;
 }
+
