@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection.c,v 1.393 2005/08/07 19:27:38 nickm Exp $ */
-const char connection_c_id[] = "$Id: connection.c,v 1.393 2005/08/07 19:27:38 nickm Exp $";
+/* $Id: connection.c,v 1.396 2005/09/08 21:21:54 nickm Exp $ */
+const char connection_c_id[] = "$Id: connection.c,v 1.396 2005/09/08 21:21:54 nickm Exp $";
 
 /**
  * \file connection.c
@@ -231,6 +231,7 @@ _connection_free(connection_t *conn)
   tor_free(conn->incoming_cmd);
   tor_free(conn->read_event); /* Probably already freed by connection_free. */
   tor_free(conn->write_event); /* Probably already freed by connection_free. */
+  tor_free(conn->requested_resource);
 
   if (conn->s >= 0) {
     log_fn(LOG_INFO,"closing fd %d.",conn->s);
@@ -669,10 +670,9 @@ connection_handle_listener_read(connection_t *conn, int new_type)
   newconn->s = news;
 
   /* remember the remote address */
-  newconn->address = tor_malloc(INET_NTOA_BUF_LEN);
-  tor_inet_ntoa(&remote.sin_addr, newconn->address, INET_NTOA_BUF_LEN);
   newconn->addr = ntohl(remote.sin_addr.s_addr);
   newconn->port = ntohs(remote.sin_port);
+  newconn->address = tor_dup_addr(newconn->addr);
 
   if (connection_add(newconn) < 0) { /* no space, forget it */
     connection_free(newconn);
@@ -1952,6 +1952,9 @@ assert_connection_ok(connection_t *conn, time_t now)
   } else if (conn->type != CONN_TYPE_DIR) {
     tor_assert(!conn->purpose); /* only used for dir types currently */
   }
+  if (conn->type != CONN_TYPE_DIR) {
+    tor_assert(!conn->requested_resource);
+  }
 
   switch (conn->type)
     {
