@@ -1,7 +1,7 @@
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: control.c,v 1.114 2005/08/07 19:27:38 nickm Exp $ */
-const char control_c_id[] = "$Id: control.c,v 1.114 2005/08/07 19:27:38 nickm Exp $";
+/* $Id: control.c,v 1.127 2005/09/08 06:37:50 nickm Exp $ */
+const char control_c_id[] = "$Id: control.c,v 1.127 2005/09/08 06:37:50 nickm Exp $";
 
 /**
  * \file control.c
@@ -75,7 +75,8 @@ const char control_c_id[] = "$Id: control.c,v 1.114 2005/08/07 19:27:38 nickm Ex
 #define _EVENT_MAX            0x000C
 
 /** Array mapping from message type codes to human-readable message
- * type names.  */
+ * type names. Used for compatibility with version 0 of the control
+ * protocol. */
 static const char * CONTROL0_COMMANDS[_CONTROL0_CMD_MAX_RECOGNIZED+1] = {
   "error",
   "done",
@@ -127,6 +128,10 @@ static char authentication_cookie[AUTHENTICATION_COOKIE_LEN];
 
 static void connection_printf_to_buf(connection_t *conn, const char *format, ...)
   CHECK_PRINTF(2,3);
+/*static*/ size_t write_escaped_data(const char *data, size_t len,
+                                     int translate_newlines, char **out);
+/*static*/ size_t read_escaped_data(const char *data, size_t len,
+                                    int translate_newlines,  char **out);
 static void send_control0_message(connection_t *conn, uint16_t type,
                                  uint32_t len, const char *body);
 static void send_control_done(connection_t *conn);
@@ -138,6 +143,8 @@ static void send_control1_event(uint16_t event, const char *format, ...)
   CHECK_PRINTF(2,3);
 static int handle_control_setconf(connection_t *conn, uint32_t len,
                                   char *body);
+static int handle_control_resetconf(connection_t *conn, uint32_t len,
+                                    char *body);
 static int handle_control_getconf(connection_t *conn, uint32_t len,
                                   const char *body);
 static int handle_control_setevents(connection_t *conn, uint32_t len,
@@ -201,8 +208,8 @@ log_severity_to_event(int severity)
   }
 }
 
-/** Set <b>global_event_mask</b> to the bitwise OR of each live control
- * connection's event_mask field. */
+/** Set <b>global_event_maskX</b> (where X is 0 or 1) to the bitwise OR
+ * of each live control connection's event_mask field. */
 void
 control_update_global_event_mask(void)
 {
@@ -402,7 +409,7 @@ get_escaped_string(const char *start, size_t in_len_max,
     *outp++ = *cp++;
   }
   *outp = '\0';
-  tor_assert((outp - *out) == *out_len);
+  tor_assert((outp - *out) == (int)*out_len);
 
   return end+1;
 }
@@ -597,10 +604,11 @@ get_stream(const char *id)
   return conn;
 }
 
-/** Called when we receive a SETCONF message: parse the body and try
- * to update our configuration.  Reply with a DONE or ERROR message. */
+/** Helper for setconf and resetconf. Acts like setconf, except
+ * it passes <b>reset</b> on to options_trial_assign().
+ */
 static int
-handle_control_setconf(connection_t *conn, uint32_t len, char *body)
+control_setconf_helper(connection_t *conn, uint32_t len, char *body, int reset)
 {
   int r;
   config_line_t *lines=NULL;
@@ -658,14 +666,14 @@ handle_control_setconf(connection_t *conn, uint32_t len, char *body)
     }
   }
 
-  if ((r=options_trial_assign(lines, 1)) < 0) {
+  if ((r=options_trial_assign(lines, reset)) < 0) {
     log_fn(LOG_WARN,"Controller gave us config lines that didn't validate.");
     if (r==-1) {
       if (v0)
         send_control0_error(conn, ERR_UNRECOGNIZED_CONFIG_KEY,
                             "Unrecognized option");
       else
-        connection_write_str_to_buf("552 Unrecognzied option\r\n", conn);
+        connection_write_str_to_buf("552 Unrecognized option\r\n", conn);
     } else {
       if (v0)
         send_control0_error(conn,ERR_INVALID_CONFIG_VALUE,"Invalid option value");
@@ -675,16 +683,29 @@ handle_control_setconf(connection_t *conn, uint32_t len, char *body)
     config_free_lines(lines);
     return 0;
   }
-
   config_free_lines(lines);
-  if (options_act() < 0) { /* acting on them failed. die. */
-    log_fn(LOG_ERR,"Acting on config options left us in a broken state. Dying.");
-    exit(1);
-  }
   send_control_done(conn);
   return 0;
 }
 
+/** Called when we receive a SETCONF message: parse the body and try
+ * to update our configuration.  Reply with a DONE or ERROR message. */
+static int
+handle_control_setconf(connection_t *conn, uint32_t len, char *body)
+{
+  return control_setconf_helper(conn, len, body, 0);
+}
+
+/** Called when we receive a RESETCONF message: parse the body and try
+ * to update our configuration.  Reply with a DONE or ERROR message. */
+static int
+handle_control_resetconf(connection_t *conn, uint32_t len, char *body)
+{
+  int v0 = STATE_IS_V0(conn->state);
+  tor_assert(!v0);
+  return control_setconf_helper(conn, len, body, 1);
+}
+
 /** Called when we receive a GETCONF message.  Parse the request, and
  * reply with a CONFVALUE or an ERROR message */
 static int
@@ -1150,6 +1171,8 @@ handle_getinfo_helper(const char *question, char **answer)
   *answer = NULL; /* unrecognized key by default */
   if (!strcmp(question, "version")) {
     *answer = tor_strdup(VERSION);
+  } else if (!strcmp(question, "config-file")) {
+    *answer = tor_strdup(get_torrc_fname());
   } else if (!strcmpstart(question, "accounting/")) {
     return accounting_getinfo_helper(question, answer);
   } else if (!strcmpstart(question, "helper-nodes")) {
@@ -1166,6 +1189,9 @@ handle_getinfo_helper(const char *question, char **answer)
     routerinfo_t *ri = router_get_by_nickname(question+strlen("desc/name/"));
     if (ri && ri->signed_descriptor)
       *answer = tor_strdup(ri->signed_descriptor);
+  } else if (!strcmpstart(question, "unregistered-servers-")) {
+    *answer = dirserver_getinfo_unregistered(question +
+                                             strlen("unregistered-servers-"));
   } else if (!strcmp(question, "network-status")) {
     routerlist_t *routerlist;
     router_get_routerlist(&routerlist);
@@ -1197,7 +1223,7 @@ handle_getinfo_helper(const char *question, char **answer)
       smartlist_add(status, s);
       tor_free(path);
     }
-    *answer = smartlist_join_strings(status, "\r\n", 1, NULL);
+    *answer = smartlist_join_strings(status, "\r\n", 0, NULL);
     SMARTLIST_FOREACH(status, char *, cp, tor_free(cp));
     smartlist_free(status);
   } else if (!strcmp(question, "stream-status")) {
@@ -1247,7 +1273,7 @@ handle_getinfo_helper(const char *question, char **answer)
                    buf);
       smartlist_add(status, s);
     }
-    *answer = smartlist_join_strings(status, "\r\n", 1, NULL);
+    *answer = smartlist_join_strings(status, "\r\n", 0, NULL);
     SMARTLIST_FOREACH(status, char *, cp, tor_free(cp));
     smartlist_free(status);
   } else if (!strcmp(question, "orconn-status")) {
@@ -1270,7 +1296,7 @@ handle_getinfo_helper(const char *question, char **answer)
       tor_snprintf(s, slen, "%s %s",conns[i]->nickname,state);
       smartlist_add(status, s);
     }
-    *answer = smartlist_join_strings(status, "\r\n", 1, NULL);
+    *answer = smartlist_join_strings(status, "\r\n", 0, NULL);
     SMARTLIST_FOREACH(status, char *, cp, tor_free(cp));
     smartlist_free(status);
   } else if (!strcmpstart(question, "addr-mappings/")) {
@@ -1289,7 +1315,7 @@ handle_getinfo_helper(const char *question, char **answer)
     }
     mappings = smartlist_create();
     addressmap_get_mappings(mappings, min_e, max_e);
-    *answer = smartlist_join_strings(mappings, "\n", 1, NULL);
+    *answer = smartlist_join_strings(mappings, "\n", 0, NULL);
     SMARTLIST_FOREACH(mappings, char *, cp, tor_free(cp));
     smartlist_free(mappings);
   }
@@ -1303,7 +1329,7 @@ handle_control_getinfo(connection_t *conn, uint32_t len, const char *body)
   smartlist_t *questions = NULL;
   smartlist_t *answers = NULL;
   smartlist_t *unrecognized = NULL;
-  char *msg = NULL, *ans;
+  char *msg = NULL, *ans = NULL;
   size_t msg_len;
   int v0 = STATE_IS_V0(conn->state);
 
@@ -1881,7 +1907,8 @@ connection_control_reached_eof(connection_t *conn)
 static int
 connection_control_process_inbuf_v1(connection_t *conn)
 {
-  size_t data_len, cmd_len;
+  size_t data_len;
+  int cmd_len;
   char *args;
 
   tor_assert(conn);
@@ -1938,7 +1965,8 @@ connection_control_process_inbuf_v1(connection_t *conn)
    * recognize it.
    */
   cmd_len = 0;
-  while (cmd_len < data_len && !isspace(conn->incoming_cmd[cmd_len]))
+  while ((size_t)cmd_len < data_len
+         && !TOR_ISSPACE(conn->incoming_cmd[cmd_len]))
     ++cmd_len;
 
   /*
@@ -1957,6 +1985,12 @@ connection_control_process_inbuf_v1(connection_t *conn)
   log_fn(LOG_NOTICE, "ARGS ARE: <%s>", args);
   */
 
+  if (!strcasecmp(conn->incoming_cmd, "QUIT")) {
+    connection_write_str_to_buf("250 closing connection\r\n", conn);
+    connection_mark_for_close(conn);
+    return 0;
+  }
+
   if (conn->state == CONTROL_CONN_STATE_NEEDAUTH_V1 &&
       strcasecmp(conn->incoming_cmd, "AUTHENTICATE")) {
     connection_write_str_to_buf("514 Authentication required.\r\n", conn);
@@ -1967,6 +2001,9 @@ connection_control_process_inbuf_v1(connection_t *conn)
   if (!strcasecmp(conn->incoming_cmd, "SETCONF")) {
     if (handle_control_setconf(conn, data_len, args))
       return -1;
+  } else if (!strcasecmp(conn->incoming_cmd, "RESETCONF")) {
+    if (handle_control_resetconf(conn, data_len, args))
+      return -1;
   } else if (!strcasecmp(conn->incoming_cmd, "GETCONF")) {
     if (handle_control_getconf(conn, data_len, args))
       return -1;
@@ -2283,6 +2320,7 @@ control_event_stream_status(connection_t *conn, stream_status_event_t tp)
                         (unsigned long)conn->global_identifier, status,
                         circ?(unsigned long)circ->global_identifier : 0ul,
                         buf);
+    /* XXX need to specify its intended exit, etc? */
   }
   return 0;
 }
