@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: directory.c,v 1.241 2005/08/08 17:32:17 nickm Exp $ */
-const char directory_c_id[] = "$Id: directory.c,v 1.241 2005/08/08 17:32:17 nickm Exp $";
+/* $Id: directory.c,v 1.270 2005/09/09 19:37:12 nickm Exp $ */
+const char directory_c_id[] = "$Id: directory.c,v 1.270 2005/09/09 19:37:12 nickm Exp $";
 
 #include "or.h"
 
@@ -49,6 +49,8 @@ directory_send_command(connection_t *conn, const char *platform,
 static int directory_handle_command(connection_t *conn);
 static int body_is_plausible(const char *body, size_t body_len, int purpose);
 static int purpose_is_private(uint8_t purpose);
+static char *http_get_header(const char *headers, const char *which);
+static char *http_get_origin(const char *headers, connection_t *conn);
 
 /********* START VARIABLES **********/
 
@@ -69,7 +71,7 @@ parse_dir_policy(void)
     addr_policy_free(dir_policy);
     dir_policy = NULL;
   }
-  config_parse_addr_policy(get_options()->DirPolicy, &dir_policy);
+  config_parse_addr_policy(get_options()->DirPolicy, &dir_policy, -1);
   /* ports aren't used. */
   for (n=dir_policy; n; n = n->next) {
     n->prt_min = 1;
@@ -111,7 +113,9 @@ purpose_is_private(uint8_t purpose)
 {
   if (purpose == DIR_PURPOSE_FETCH_DIR ||
       purpose == DIR_PURPOSE_UPLOAD_DIR ||
-      purpose == DIR_PURPOSE_FETCH_RUNNING_LIST)
+      purpose == DIR_PURPOSE_FETCH_RUNNING_LIST ||
+      purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS ||
+      purpose == DIR_PURPOSE_FETCH_SERVERDESC)
     return 0;
   return 1;
 }
@@ -138,7 +142,7 @@ directory_post_to_dirservers(uint8_t purpose, const char *payload,
        * router descriptor, but not when uploading a service
        * descriptor -- those use Tor. */
       if (purpose == DIR_PURPOSE_UPLOAD_DIR && !get_options()->HttpProxy) {
-        if (!fascist_firewall_allows_address(get_options(),ds->addr,ds->dir_port))
+        if (!fascist_firewall_allows_address(ds->addr,ds->dir_port))
           continue;
       }
       directory_initiate_command_trusted_dir(ds, purpose, purpose_is_private(purpose),
@@ -159,41 +163,59 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
 {
   routerinfo_t *r = NULL;
   trusted_dir_server_t *ds = NULL;
-  int fascistfirewall = get_options()->FascistFirewall;
-  int directconn = purpose == DIR_PURPOSE_FETCH_DIR ||
-                   purpose == DIR_PURPOSE_FETCH_RUNNING_LIST;
-  int fetch_fresh_first = advertised_server_mode();
-  int priv = purpose_is_private(purpose);
+  int fascistfirewall = firewall_is_fascist();
+  or_options_t *options = get_options();
+  int fetch_fresh_first = server_mode(options) && options->DirPort != 0;
+  int directconn = !purpose_is_private(purpose);
+
+  int need_v1_support = purpose == DIR_PURPOSE_FETCH_DIR ||
+                        purpose == DIR_PURPOSE_FETCH_RUNNING_LIST;
+  int need_v2_support = purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS ||
+                        purpose == DIR_PURPOSE_FETCH_SERVERDESC;
 
   if (directconn) {
-    if (fetch_fresh_first) {
+    if (fetch_fresh_first && purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS &&
+        !strcmpstart(resource,"fp/") && strlen(resource) == HEX_DIGEST_LEN+3) {
+      /* Try to ask the actual dirserver its opinion. */
+      char digest[DIGEST_LEN];
+      base16_decode(digest, DIGEST_LEN, resource+3, HEX_DIGEST_LEN);
+      ds = router_get_trusteddirserver_by_digest(digest);
+    }
+    if (!ds && fetch_fresh_first) {
       /* only ask authdirservers, and don't ask myself */
-      ds = router_pick_trusteddirserver(1, fascistfirewall,
+      ds = router_pick_trusteddirserver(need_v1_support, 1, fascistfirewall,
                                         retry_if_no_servers);
     }
     if (!ds) {
       /* anybody with a non-zero dirport will do */
-      r = router_pick_directory_server(1, fascistfirewall,
-                                purpose==DIR_PURPOSE_FETCH_RUNNING_LIST,
+      r = router_pick_directory_server(1, fascistfirewall, need_v2_support,
                                        retry_if_no_servers);
       if (!r) {
-        log_fn(LOG_INFO, "No router found for %s; falling back to dirserver list",
-               purpose == DIR_PURPOSE_FETCH_RUNNING_LIST
-               ? "status list" : "directory");
-        ds = router_pick_trusteddirserver(1, fascistfirewall,
+        const char *which;
+        if (purpose == DIR_PURPOSE_FETCH_DIR)
+          which = "directory";
+        else if (purpose == DIR_PURPOSE_FETCH_RUNNING_LIST)
+          which = "status list";
+        else if (purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS)
+          which = "network status";
+        else // if (purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS)
+          which = "server descriptors";
+        log_fn(LOG_INFO,
+               "No router found for %s; falling back to dirserver list",which);
+        ds = router_pick_trusteddirserver(1, 1, fascistfirewall,
                                           retry_if_no_servers);
       }
     }
   } else { // (purpose == DIR_PURPOSE_FETCH_RENDDESC)
     /* only ask authdirservers, any of them will do */
     /* Never use fascistfirewall; we're going via Tor. */
-    ds = router_pick_trusteddirserver(0, 0, retry_if_no_servers);
+    ds = router_pick_trusteddirserver(0, 0, 0, retry_if_no_servers);
   }
 
   if (r)
-    directory_initiate_command_router(r, purpose, priv, resource, NULL, 0);
+    directory_initiate_command_router(r, purpose, !directconn, resource, NULL, 0);
   else if (ds)
-    directory_initiate_command_trusted_dir(ds, purpose, priv, resource, NULL, 0);
+    directory_initiate_command_trusted_dir(ds, purpose, !directconn, resource, NULL, 0);
   else {
     log_fn(LOG_NOTICE,"No running dirservers known. Will try again later. (purpose %d)",
            purpose);
@@ -290,6 +312,12 @@ directory_initiate_command(const char *address, uint32_t addr,
     case DIR_PURPOSE_FETCH_RUNNING_LIST:
       log_fn(LOG_DEBUG,"initiating running-routers fetch");
       break;
+    case DIR_PURPOSE_FETCH_NETWORKSTATUS:
+      log_fn(LOG_DEBUG,"initiating network-status fetch");
+      break;
+    case DIR_PURPOSE_FETCH_SERVERDESC:
+      log_fn(LOG_DEBUG,"initiating server descriptor fetch");
+      break;
     default:
       log_fn(LOG_ERR, "Unrecognized directory connection purpose.");
       tor_assert(0);
@@ -361,16 +389,21 @@ directory_send_command(connection_t *conn, const char *platform,
                        int purpose, const char *resource,
                        const char *payload, size_t payload_len)
 {
-  char tmp[8192];
   char proxystring[256];
   char proxyauthstring[256];
   char hoststring[128];
-  char url[128];
+  char *url;
+  char request[8192];
   const char *httpcommand = NULL;
+  size_t len;
 
   tor_assert(conn);
   tor_assert(conn->type == CONN_TYPE_DIR);
 
+  tor_free(conn->requested_resource);
+  if (resource)
+    conn->requested_resource = tor_strdup(resource);
+
   /* come up with a string for which Host: we want */
   if (conn->port == 80) {
     strlcpy(hoststring, conn->address, sizeof(hoststring));
@@ -409,19 +442,31 @@ directory_send_command(connection_t *conn, const char *platform,
       log_fn(LOG_DEBUG, "Asking for compressed directory from server running %s",
              platform?platform:"<unknown version>");
       httpcommand = "GET";
-      strlcpy(url, "/tor/dir.z", sizeof(url));
+      url = tor_strdup("/tor/dir.z");
       break;
     case DIR_PURPOSE_FETCH_RUNNING_LIST:
       tor_assert(!resource);
       tor_assert(!payload);
       httpcommand = "GET";
-      strlcpy(url, "/tor/running-routers", sizeof(url));
+      url = tor_strdup("/tor/running-routers");
+      break;
+    case DIR_PURPOSE_FETCH_NETWORKSTATUS:
+      httpcommand = "GET";
+      len = strlen(resource)+32;
+      url = tor_malloc(len);
+      tor_snprintf(url, len, "/tor/status/%s", resource);
+      break;
+    case DIR_PURPOSE_FETCH_SERVERDESC:
+      httpcommand = "GET";
+      len = strlen(resource)+32;
+      url = tor_malloc(len);
+      tor_snprintf(url, len, "/tor/server/%s", resource);
       break;
     case DIR_PURPOSE_UPLOAD_DIR:
       tor_assert(!resource);
       tor_assert(payload);
       httpcommand = "POST";
-      strlcpy(url, "/tor/", sizeof(url));
+      url = tor_strdup("/tor/");
       break;
     case DIR_PURPOSE_FETCH_RENDDESC:
       tor_assert(resource);
@@ -433,28 +478,33 @@ directory_send_command(connection_t *conn, const char *platform,
       strlcpy(conn->rend_query, resource, sizeof(conn->rend_query));
 
       httpcommand = "GET";
-      tor_snprintf(url, sizeof(url), "/tor/rendezvous/%s", resource);
-      /* XXXX011 Once directories understand versioned descriptors, switch to this
-       * URL in order to get the most recent version */
-      // tor_snprintf(url, sizeof(url), "/tor/rendezvous1/%s", resource);
-
+      /* Request the most recent versioned descriptor. */
+      // XXXX011
+      //tor_snprintf(url, sizeof(url), "/tor/rendezvous1/%s", resource);
+      len = strlen(resource)+32;
+      url = tor_malloc(len);
+      tor_snprintf(url, len, "/tor/rendezvous/%s", resource);
       break;
     case DIR_PURPOSE_UPLOAD_RENDDESC:
       tor_assert(!resource);
       tor_assert(payload);
       httpcommand = "POST";
-      tor_snprintf(url, sizeof(url), "/tor/rendezvous/publish");
+      url = tor_strdup("/tor/rendezvous/publish");
       break;
+    default:
+      tor_assert(0);
+      return;
   }
+  tor_snprintf(request, sizeof(request), "%s %s", httpcommand, proxystring);
+  connection_write_to_buf(request, strlen(request), conn);
+  connection_write_to_buf(url, strlen(url), conn);
+  tor_free(url);
 
-  tor_snprintf(tmp, sizeof(tmp), "%s %s%s HTTP/1.0\r\nContent-Length: %lu\r\nHost: %s%s\r\n\r\n",
-           httpcommand,
-           proxystring,
-           url,
+  tor_snprintf(request, sizeof(request), " HTTP/1.0\r\nContent-Length: %lu\r\nHost: %s%s\r\n\r\n",
            payload ? (unsigned long)payload_len : 0,
            hoststring,
            proxyauthstring);
-  connection_write_to_buf(tmp, strlen(tmp), conn);
+  connection_write_to_buf(request, strlen(request), conn);
 
   if (payload) {
     /* then send the payload afterwards too */
@@ -510,6 +560,56 @@ parse_http_url(char *headers, char **url)
   return 0;
 }
 
+/** Return a copy of the first HTTP header in <b>headers</b> whose key is
+ * <b>which</b>.  The key should be given with a terminating colon and space;
+ * this function copies everything after, up to but not including the
+ * following \\r\\n. */
+static char *
+http_get_header(const char *headers, const char *which)
+{
+  const char *cp = headers;
+  while (cp) {
+    if (!strcmpstart(cp, which)) {
+      char *eos;
+      cp += strlen(which);
+      if ((eos = strchr(cp,'\r')))
+        return tor_strndup(cp, eos-cp);
+      else
+        return tor_strdup(cp);
+    }
+    cp = strchr(cp, '\n');
+    if (cp)
+      ++cp;
+  }
+  return NULL;
+}
+
+/** Allocate and return a string describing the source of an HTTP request with
+ * headers <b>headers</b> received on <b>conn</b>.  The format is either
+ * "'1.2.3.4'", or "'1.2.3.4' (forwarded for '5.6.7.8')".
+ */
+static char *
+http_get_origin(const char *headers, connection_t *conn)
+{
+  char *fwd;
+
+  fwd = http_get_header(headers, "Forwarded-For: ");
+  if (!fwd)
+    fwd = http_get_header(headers, "X-Forwarded-For: ");
+  if (fwd) {
+    size_t len = strlen(fwd)+strlen(conn->address)+32;
+    char *result = tor_malloc(len);
+    tor_snprintf(result, len, "'%s' (forwarded for '%s')", conn->address, fwd);
+    tor_free(fwd);
+    return result;
+  } else {
+    size_t len = strlen(conn->address)+3;
+    char *result = tor_malloc(len);
+    tor_snprintf(result, len, "'%s'", conn->address);
+    return result;
+  }
+}
+
 /** Parse an HTTP response string <b>headers</b> of the form
  * \verbatim
  * "HTTP/1.\%d \%d\%s\r\n...".
@@ -649,7 +749,7 @@ connection_dir_client_reached_eof(connection_t *conn)
                               &headers, MAX_HEADERS_SIZE,
                               &body, &body_len, MAX_DIR_SIZE)) {
     case -1: /* overflow */
-      log_fn(LOG_WARN,"'fetch' response too large (server '%s'). Failing.", conn->address);
+      log_fn(LOG_WARN,"'fetch' response too large (server '%s:%d'). Closing.", conn->address, conn->port);
       return -1;
     case 0:
       log_fn(LOG_INFO,"'fetch' response not all here, but we're at eof. Closing.");
@@ -659,23 +759,23 @@ connection_dir_client_reached_eof(connection_t *conn)
 
   if (parse_http_response(headers, &status_code, &date_header,
                           &compression, &reason) < 0) {
-    log_fn(LOG_WARN,"Unparseable headers (server '%s'). Closing.", conn->address);
+    log_fn(LOG_WARN,"Unparseable headers (server '%s:%d'). Closing.", conn->address, conn->port);
     tor_free(body); tor_free(headers);
     return -1;
   }
   if (!reason) reason = tor_strdup("[no reason given]");
 
   log_fn(LOG_DEBUG,
-         "Received response from directory server '%s': %d \"%s\"",
-         conn->address, status_code, reason);
+         "Received response from directory server '%s:%d': %d \"%s\"",
+         conn->address, conn->port, status_code, reason);
 
   if (date_header > 0) {
     now = time(NULL);
     delta = now-date_header;
     if (abs(delta)>ALLOW_DIRECTORY_TIME_SKEW) {
       log_fn(router_digest_is_trusted_dir(conn->identity_digest) ? LOG_WARN : LOG_INFO,
-             "Received directory with skewed time (server '%s'): we are %d minutes %s, or the directory is %d minutes %s.",
-             conn->address,
+             "Received directory with skewed time (server '%s:%d'): we are %d minutes %s, or the directory is %d minutes %s.",
+             conn->address, conn->port,
              abs(delta)/60, delta>0 ? "ahead" : "behind",
              abs(delta)/60, delta>0 ? "behind" : "ahead");
       skewed = 1; /* don't check the recommended-versions line */
@@ -709,9 +809,9 @@ connection_dir_client_reached_eof(connection_t *conn)
       else
         description2 = "uncompressed";
 
-      log_fn(LOG_INFO, "HTTP body from server '%s' was labeled %s, "
+      log_fn(LOG_INFO, "HTTP body from server '%s:%d' was labeled %s, "
              "but it seems to be %s.%s",
-             conn->address, description1, description2,
+             conn->address, conn->port, description1, description2,
              (compression>0 && guessed>0)?"  Trying both.":"");
     }
     /* Try declared compression first if we can. */
@@ -724,7 +824,8 @@ connection_dir_client_reached_eof(connection_t *conn)
     /* If we're pretty sure that we have a compressed directory, and
      * we didn't manage to uncompress it, then warn and bail. */
     if (!plausible && !new_body) {
-      log_fn(LOG_WARN, "Unable to decompress HTTP body (server '%s').", conn->address);
+      log_fn(LOG_WARN, "Unable to decompress HTTP body (server '%s:%d').",
+             conn->address, conn->port);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
@@ -737,8 +838,8 @@ connection_dir_client_reached_eof(connection_t *conn)
 
   if (conn->purpose == DIR_PURPOSE_FETCH_DIR) {
     /* fetch/process the directory to learn about new routers. */
-    log_fn(LOG_INFO,"Received directory (size %d) from server '%s'",
-           (int)body_len, conn->address);
+    log_fn(LOG_INFO,"Received directory (size %d) from server '%s:%d'",
+           (int)body_len, conn->address, conn->port);
     if (status_code == 503 || body_len == 0) {
       log_fn(LOG_INFO,"Empty directory; status %d (\"%s\") Ignoring.",
              status_code, reason);
@@ -746,18 +847,17 @@ connection_dir_client_reached_eof(connection_t *conn)
       return 0;
     }
     if (status_code != 200) {
-      log_fn(LOG_WARN,"Received http status code %d (\"%s\") from server '%s'. Failing.",
-             status_code, reason, conn->address);
+      log_fn(LOG_WARN,"Received http status code %d (\"%s\") from server '%s:%d'. I'll try again soon.",
+             status_code, reason, conn->address, conn->port);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
     if (router_load_routerlist_from_directory(body, NULL, !skewed, 0) < 0) {
-      log_fn(LOG_NOTICE,"I failed to parse the directory I fetched from %s:%d. Ignoring.", conn->address, conn->port);
+      log_fn(LOG_NOTICE,"I failed to parse the directory I fetched from '%s:%d'. Ignoring.", conn->address, conn->port);
     } else {
       log_fn(LOG_INFO,"updated routers.");
     }
     /* do things we've been waiting to do */
-    helper_nodes_set_status_from_directory();
     directory_has_arrived(time(NULL), conn->identity_digest);
   }
 
@@ -767,13 +867,14 @@ connection_dir_client_reached_eof(connection_t *conn)
     /* just update our list of running routers, if this list is new info */
     log_fn(LOG_INFO,"Received running-routers list (size %d)", (int)body_len);
     if (status_code != 200) {
-      log_fn(LOG_WARN,"Received http status code %d (\"%s\") from server '%s'. Failing.",
-             status_code, reason, conn->address);
+      log_fn(LOG_WARN,"Received http status code %d (\"%s\") from server '%s:%d'. I'll try again soon.",
+             status_code, reason, conn->address, conn->port);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
     if (!(rrs = router_parse_runningrouters(body, 1))) {
-      log_fn(LOG_WARN, "Can't parse runningrouters list (server '%s')", conn->address);
+      log_fn(LOG_WARN, "Can't parse runningrouters list (server '%s:%d')",
+             conn->address, conn->port);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
@@ -786,19 +887,64 @@ connection_dir_client_reached_eof(connection_t *conn)
     }
   }
 
+  if (conn->purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS) {
+    smartlist_t *which = NULL;
+    char *cp;
+    log_fn(LOG_INFO,"Received networkstatus objects (size %d) from server '%s:%d'",(int) body_len, conn->address, conn->port);
+    if (status_code != 200) {
+      log_fn(LOG_WARN,"Received http status code %d (\"%s\") from server '%s:%d' while fetching \"/tor/status/%s\". I'll try again soon.",
+             status_code, reason, conn->address, conn->port,
+             conn->requested_resource);
+      tor_free(body); tor_free(headers); tor_free(reason);
+      return -1;
+    }
+    if (conn->requested_resource &&
+        !strcmpstart(conn->requested_resource,"fp/")) {
+      int n;
+      which = smartlist_create();
+      smartlist_split_string(which, conn->requested_resource+3, "+", 0, -1);
+      n = smartlist_len(which);
+      if (n && strlen(smartlist_get(which,n-1))==HEX_DIGEST_LEN+2)
+        ((char*)smartlist_get(which,n-1))[HEX_DIGEST_LEN] = '\0';
+    }
+    cp = body;
+    while (*cp) {
+      char *next = strstr(cp, "\nnetwork-status-version");
+      if (next)
+        next[1] = '\0';
+      if (router_set_networkstatus(cp, time(NULL), NS_FROM_DIR, which)<0)
+        break;
+      if (next) {
+        next[1] = 'n';
+        cp = next+1;
+      }
+      else
+        break;
+    }
+    if (which) {
+      SMARTLIST_FOREACH(which, char *, cp, tor_free(cp));
+      smartlist_free(which);
+    }
+  }
+
+  if (conn->purpose == DIR_PURPOSE_FETCH_SERVERDESC) {
+    /* XXXX NM implement this. */
+    log_fn(LOG_WARN, "Somehow, we requested some individual server descriptors. Skipping.");
+  }
+
   if (conn->purpose == DIR_PURPOSE_UPLOAD_DIR) {
     switch (status_code) {
       case 200:
         log_fn(LOG_INFO,"eof (status 200) after uploading server descriptor: finished.");
         break;
       case 400:
-        log_fn(LOG_WARN,"http status 400 (\"%s\") response from dirserver '%s'. Malformed server descriptor?", reason, conn->address);
+        log_fn(LOG_WARN,"http status 400 (\"%s\") response from dirserver '%s:%d'. Malformed server descriptor?", reason, conn->address, conn->port);
         break;
       case 403:
-        log_fn(LOG_WARN,"http status 403 (\"%s\") response from dirserver '%s'. Is your clock skewed? Have you mailed us your key fingerprint? Are you using the right key? Are you using a private IP address? See http://tor.eff.org/doc/tor-doc.html#server.", reason, conn->address);
+        log_fn(LOG_WARN,"http status 403 (\"%s\") response from dirserver '%s:%d'. Is your clock skewed? Have you mailed us your key fingerprint? Are you using the right key? Are you using a private IP address? See http://tor.eff.org/doc/tor-doc-server.html", reason, conn->address, conn->port);
         break;
       default:
-        log_fn(LOG_WARN,"http status %d (\"%s\") reason unexpected (server '%s').", status_code, reason, conn->address);
+        log_fn(LOG_WARN,"http status %d (\"%s\") reason unexpected (server '%s:%d').", status_code, reason, conn->address, conn->port);
         break;
     }
   }
@@ -826,7 +972,7 @@ connection_dir_client_reached_eof(connection_t *conn)
         log_fn(LOG_WARN,"http status 400 (\"%s\"). Dirserver didn't like our rendezvous query?", reason);
         break;
       default:
-        log_fn(LOG_WARN,"http status %d (\"%s\") response unexpected (server '%s').", status_code, reason, conn->address);
+        log_fn(LOG_WARN,"http status %d (\"%s\") response unexpected (server '%s:%d').", status_code, reason, conn->address, conn->port);
         break;
     }
   }
@@ -837,10 +983,10 @@ connection_dir_client_reached_eof(connection_t *conn)
         log_fn(LOG_INFO,"Uploading rendezvous descriptor: finished with status 200 (\"%s\")", reason);
         break;
       case 400:
-        log_fn(LOG_WARN,"http status 400 (\"%s\") response from dirserver '%s'. Malformed rendezvous descriptor?", reason, conn->address);
+        log_fn(LOG_WARN,"http status 400 (\"%s\") response from dirserver '%s:%d'. Malformed rendezvous descriptor?", reason, conn->address, conn->port);
         break;
       default:
-        log_fn(LOG_WARN,"http status %d (\"%s\") response unexpected (server '%s').", status_code, reason, conn->address);
+        log_fn(LOG_WARN,"http status %d (\"%s\") response unexpected (server '%s:%d').", status_code, reason, conn->address, conn->port);
         break;
     }
   }
@@ -944,7 +1090,7 @@ directory_handle_command_get(connection_t *conn, char *headers,
 {
   size_t dlen;
   const char *cp;
-  char *url;
+  char *url = NULL;
   char tmp[8192];
   char date[RFC1123_TIME_LEN+1];
 
@@ -1008,6 +1154,101 @@ directory_handle_command_get(connection_t *conn, char *headers,
     return 0;
   }
 
+  if (!strcmpstart(url,"/tor/status/")) {
+    /* v2 network status fetch. */
+    size_t url_len = strlen(url);
+    int deflated = !strcmp(url+url_len-2, ".z");
+    smartlist_t *dir_objs = smartlist_create();
+    const char *key = url + strlen("/tor/status/");
+    if (deflated)
+      url[url_len-2] = '\0';
+    if (dirserv_get_networkstatus_v2(dir_objs, key)) {
+      smartlist_free(dir_objs);
+      return 0;
+    }
+    tor_free(url);
+    if (!smartlist_len(dir_objs)) { /* we failed to create/cache cp */
+      write_http_status_line(conn, 503, "Network status object unavailable");
+      smartlist_free(dir_objs);
+      return 0;
+    }
+    dlen = 0;
+    SMARTLIST_FOREACH(dir_objs, cached_dir_t *, d,
+                      dlen += deflated?d->dir_z_len:d->dir_len);
+    format_rfc1123_time(date, time(NULL));
+    tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: text/plain\r\nContent-Encoding: %s\r\n\r\n",
+                 date,
+                 (int)dlen,
+                 deflated?"deflate":"identity");
+    connection_write_to_buf(tmp, strlen(tmp), conn);
+    SMARTLIST_FOREACH(dir_objs, cached_dir_t *, d,
+       {
+         if (deflated)
+           connection_write_to_buf(d->dir_z, d->dir_z_len, conn);
+         else
+           connection_write_to_buf(d->dir, d->dir_len, conn);
+       });
+    smartlist_free(dir_objs);
+    return 0;
+  }
+
+  if (!strcmpstart(url,"/tor/server/")) {
+    size_t url_len = strlen(url);
+    int deflated = !strcmp(url+url_len-2, ".z");
+    smartlist_t *descs = smartlist_create();
+    if (deflated)
+      url[url_len-2] = '\0';
+    dirserv_get_routerdescs(descs, url);
+    tor_free(url);
+    if (!smartlist_len(descs)) {
+      write_http_status_line(conn, 400, "Servers unavailable.");
+    } else {
+      size_t len = 0;
+      format_rfc1123_time(date, time(NULL));
+      SMARTLIST_FOREACH(descs, routerinfo_t *, ri,
+                        len += ri->signed_descriptor_len);
+      if (deflated) {
+        size_t compressed_len;
+        char *compressed;
+        char *inp = tor_malloc(len+smartlist_len(descs)+1);
+        char *cp = inp;
+        SMARTLIST_FOREACH(descs, routerinfo_t *, ri,
+           {
+             memcpy(cp, ri->signed_descriptor,
+                    ri->signed_descriptor_len);
+             cp += ri->signed_descriptor_len;
+             *cp++ = '\n';
+           });
+        *cp = '\0';
+        /* XXXX NM This could be way more efficiently handled. */
+        if (tor_gzip_compress(&compressed, &compressed_len,
+                              inp, cp-inp, ZLIB_METHOD)<0) {
+          tor_free(inp);
+          smartlist_free(descs);
+          return -1;
+        }
+        tor_free(inp);
+        tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: application/octet-stream\r\n\r\n",
+                     date,
+                     (int)compressed_len);
+        connection_write_to_buf(tmp, strlen(tmp), conn);
+        connection_write_to_buf(compressed, compressed_len, conn);
+        tor_free(compressed);
+      } else {
+        tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: application/octet-stream\r\n\r\n",
+                     date,
+                     (int)len);
+        connection_write_to_buf(tmp, strlen(tmp), conn);
+        SMARTLIST_FOREACH(descs, routerinfo_t *, ri,
+                          connection_write_to_buf(ri->signed_descriptor,
+                                                  ri->signed_descriptor_len,
+                                                  conn));
+      }
+    }
+    smartlist_free(descs);
+    return 0;
+  }
+
   if (!strcmpstart(url,"/tor/rendezvous/") ||
       !strcmpstart(url,"/tor/rendezvous1/")) {
     /* rendezvous descriptor fetch */
@@ -1062,7 +1303,8 @@ directory_handle_command_post(connection_t *conn, char *headers,
                               char *body, size_t body_len)
 {
   const char *cp;
-  char *url;
+  char *origin = NULL;
+  char *url = NULL;
 
   log_fn(LOG_DEBUG,"Received POST command.");
 
@@ -1080,28 +1322,28 @@ directory_handle_command_post(connection_t *conn, char *headers,
     return 0;
   }
   log_fn(LOG_INFO,"rewritten url as '%s'.", url);
+  origin = http_get_origin(headers, conn);
 
   if (!strcmp(url,"/tor/")) { /* server descriptor post */
     const char *msg;
-    cp = body;
-    switch (dirserv_add_descriptor(&cp, &msg)) {
+    int r = dirserv_add_descriptor(body, &msg);
+    tor_assert(msg);
+    if (r > 0)
+      dirserv_get_directory(&cp, 0); /* rebuild and write to disk */
+    switch (r) {
       case -2:
       case -1:
-        /* malformed descriptor, or something wrong */
-        write_http_status_line(conn, 400, msg?msg:"Malformed or unacceptable server descriptor");
-        log_fn(LOG_NOTICE,"Rejected descriptor published by '%s'.", conn->address);
-        break;
-      case 0:
-        /* descriptor was well-formed but server has not been approved */
-        write_http_status_line(conn, 200, msg?msg:"Unverified server descriptor accepted");
-        break;
       case 1:
-        dirserv_get_directory(&cp, 0); /* rebuild and write to disk */
-        write_http_status_line(conn, 200, msg?msg:"Verified server descriptor accepted");
+        log_fn(LOG_NOTICE,"Rejected descriptor from %s.", origin);
+        /* malformed descriptor, or something wrong */
+        write_http_status_line(conn, 400, msg);
+        break;
+      case 0: /* accepted but discarded */
+      case 2: /* accepted */
+        write_http_status_line(conn, 200, msg);
         break;
     }
-    tor_free(url);
-    return 0;
+    goto done;
   }
 
   if (!strcmpstart(url,"/tor/rendezvous/publish")) {
@@ -1110,14 +1352,17 @@ directory_handle_command_post(connection_t *conn, char *headers,
       write_http_status_line(conn, 400, "Invalid service descriptor rejected");
     else
       write_http_status_line(conn, 200, "Service descriptor stored");
-    tor_free(url);
-    return 0;
+    goto done;
   }
 
   /* we didn't recognize the url */
   write_http_status_line(conn, 404, "Not found");
+
+ done:
   tor_free(url);
+  tor_free(origin);
   return 0;
+
 }
 
 /** Called when a dirserver receives data on a directory connection;
