@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dns.c,v 1.162 2005/07/22 21:12:10 nickm Exp $ */
-const char dns_c_id[] = "$Id: dns.c,v 1.162 2005/07/22 21:12:10 nickm Exp $";
+/* $Id: dns.c,v 1.167 2005/09/09 02:03:31 arma Exp $ */
+const char dns_c_id[] = "$Id: dns.c,v 1.167 2005/09/09 02:03:31 arma Exp $";
 
 /**
  * \file dns.c
@@ -102,8 +102,7 @@ void
 dns_init(void)
 {
   init_cache_tree();
-  last_rotation_time=time(NULL);
-  spawn_enough_dnsworkers();
+  dnsworkers_rotate();
 }
 
 /** Helper: free storage held by an entry in the DNS cache. */
@@ -193,14 +192,21 @@ send_resolved_cell(connection_t *conn, uint8_t answer_type)
     case RESOLVED_TYPE_IPV4:
       buf[1] = 4;
       set_uint32(buf+2, htonl(conn->addr));
-      buflen = 6;
+      set_uint32(buf+6, htonl(MAX_DNS_ENTRY_AGE)); /*XXXX send a real TTL*/
+      buflen = 10;
       break;
     case RESOLVED_TYPE_ERROR_TRANSIENT:
     case RESOLVED_TYPE_ERROR:
-      buf[1] = 24; /* length of "error resolving hostname" */
-      strlcpy(buf+2, "error resolving hostname", sizeof(buf)-2);
-      buflen = 26;
-      break;
+      {
+        const char *errmsg = "Error resolving hostname";
+        int msglen = strlen(errmsg);
+        int ttl = (answer_type == RESOLVED_TYPE_ERROR ? MAX_DNS_ENTRY_AGE : 0);
+        buf[1] = msglen;
+        strlcpy(buf+2, errmsg, sizeof(buf)-2);
+        set_uint32(buf+2+msglen, htonl((uint32_t)ttl));
+        buflen = 6+msglen;
+        break;
+      }
     default:
       tor_assert(0);
     }
@@ -718,14 +724,14 @@ void
 dnsworkers_rotate(void)
 {
   connection_t *dnsconn;
-  log_fn(LOG_INFO, "Rotating DNS workers.");
   while ((dnsconn = connection_get_by_type_state(CONN_TYPE_DNSWORKER,
                                                  DNSWORKER_STATE_IDLE))) {
     connection_mark_for_close(dnsconn);
     num_dnsworkers--;
   }
   last_rotation_time = time(NULL);
-  spawn_enough_dnsworkers();
+  if (server_mode(get_options()))
+    spawn_enough_dnsworkers();
 }
 
 /** Implementation for DNS workers; this code runs in a separate
@@ -904,7 +910,7 @@ spawn_enough_dnsworkers(void)
 
   while (num_dnsworkers < num_dnsworkers_needed) {
     if (spawn_dnsworker() < 0) {
-      log(LOG_WARN,"spawn_enough_dnsworkers(): spawn failed!");
+      log_fn(LOG_WARN,"spawn failed!");
       return;
     }
     num_dnsworkers++;
@@ -913,7 +919,7 @@ spawn_enough_dnsworkers(void)
   while (num_dnsworkers > num_dnsworkers_busy+MAX_IDLE_DNSWORKERS) {
     /* too many idle? */
     /* cull excess workers */
-    log_fn(LOG_NOTICE,"%d of %d dnsworkers are idle. Killing one.",
+    log_fn(LOG_INFO,"%d of %d dnsworkers are idle. Killing one.",
            num_dnsworkers-num_dnsworkers_busy, num_dnsworkers);
     dnsconn = connection_get_by_type_state(CONN_TYPE_DNSWORKER, DNSWORKER_STATE_IDLE);
     tor_assert(dnsconn);
