@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: main.c,v 1.533 2005/07/28 19:01:48 nickm Exp $ */
-const char main_c_id[] = "$Id: main.c,v 1.533 2005/07/28 19:01:48 nickm Exp $";
+/* $Id: main.c,v 1.548 2005/09/08 20:18:14 nickm Exp $ */
+const char main_c_id[] = "$Id: main.c,v 1.548 2005/09/08 20:18:14 nickm Exp $";
 
 /**
  * \file main.c
@@ -46,8 +46,6 @@ long time_of_process_start = 0;
 long stats_n_seconds_working = 0;
 /** When do we next download a directory? */
 static time_t time_to_fetch_directory = 0;
-/** When do we next upload our descriptor? */
-static time_t time_to_force_upload_descriptor = 0;
 /** When do we next download a running-routers summary? */
 static time_t time_to_fetch_running_routers = 0;
 
@@ -94,6 +92,7 @@ static char* nt_strerror(uint32_t errnum);
 #define nt_service_is_stopped() (0)
 #endif
 
+#define FORCE_REGENERATE_DESCRIPTOR_INTERVAL 18*60*60 /* 18 hours */
 #define CHECK_DESCRIPTOR_INTERVAL 60 /* one minute */
 #define BUF_SHRINK_INTERVAL 60 /* one minute */
 #define TIMEOUT_UNTIL_UNREACHABILITY_COMPLAINT (20*60) /* 20 minutes */
@@ -544,12 +543,12 @@ directory_has_arrived(time_t now, char *identity_digest)
   if (!time_to_fetch_directory)
     time_to_fetch_directory = now + get_dir_fetch_period(options);
 
-  if (!time_to_force_upload_descriptor)
-    time_to_force_upload_descriptor = now + options->DirPostPeriod;
-
   if (!time_to_fetch_running_routers)
     time_to_fetch_running_routers = now + get_status_fetch_period(options);
 
+  if (identity_digest) /* if it's fresh */
+    helper_nodes_set_status_from_directory();
+
   if (server_mode(options) && identity_digest) {
     /* if this is us, then our dirport is reachable */
     if (router_digest_is_me(identity_digest))
@@ -558,7 +557,8 @@ directory_has_arrived(time_t now, char *identity_digest)
 
   if (server_mode(options) &&
       !we_are_hibernating()) { /* connect to the appropriate routers */
-    router_retry_connections();
+    if (!authdir_mode(options))
+      router_retry_connections(0);
     if (identity_digest) /* we got a fresh directory */
       consider_testing_reachability();
   }
@@ -692,13 +692,9 @@ run_scheduled_events(time_t now)
     routerlist_remove_old_routers(ROUTER_MAX_AGE);
 
     if (authdir_mode(options)) {
-      /* We're a directory; dump any old descriptors. */
-      dirserv_remove_old_servers(ROUTER_MAX_AGE);
-    }
-    if (server_mode(options) && !we_are_hibernating()) {
-      /* dirservers try to reconnect, in case connections have failed;
-       * and normal servers try to reconnect to dirservers */
-      router_retry_connections();
+      if (!we_are_hibernating()) { /* try to determine reachability */
+        router_retry_connections(1);
+      }
     }
 
     directory_get_from_dirserver(DIR_PURPOSE_FETCH_DIR, NULL, 1);
@@ -708,29 +704,26 @@ run_scheduled_events(time_t now)
       time_to_fetch_running_routers = next_status_fetch;
     }
 
-    /* Also, take this chance to remove old information from rephist. */
+    /* Also, take this chance to remove old information from rephist
+     * and the rend cache. */
     rep_history_clean(now - options->RephistTrackTime);
+    rend_cache_clean();
   }
 
   if (time_to_fetch_running_routers < now) {
-    if (!authdir_mode(options)) {
+    if (!authdir_mode(options) || !options->V1AuthoritativeDir) {
       directory_get_from_dirserver(DIR_PURPOSE_FETCH_RUNNING_LIST, NULL, 1);
     }
     time_to_fetch_running_routers = now + get_status_fetch_period(options);
   }
 
-  if (time_to_force_upload_descriptor < now) {
-    consider_publishable_server(now, 1);
-
-    rend_cache_clean(); /* this should go elsewhere? */
-
-    time_to_force_upload_descriptor = now + options->DirPostPeriod;
-  }
-
   /* 2b. Once per minute, regenerate and upload the descriptor if the old
    * one is inaccurate. */
   if (time_to_check_descriptor < now) {
     time_to_check_descriptor = now + CHECK_DESCRIPTOR_INTERVAL;
+    check_descriptor_bandwidth_changed(now);
+    mark_my_descriptor_dirty_if_older_than(
+                                    now - FORCE_REGENERATE_DESCRIPTOR_INTERVAL);
     consider_publishable_server(now, 0);
     /* also, check religiously for reachability, if it's within the first
      * 20 minutes of our uptime. */
@@ -738,6 +731,13 @@ run_scheduled_events(time_t now)
         stats_n_seconds_working < TIMEOUT_UNTIL_UNREACHABILITY_COMPLAINT &&
         !we_are_hibernating())
       consider_testing_reachability();
+
+    if (server_mode(options) && options->DirPort)
+      update_networkstatus_cache_downloads(now);
+    /* XXXX Disabled until 0.1.1.6 is out: only servers need networkstatus.
+    else
+      update_networkstatus_client_downloads(now);
+    */
   }
 
   /** 3a. Every second, we examine pending circuits and prune the
@@ -854,12 +854,12 @@ second_elapsed_callback(int fd, short event, void *args)
         TIMEOUT_UNTIL_UNREACHABILITY_COMPLAINT) {
     /* every 20 minutes, check and complain if necessary */
     routerinfo_t *me = router_get_my_routerinfo();
-    if (!check_whether_orport_reachable())
-      log(LOG_WARN,"Your server (%s:%d) has not managed to confirm that its ORPort is reachable. Please check your firewalls, ports, address, etc.",
-          me ? me->address : options->Address, options->ORPort);
-    if (!check_whether_dirport_reachable())
-      log(LOG_WARN,"Your server (%s:%d) has not managed to confirm that its DirPort is reachable. Please check your firewalls, ports, address, etc.",
-          me ? me->address : options->Address, options->DirPort);
+    if (me && !check_whether_orport_reachable())
+      log(LOG_WARN,"Your server (%s:%d) has not managed to confirm that its ORPort is reachable. Please check your firewalls, ports, address, /etc/hosts file, etc.",
+          me->address, me->or_port);
+    if (me && !check_whether_dirport_reachable())
+      log(LOG_WARN,"Your server (%s:%d) has not managed to confirm that its DirPort is reachable. Please check your firewalls, ports, address, /etc/hosts file, etc.",
+          me->address, me->dir_port);
   }
 
   /* if more than 100s have elapsed, probably the clock jumped: doesn't count. */
@@ -921,7 +921,7 @@ do_hup(void)
   if (authdir_mode(options)) {
     /* reload the approved-routers file */
     tor_snprintf(keydir,sizeof(keydir),"%s/approved-routers", options->DataDirectory);
-    log_fn(LOG_INFO,"Reloading approved fingerprints from %s...",keydir);
+    log_fn(LOG_INFO,"Reloading approved fingerprints from \"%s\"...",keydir);
     if (dirserv_parse_fingerprint_file(keydir) < 0) {
       log_fn(LOG_NOTICE, "Error reloading fingerprints. Continuing with old list.");
     }
@@ -934,7 +934,7 @@ do_hup(void)
      * configuration options. */
     cpuworkers_rotate();
     dnsworkers_rotate();
-    /* Rebuild fresh descriptor, but leave old one on failure. */
+    /* Write out a fresh descriptor, but leave old one on failure. */
     router_rebuild_descriptor(1);
     descriptor = router_get_my_descriptor();
     if (!descriptor) {
@@ -943,7 +943,7 @@ do_hup(void)
     }
     tor_snprintf(keydir,sizeof(keydir),"%s/router.desc",
                  options->DataDirectory);
-    log_fn(LOG_INFO,"Saving descriptor to %s...",keydir);
+    log_fn(LOG_INFO,"Saving descriptor to \"%s\"...",keydir);
     if (write_str_to_file(keydir, descriptor, 0)) {
       return 0;
     }
@@ -957,10 +957,7 @@ do_main_loop(void)
 {
   int loop_result;
 
-  /* only spawn dns handlers if we're a router */
-  if (server_mode(get_options())) {
-    dns_init(); /* initialize the dns resolve tree, and spawn workers */
-  }
+  dns_init(); /* initialize dns resolve tree, spawn workers if needed */
 
   handle_signals(1);
 
@@ -982,10 +979,13 @@ do_main_loop(void)
   if (router_reload_router_list()) {
     return -1;
   }
+  if (router_reload_networkstatus()) {
+    return -1;
+  }
 
   if (authdir_mode(get_options())) {
     /* the directory is already here, run startup things */
-    router_retry_connections();
+    router_retry_connections(1);
   }
 
   if (server_mode(get_options())) {
@@ -1750,7 +1750,7 @@ nt_service_install(void)
   if (tor_snprintf(command, len, "\"%s\" --nt-service -f \"%s%storrc\"",
                    szPath,  szDrive, szDir)<0) {
     printf("Failed: tor_snprinf()\n");
-    free(command);
+    tor_free(command);
     return 0;
   }
 
