@@ -2,7 +2,7 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: or.h,v 1.636 2005/08/07 21:24:00 nickm Exp $ */
+/* $Id: or.h,v 1.674 2005/09/08 20:36:40 nickm Exp $ */
 
 /**
  * \file or.h
@@ -11,7 +11,7 @@
 
 #ifndef __OR_H
 #define __OR_H
-#define OR_H_ID "$Id: or.h,v 1.636 2005/08/07 21:24:00 nickm Exp $"
+#define OR_H_ID "$Id: or.h,v 1.674 2005/09/08 20:36:40 nickm Exp $"
 
 #include "orconfig.h"
 #ifdef MS_WINDOWS
@@ -323,14 +323,20 @@ typedef enum {
 /** Purpose for connection to directory server: set after a rendezvous
  * descriptor is downloaded. */
 #define DIR_PURPOSE_HAS_FETCHED_RENDDESC 4
+/** A connection to a directory server: download one or more network-status
+ * objects */
+#define DIR_PURPOSE_FETCH_NETWORKSTATUS 5
+/**  A connection to a directory server: download one or more server
+ * descriptors. */
+#define DIR_PURPOSE_FETCH_SERVERDESC 6
 /** Purpose for connection to directory server: upload a server descriptor. */
-#define DIR_PURPOSE_UPLOAD_DIR 5
+#define DIR_PURPOSE_UPLOAD_DIR 7
 /** Purpose for connection to directory server: upload a rendezvous
  * descriptor. */
-#define DIR_PURPOSE_UPLOAD_RENDDESC 6
+#define DIR_PURPOSE_UPLOAD_RENDDESC 8
 /** Purpose for connection at a directory server. */
-#define DIR_PURPOSE_SERVER 7
-#define _DIR_PURPOSE_MAX 7
+#define DIR_PURPOSE_SERVER 9
+#define _DIR_PURPOSE_MAX 9
 
 #define _EXIT_PURPOSE_MIN 1
 #define EXIT_PURPOSE_CONNECT 1
@@ -626,7 +632,7 @@ struct connection_t {
   char identity_digest[DIGEST_LEN]; /**< Hash of identity_pkey */
   char *nickname; /**< Nickname of OR on other side (if any). */
 
-  /** Nickname of planned exit node -- to be used with .exit support. */
+  /** Nickname of planned exit node -- used with .exit support. */
   char *chosen_exit_name;
 
 /* Used only by OR connections: */
@@ -667,7 +673,10 @@ struct connection_t {
   struct circuit_t *on_circuit; /**< The circuit (if any) that this edge
                                  * connection is using. */
 
-  /* Used only by AP connections */
+/* Used only by Dir connections */
+  char *requested_resource; /**< Which 'resource' did we ask the directory for?*/
+
+/* Used only by AP connections */
   socks_request_t *socks_request; /**< SOCKS structure describing request (AP
                                    * only.) */
 
@@ -687,12 +696,14 @@ struct connection_t {
 
 typedef struct connection_t connection_t;
 
-#define ADDR_POLICY_ACCEPT 1
-#define ADDR_POLICY_REJECT 2
+typedef enum {
+  ADDR_POLICY_ACCEPT=1,
+  ADDR_POLICY_REJECT=2,
+} addr_policy_action_t;
 
 /** A linked list of policy rules */
 typedef struct addr_policy_t {
-  char policy_type; /**< One of ADDR_POLICY_ACCEPT or ADDR_POLICY_REJECT. */
+  addr_policy_action_t policy_type; /**< What to do when the policy matches.*/
   char *string; /**< String representation of this rule. */
   uint32_t addr; /**< Base address to accept or reject. */
   uint32_t msk; /**< Accept/reject all addresses <b>a</b> such that
@@ -703,9 +714,21 @@ typedef struct addr_policy_t {
   struct addr_policy_t *next; /**< Next rule in list. */
 } addr_policy_t;
 
+/** A cached_dir_t represents a cacheable directory object, along with its
+ * compressed form. */
+typedef struct cached_dir_t {
+  char *dir; /**< Contents of this object */
+  char *dir_z; /**< Compressed contents of this object. */
+  size_t dir_len; /**< Length of <b>dir</b> */
+  size_t dir_z_len; /**< Length of <b>dir_z</b> */
+  time_t published; /**< When was this object published */
+} cached_dir_t;
+
 /** Information about another onion router in the network. */
 typedef struct {
   char *signed_descriptor; /**< The original signed descriptor for this router*/
+  size_t signed_descriptor_len; /**< The length of signed_descriptor */
+  char signed_descriptor_digest[DIGEST_LEN]; /**< The digest of the signed descriptor. */
 
   char *address; /**< Location of OR: either a hostname or an IP address. */
   char *nickname; /**< Human-readable OR name. */
@@ -732,15 +755,27 @@ typedef struct {
   addr_policy_t *exit_policy; /**< What streams will this OR permit
                                       * to exit? */
   long uptime; /**< How many seconds the router claims to have been up */
+  uint8_t is_hibernating; /**< Whether the router claims to be hibernating */
+  smartlist_t *declared_family; /**< Nicknames of router which this router
+                                 * claims are its family. */
+  char *contact_info; /**< Declared contact info for this router. */
+
   /* local info */
   int is_running; /**< As far as we know, is this OR currently running? */
   time_t status_set_at; /**< When did we last update is_running? */
-  int is_verified; /**< Has a trusted dirserver validated this OR? */
+  int is_verified; /**< Has a trusted dirserver validated this OR?
+                    * (For Authdir: Have we validated this OR?)
+                    */
 
-  smartlist_t *declared_family; /**< Nicknames of router which this router
-                                 * claims are its family. */
-
-  char *contact_info; /**< Declared contact info for this router. */
+  /* The below items are used only by authdirservers for
+   * reachability testing. */
+  /** When was the last time we could reach this OR? */
+  time_t last_reachable;
+  /** When did we start testing reachability for this OR? */
+  time_t testing_since;
+  /** How many times has a descriptor been posted and we believed
+   * this router to be unreachable? We only actually warn on the third. */
+  int num_unreachable_notifications;
 } routerinfo_t;
 
 /** Contents of a running-routers list */
@@ -750,21 +785,64 @@ typedef struct running_routers_t {
   smartlist_t *running_routers;
 } running_routers_t;
 
+/** Contents of a single per-router entry in a network status object.
+ */
+typedef struct routerstatus_t {
+  time_t published_on; /**< When was this router published? */
+  char nickname[MAX_NICKNAME_LEN+1]; /**<The nickname this router says it has. */
+  char identity_digest[DIGEST_LEN]; /**< Digest of the router's identity key. */
+  char descriptor_digest[DIGEST_LEN]; /**< Digest of the router's most recent
+                                       * descriptor. */
+  uint32_t addr; /**< IPv4 address for this router. */
+  uint16_t or_port; /**< OR port for this router. */
+  uint16_t dir_port; /**< Directory port for this router. */
+  unsigned int is_exit:1; /**< True iff this router is a good exit. */
+  unsigned int is_stable:1; /**< True iff this router stays up a long time. */
+  unsigned int is_fast:1; /**< True iff this router has good bandwidth. */
+  unsigned int is_running:1; /**< True iff this router is up. */
+  unsigned int is_named:1; /**< True iff "nickname" belongs to this router. */
+  unsigned int is_valid:1; /**< True iff this router is validated. */
+} routerstatus_t;
+
+/** Contents of a (v2 or later) network status object. */
+typedef struct networkstatus_t {
+  /** When did we receive the network-status document? */
+  time_t received_on;
+  /** What was the digest of the document? */
+  char networkstatus_digest[DIGEST_LEN];
+
+  /* These fields come from the actual network-status document.*/
+  time_t published_on; /**< Declared publication date. */
+
+  char *source_address; /**< Canonical directory server hostname. */
+  uint32_t source_addr; /**< Canonical directory server IP. */
+  uint16_t source_dirport; /**< Canonical directory server dirport. */
+
+  char identity_digest[DIGEST_LEN]; /**< Digest of signing key. */
+  char *contact; /**< How to contact directory admin? (may be NULL). */
+  crypto_pk_env_t *signing_key; /**< Key used to sign this directory. */
+  char *client_versions; /**< comma-separated list of recommended client
+                          * versions. */
+  char *server_versions; /**< comma-separated list of recommended server
+                          * versions. */
+
+  int binds_names:1; /**< True iff this directory server binds names. */
+
+  smartlist_t *entries; /**< List of router_status_t*. */
+} networkstatus_t;
+
 /** Contents of a directory of onion routers. */
 typedef struct {
-  /** List of routerinfo_t */
+  /** List of routerinfo_t. */
   smartlist_t *routers;
   /** Which versions of tor are recommended by this directory? */
   char *software_versions;
   /** When was the most recent directory that contributed to this list
-   * published?
-   */
+   * published? */
   time_t published_on;
   time_t running_routers_updated_on;
   /** What is the most recently received running_routers structure? */
   running_routers_t *running_routers;
-  /** Which router is claimed to have signed it? */
-  char *signing_router;
 } routerlist_t;
 
 /** Information on router used when extending a circuit.  (We don't need a
@@ -1026,7 +1104,7 @@ typedef struct {
   /** What should the tor process actually do? */
   enum {
     CMD_RUN_TOR=0, CMD_LIST_FINGERPRINT, CMD_HASH_PASSWORD,
-    CMD_VERIFY_CONFIG,
+    CMD_VERIFY_CONFIG, CMD_RUN_UNITTESTS
   } command;
   const char *command_arg; /**< Argument for command-line option. */
 
@@ -1074,6 +1152,8 @@ typedef struct {
   /** Directory server only: which versions of
    * Tor should we tell users to run? */
   config_line_t *RecommendedVersions;
+  config_line_t *RecommendedClientVersions;
+  config_line_t *RecommendedServerVersions;
   /** Whether dirservers refuse router descriptors with private IPs. */
   int DirAllowPrivateAddresses;
   char *User; /**< Name of user to run Tor as. */
@@ -1084,7 +1164,9 @@ typedef struct {
   int SocksPort; /**< Port to listen on for SOCKS connections. */
   int ControlPort; /**< Port to listen on for control connections. */
   int DirPort; /**< Port to listen on for directory connections. */
+  int AssumeReachable; /**< Whether to publish our descriptor regardless. */
   int AuthoritativeDir; /**< Boolean: is this an authoritative directory? */
+  int V1AuthoritativeDir; /**< Boolean: is this an authoritative directory? */
   int ClientOnly; /**< Boolean: should we never evolve into a server role? */
   int NoPublish; /**< Boolean: should we never publish a descriptor? */
   int ConnLimit; /**< Requested maximum number of simultaneous connections. */
@@ -1094,7 +1176,9 @@ typedef struct {
   int RunAsDaemon; /**< If true, run in the background. (Unix only) */
   int FascistFirewall; /**< Whether to prefer ORs reachable on open ports. */
   smartlist_t *FirewallPorts; /**< Which ports our firewall allows (strings). */
-  smartlist_t *FirewallIPs; /**< Which IPs our firewall allows (strings). */
+  config_line_t *ReachableAddresses; /**< Which IP:ports our firewall allows
+                                      * (exit policy.) */
+
   /** Application ports that require all nodes in circ to have sufficient uptime. */
   smartlist_t *LongLivedPorts;
   /** Should we try to reuse the same exit node for a given host */
@@ -1147,8 +1231,6 @@ typedef struct {
   config_line_t *RedirectExit; /**< List of config lines for simple
                                        * addr/port redirection */
   smartlist_t *RedirectExitList; /**< List of exit_redirect_t */
-  int _MonthlyAccountingStart; /**< Deprecated: day of month when accounting
-                                * interval starts */
   char *AccountingStart; /**< How long is the accounting interval, and when
                           * does it start? */
   uint64_t AccountingMax; /**< How many bytes do we allow per accounting
@@ -1175,6 +1257,8 @@ typedef struct {
                        * of fixed nodes? */
   int NumHelperNodes; /**< How many helper nodes do we try to establish? */
   int RephistTrackTime; /**< How many seconds do we keep rephist info? */
+
+  addr_policy_t *reachable_addr_policy; /**< Parsed from ReachableAddresses */
 } or_options_t;
 
 /** Persistent state for an onion router, as saved to disk. */
@@ -1348,19 +1432,20 @@ extern unsigned long stats_n_destroy_cells_processed;
 
 or_options_t *get_options(void);
 void set_options(or_options_t *new_val);
-int options_act(void);
 void config_free_all(void);
 const char *safe_str(const char *address);
 
 int config_get_lines(char *string, config_line_t **result);
 void config_free_lines(config_line_t *front);
 int options_trial_assign(config_line_t *list, int reset);
-int resolve_my_address(or_options_t *options, uint32_t *addr);
+int resolve_my_address(or_options_t *options, uint32_t *addr,
+                       char **hostname_out);
 void options_init(or_options_t *options);
 int options_init_from_torrc(int argc, char **argv);
 int options_init_logs(or_options_t *options, int validate_only);
 int config_parse_addr_policy(config_line_t *cfg,
-                             addr_policy_t **dest);
+                             addr_policy_t **dest,
+                             int assume_action);
 void options_append_default_exit_policy(addr_policy_t **policy);
 void addr_policy_free(addr_policy_t *p);
 int option_is_recognized(const char *key);
@@ -1369,6 +1454,7 @@ config_line_t *option_get_assignment(or_options_t *options,
                                      const char *key);
 char *options_dump(or_options_t *options, int minimal);
 int options_save_current(void);
+const char *get_torrc_fname(void);
 
 or_state_t *get_or_state(void);
 int or_state_load(void);
@@ -1376,8 +1462,8 @@ int or_state_save(void);
 
 int config_getinfo_helper(const char *question, char **answer);
 
-int fascist_firewall_allows_address(or_options_t *options, uint32_t addr,
-                                    uint16_t port);
+int firewall_is_fascist(void);
+int fascist_firewall_allows_address(uint32_t addr, uint16_t port);
 
 /********************************* connection.c ***************************/
 
@@ -1462,7 +1548,8 @@ void connection_ap_handshake_socks_reply(connection_t *conn, char *reply,
 void connection_ap_handshake_socks_resolved(connection_t *conn,
                                             int answer_type,
                                             size_t answer_len,
-                                            const char *answer);
+                                            const char *answer,
+                                            int ttl);
 
 int connection_exit_begin_conn(cell_t *cell, circuit_t *circ);
 int connection_exit_begin_resolve(cell_t *cell, circuit_t *circ);
@@ -1482,7 +1569,8 @@ void addressmap_rewrite(char *address, size_t maxlen);
 int addressmap_already_mapped(const char *address);
 void addressmap_register(const char *address, char *new_address, time_t expires);
 int client_dns_incr_failures(const char *address);
-void client_dns_set_addressmap(const char *address, uint32_t val, const char *exitname);
+void client_dns_clear_failures(const char *address);
+void client_dns_set_addressmap(const char *address, uint32_t val, const char *exitname, int ttl);
 const char *addressmap_register_virtual_address(int type, char *new_address);
 void addressmap_get_mappings(smartlist_t *sl, time_t min_expires, time_t max_expires);
 
@@ -1612,11 +1700,12 @@ int dirserv_parse_fingerprint_file(const char *fname);
 int dirserv_router_fingerprint_is_known(const routerinfo_t *router);
 void dirserv_free_fingerprint_list(void);
 const char *dirserv_get_nickname_by_digest(const char *digest);
-int dirserv_add_descriptor(const char **desc, const char **msg);
-int dirserv_load_from_directory_string(const char *dir);
+int dirserv_add_descriptor(const char *desc, const char **msg);
+char *dirserver_getinfo_unregistered(const char *question);
 void dirserv_free_descriptors(void);
+int dirserv_thinks_router_is_blatantly_unreachable(routerinfo_t *router,
+                                                   time_t now);
 int list_server_status(smartlist_t *routers, char **router_status_out);
-void dirserv_remove_old_servers(int age);
 int dirserv_dump_directory_to_string(char **dir_out,
                                      crypto_pk_env_t *private_key);
 void directory_set_dirty(void);
@@ -1624,11 +1713,17 @@ size_t dirserv_get_directory(const char **cp, int compress);
 size_t dirserv_get_runningrouters(const char **rr, int compress);
 void dirserv_set_cached_directory(const char *directory, time_t when,
                                   int is_running_routers);
+void dirserv_set_cached_networkstatus_v2(const char *directory, const char *fp,
+                                         time_t published);
+int dirserv_get_networkstatus_v2(smartlist_t *result, const char *key);
+void dirserv_get_routerdescs(smartlist_t *descs_out, const char *key);
 void dirserv_orconn_tls_done(const char *address,
                              uint16_t or_port,
                              const char *digest_rcvd,
                              const char *nickname,
                              int as_advertised);
+int dirserv_wants_to_reject_router(routerinfo_t *ri, int *verified,
+                                   const char **msg);
 void dirserv_free_all(void);
 
 /********************************* dns.c ***************************/
@@ -1838,6 +1933,7 @@ int rend_cache_store(const char *desc, size_t desc_len);
 
 /********************************* rendservice.c ***************************/
 
+int num_rend_services(void);
 int rend_config_services(or_options_t *options, int validate_only);
 int rend_service_load_keys(void);
 void rend_services_init(void);
@@ -1893,15 +1989,18 @@ int server_mode(or_options_t *options);
 int advertised_server_mode(void);
 int proxy_mode(or_options_t *options);
 
-void router_retry_connections(void);
+void router_retry_connections(int force);
 int router_is_clique_mode(routerinfo_t *router);
 void router_upload_dir_desc_to_dirservers(int force);
+void mark_my_descriptor_dirty_if_older_than(time_t when);
 void mark_my_descriptor_dirty(void);
+void check_descriptor_bandwidth_changed(time_t now);
 int router_compare_to_my_exit_policy(connection_t *conn);
 routerinfo_t *router_get_my_routerinfo(void);
 const char *router_get_my_descriptor(void);
 int router_digest_is_me(const char *digest);
 int router_is_me(routerinfo_t *router);
+int router_fingerprint_is_me(const char *fp);
 int router_rebuild_descriptor(int force);
 int router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
                                  crypto_pk_env_t *ident_key);
@@ -1917,21 +2016,25 @@ typedef struct trusted_dir_server_t {
   uint16_t dir_port;
   char digest[DIGEST_LEN];
   int is_running;
+  int supports_v1_protocol;
 } trusted_dir_server_t;
 
 int router_reload_router_list(void);
+int router_reload_networkstatus(void);
 void router_get_trusted_dir_servers(smartlist_t **outp);
 routerinfo_t *router_pick_directory_server(int requireother,
                                            int fascistfirewall,
-                                           int for_running_routers,
+                                           int for_v2_directory,
                                            int retry_if_no_servers);
-trusted_dir_server_t *router_pick_trusteddirserver(int requireother,
+trusted_dir_server_t *router_pick_trusteddirserver(int need_v1_support,
+                                                   int requireother,
                                                    int fascistfirewall,
                                                    int retry_if_no_servers);
+trusted_dir_server_t *router_get_trusteddirserver_by_digest(
+     const char *digest);
 int all_trusted_directory_servers_down(void);
-struct smartlist_t;
-void routerlist_add_family(struct smartlist_t *sl, routerinfo_t *router);
-void add_nickname_list_to_smartlist(struct smartlist_t *sl, const char *list, int warn_if_down);
+void routerlist_add_family(smartlist_t *sl, routerinfo_t *router);
+void add_nickname_list_to_smartlist(smartlist_t *sl, const char *list, int warn_if_down);
 routerinfo_t *routerlist_find_my_routerinfo(void);
 int exit_policy_implicitly_allows_local_networks(addr_policy_t *policy,
                                                  int warn);
@@ -1942,11 +2045,12 @@ int exit_policy_implicitly_allows_local_networks(addr_policy_t *policy,
 #define ROUTER_REQUIRED_MIN_UPTIME (24*3600) /* a day */
 #define ROUTER_REQUIRED_MIN_BANDWIDTH 10000
 
+routerinfo_t *router_find_exact_exit_enclave(const char *address, uint16_t port);
 int router_is_unreliable(routerinfo_t *router, int need_uptime, int need_capacity);
 routerinfo_t *routerlist_sl_choose_by_bandwidth(smartlist_t *sl);
 routerinfo_t *router_choose_random_node(const char *preferred,
                                         const char *excluded,
-                                        struct smartlist_t *excludedsmartlist,
+                                        smartlist_t *excludedsmartlist,
                                         int need_uptime, int need_bandwidth,
                                         int allow_unverified, int strict);
 routerinfo_t *router_get_by_nickname(const char *nickname);
@@ -1957,14 +2061,21 @@ void router_get_routerlist(routerlist_t **prouterlist);
 time_t routerlist_get_published_time(void);
 void routerlist_free(routerlist_t *routerlist);
 void routerinfo_free(routerinfo_t *router);
+void routerstatus_free(routerstatus_t *routerstatus);
+void networkstatus_free(networkstatus_t *networkstatus);
 void routerlist_free_current(void);
 void free_trusted_dir_servers(void);
 routerinfo_t *routerinfo_copy(const routerinfo_t *router);
 void router_mark_as_down(const char *digest);
 void routerlist_remove_old_routers(int age);
+int router_add_to_routerlist(routerinfo_t *router, const char **msg);
 int router_load_single_router(const char *s, const char **msg);
 int router_load_routerlist_from_directory(const char *s,crypto_pk_env_t *pkey,
                                         int dir_is_recent, int dir_is_cached);
+typedef enum { NS_FROM_CACHE, NS_FROM_DIR, NS_GENERATED} networkstatus_source_t;
+int router_set_networkstatus(const char *s, time_t arrived_at,
+                             networkstatus_source_t source,
+                             smartlist_t *requested_fingerprints);
 addr_policy_result_t router_compare_addr_to_addr_policy(uint32_t addr,
                               uint16_t port, addr_policy_t *policy);
 
@@ -1980,8 +2091,12 @@ int routers_update_status_from_entry(smartlist_t *routers,
 int router_update_status_from_smartlist(routerinfo_t *r,
                                         time_t list_time,
                                         smartlist_t *running_list);
-void add_trusted_dir_server(const char *addr, uint16_t port,const char *digest);
+void add_trusted_dir_server(const char *addr, uint16_t port,
+                            const char *digest, int supports_v1);
 void clear_trusted_dir_servers(void);
+networkstatus_t *networkstatus_get_by_digest(const char *digest);
+void update_networkstatus_cache_downloads(time_t now);
+void update_networkstatus_client_downloads(time_t now);
 
 /********************************* routerparse.c ************************/
 
@@ -2008,6 +2123,9 @@ typedef struct tor_version_t {
 int router_get_router_hash(const char *s, char *digest);
 int router_get_dir_hash(const char *s, char *digest);
 int router_get_runningrouters_hash(const char *s, char *digest);
+int router_get_networkstatus_v2_hash(const char *s, char *digest);
+int router_append_dirobj_signature(char *buf, size_t buf_len, const char *digest,
+                                   crypto_pk_env_t *private_key);
 int router_parse_list_from_string(const char **s,
                                   routerlist_t **dest,
                                   smartlist_t *good_nickname_list,
@@ -2022,13 +2140,15 @@ running_routers_t *router_parse_runningrouters(const char *str,
                                                int write_to_cache);
 routerinfo_t *router_parse_entry_from_string(const char *s, const char *end);
 int router_add_exit_policy_from_string(routerinfo_t *router, const char *s);
-addr_policy_t *router_parse_addr_policy_from_string(const char *s);
-int check_software_version_against_directory(const char *directory,
-                                             int ignoreversion);
+addr_policy_t *router_parse_addr_policy_from_string(const char *s,
+                                                    int assume_action);
+int check_software_version_against_directory(const char *directory);
 int tor_version_parse(const char *s, tor_version_t *out);
 int tor_version_as_new_as(const char *platform, const char *cutoff);
 int tor_version_compare(tor_version_t *a, tor_version_t *b);
 void assert_addr_policy_ok(addr_policy_t *t);
 
+networkstatus_t *networkstatus_parse_from_string(const char *s);
+
 #endif
 
