@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: router.c,v 1.185 2005/08/03 20:42:17 nickm Exp $ */
-const char router_c_id[] = "$Id: router.c,v 1.185 2005/08/03 20:42:17 nickm Exp $";
+/* $Id: router.c,v 1.205 2005/09/08 20:18:15 nickm Exp $ */
+const char router_c_id[] = "$Id: router.c,v 1.205 2005/09/08 20:18:15 nickm Exp $";
 
 #include "or.h"
 
@@ -145,7 +145,7 @@ rotate_onion_key(void)
       goto error;
   }
   if (crypto_pk_write_private_key_to_filename(prkey, fname)) {
-    log(LOG_ERR, "Couldn't write generated key to %s.", fname);
+    log(LOG_ERR, "Couldn't write generated key to \"%s\".", fname);
     goto error;
   }
   log_fn(LOG_INFO, "Rotating onion key");
@@ -175,7 +175,7 @@ init_key_from_file_name_changed(const char *fname_old,
 
   /* The old filename exists, and the new one doesn't.  Rename and load. */
   if (rename(fname_old, fname_new) < 0) {
-    log_fn(LOG_ERR, "Couldn't rename %s to %s: %s", fname_old, fname_new,
+    log_fn(LOG_ERR, "Couldn't rename \"%s\" to \"%s\": %s", fname_old, fname_new,
            strerror(errno));
     return NULL;
   }
@@ -200,10 +200,10 @@ init_key_from_file(const char *fname)
   switch (file_status(fname)) {
     case FN_DIR:
     case FN_ERROR:
-      log(LOG_ERR, "Can't read key from %s", fname);
+      log(LOG_ERR, "Can't read key from \"%s\"", fname);
       goto error;
     case FN_NOENT:
-      log(LOG_INFO, "No key found in %s; generating fresh key.", fname);
+      log(LOG_INFO, "No key found in \"%s\"; generating fresh key.", fname);
       if (crypto_pk_generate_key(prkey)) {
         log(LOG_ERR, "Error generating onion key");
         goto error;
@@ -214,7 +214,7 @@ init_key_from_file(const char *fname)
       }
       log(LOG_INFO, "Generated key seems valid");
       if (crypto_pk_write_private_key_to_filename(prkey, fname)) {
-        log(LOG_ERR, "Couldn't write generated key to %s.", fname);
+        log(LOG_ERR, "Couldn't write generated key to \"%s\".", fname);
         goto error;
       }
       return prkey;
@@ -251,7 +251,7 @@ init_keys(void)
   char fingerprint[FINGERPRINT_LEN+1];
   char fingerprint_line[FINGERPRINT_LEN+MAX_NICKNAME_LEN+3];/*nickname fp\n\0 */
   char *cp;
-  const char *tmp, *mydesc, *datadir;
+  const char *mydesc, *datadir;
   crypto_pk_env_t *prkey;
   char digest[20];
   or_options_t *options = get_options();
@@ -291,14 +291,14 @@ init_keys(void)
   /* 1. Read identity key. Make it if none is found. */
   tor_snprintf(keydir,sizeof(keydir),"%s/keys/identity.key",datadir);
   tor_snprintf(keydir2,sizeof(keydir2),"%s/keys/secret_id_key",datadir);
-  log_fn(LOG_INFO,"Reading/making identity key %s...",keydir2);
+  log_fn(LOG_INFO,"Reading/making identity key \"%s\"...",keydir2);
   prkey = init_key_from_file_name_changed(keydir,keydir2);
   if (!prkey) return -1;
   set_identity_key(prkey);
   /* 2. Read onion key.  Make it if none is found. */
   tor_snprintf(keydir,sizeof(keydir),"%s/keys/onion.key",datadir);
   tor_snprintf(keydir2,sizeof(keydir2),"%s/keys/secret_onion_key",datadir);
-  log_fn(LOG_INFO,"Reading/making onion key %s...",keydir2);
+  log_fn(LOG_INFO,"Reading/making onion key \"%s\"...",keydir2);
   prkey = init_key_from_file_name_changed(keydir,keydir2);
   if (!prkey) return -1;
   set_onion_key(prkey);
@@ -317,7 +317,7 @@ init_keys(void)
   }
   /* 4. Dump router descriptor to 'router.desc' */
   /* Must be called after keys are initialized. */
-  tmp = mydesc = router_get_my_descriptor();
+  mydesc = router_get_my_descriptor();
   if (!mydesc) {
     log_fn(LOG_ERR, "Error initializing descriptor.");
     return -1;
@@ -329,7 +329,7 @@ init_keys(void)
       log_fn(LOG_ERR, "Error adding own fingerprint to approved set");
       return -1;
     }
-    if (dirserv_add_descriptor(&tmp, &m) != 1) {
+    if (dirserv_add_descriptor(mydesc, &m) < 0) {
       log(LOG_ERR, "Unable to add own descriptor to directory: %s",
           m?m:"<unknown error>");
       return -1;
@@ -337,13 +337,13 @@ init_keys(void)
   }
 
   tor_snprintf(keydir,sizeof(keydir),"%s/router.desc", datadir);
-  log_fn(LOG_INFO,"Dumping descriptor to %s...",keydir);
+  log_fn(LOG_INFO,"Dumping descriptor to \"%s\"...",keydir);
   if (write_str_to_file(keydir, mydesc,0)) {
     return -1;
   }
   /* 5. Dump fingerprint to 'fingerprint' */
   tor_snprintf(keydir,sizeof(keydir),"%s/fingerprint", datadir);
-  log_fn(LOG_INFO,"Dumping fingerprint to %s...",keydir);
+  log_fn(LOG_INFO,"Dumping fingerprint to \"%s\"...",keydir);
   if (crypto_pk_get_fingerprint(get_identity_key(), fingerprint, 1)<0) {
     log_fn(LOG_ERR, "Error computing fingerprint");
     return -1;
@@ -360,7 +360,7 @@ init_keys(void)
     return 0;
   /* 6. [authdirserver only] load approved-routers file */
   tor_snprintf(keydir,sizeof(keydir),"%s/approved-routers", datadir);
-  log_fn(LOG_INFO,"Loading approved fingerprints from %s...",keydir);
+  log_fn(LOG_INFO,"Loading approved fingerprints from \"%s\"...",keydir);
   if (dirserv_parse_fingerprint_file(keydir) < 0) {
     log_fn(LOG_ERR, "Error loading fingerprints");
     return -1;
@@ -368,21 +368,8 @@ init_keys(void)
   /* 6b. [authdirserver only] add own key to approved directories. */
   crypto_pk_get_digest(get_identity_key(), digest);
   if (!router_digest_is_trusted_dir(digest)) {
-    add_trusted_dir_server(options->Address, (uint16_t)options->DirPort, digest);
-  }
-  /* 7. [authdirserver only] load old directory, if it's there */
-  tor_snprintf(keydir,sizeof(keydir),"%s/cached-directory", datadir);
-  log_fn(LOG_INFO,"Loading cached directory from %s...",keydir);
-  cp = read_file_to_str(keydir,0);
-  if (!cp) {
-    log_fn(LOG_INFO,"Cached directory %s not present. Ok.",keydir);
-  } else {
-    if (dirserv_load_from_directory_string(cp) < 0) {
-      log_fn(LOG_WARN, "Cached directory %s is corrupt, only loaded part of it.", keydir);
-      tor_free(cp);
-      return 0;
-    }
-    tor_free(cp);
+    add_trusted_dir_server(NULL, (uint16_t)options->DirPort, digest,
+                           options->V1AuthoritativeDir);
   }
   /* success */
   return 0;
@@ -401,14 +388,20 @@ static int can_reach_dir_port = 0;
 int
 check_whether_orport_reachable(void)
 {
-  return clique_mode(get_options()) || can_reach_or_port;
+  or_options_t *options = get_options();
+  return clique_mode(options) ||
+         options->AssumeReachable ||
+         can_reach_or_port;
 }
 
 /** Return 1 if we don't have a dirport configured, or if it's reachable. */
 int
 check_whether_dirport_reachable(void)
 {
-  return !get_options()->DirPort || can_reach_dir_port;
+  or_options_t *options = get_options();
+  return !options->DirPort ||
+         options->AssumeReachable ||
+         can_reach_dir_port;
 }
 
 /**DOCDOC*/
@@ -443,6 +436,7 @@ router_orport_found_reachable(void)
       log(LOG_NOTICE,"Your ORPort is reachable from the outside. Excellent.%s",
           get_options()->NoPublish ? "" : " Publishing server descriptor.");
     can_reach_or_port = 1;
+    mark_my_descriptor_dirty();
     consider_publishable_server(time(NULL), 1);
   }
 }
@@ -554,7 +548,7 @@ consider_publishable_server(time_t now, int force)
 {
   if (decide_if_publishable_server(now)) {
     set_server_advertised(1);
-    if (router_rebuild_descriptor(force) == 0)
+    if (router_rebuild_descriptor(0) == 0)
       router_upload_dir_desc_to_dirservers(force);
   } else {
     set_server_advertised(0);
@@ -567,12 +561,16 @@ consider_publishable_server(time_t now, int force)
 
 /** OR only: if in clique mode, try to open connections to all of the
  * other ORs we know about. Otherwise, open connections to those we
- * think are in clique mode.
+ * think are in clique mode.o
+ *
+ * If <b>force</b> is zero, only open the connection if we don't already
+ * have one.
  */
 void
-router_retry_connections(void)
+router_retry_connections(int force)
 {
   int i;
+  time_t now = time(NULL);
   routerinfo_t *router;
   routerlist_t *rl;
   or_options_t *options = get_options();
@@ -587,10 +585,15 @@ router_retry_connections(void)
       continue;
     if (!clique_mode(options) && !router_is_clique_mode(router))
       continue;
-    if (!connection_get_by_identity_digest(router->identity_digest,
+    if (force ||
+        !connection_get_by_identity_digest(router->identity_digest,
                                            CONN_TYPE_OR)) {
-      /* not in the list */
-      log_fn(LOG_DEBUG,"connecting to OR at %s:%u.",router->address,router->or_port);
+      log_fn(LOG_INFO,"%sconnecting to %s at %s:%u.",
+             clique_mode(options) ? "(forced) " : "",
+             router->nickname, router->address, router->or_port);
+      /* Remember when we started trying to determine reachability */
+      if (!router->testing_since)
+        router->testing_since = now;
       connection_or_connect(router->addr, router->or_port, router->identity_digest);
     }
   }
@@ -612,8 +615,9 @@ router_is_clique_mode(routerinfo_t *router)
 
 /** My routerinfo. */
 static routerinfo_t *desc_routerinfo = NULL;
-/** Boolean: do we need to regenerate the above? */
-static int desc_is_dirty = 1;
+/** Since when has our descriptor been "clean"?  0 if we need to regenerate it
+ * now. */
+static time_t desc_clean_since = 0;
 /** Boolean: do we need to regenerate the above? */
 static int desc_needs_upload = 0;
 
@@ -631,7 +635,7 @@ router_upload_dir_desc_to_dirservers(int force)
     log_fn(LOG_WARN, "No descriptor; skipping upload");
     return;
   }
-  if (!force || !desc_needs_upload)
+  if (!force && !desc_needs_upload)
     return;
   desc_needs_upload = 0;
   directory_post_to_dirservers(DIR_PURPOSE_UPLOAD_DIR, s, strlen(s));
@@ -672,6 +676,18 @@ router_is_me(routerinfo_t *router)
   return router_digest_is_me(router->identity_digest);
 }
 
+/** Return true iff <b>fp</b> is a hex fingerprint of my identity digest. */
+int
+router_fingerprint_is_me(const char *fp)
+{
+  char digest[DIGEST_LEN];
+  if (strlen(fp) == HEX_DIGEST_LEN &&
+      base16_decode(digest, sizeof(digest), fp, HEX_DIGEST_LEN) == 0)
+    return router_digest_is_me(digest);
+
+  return 0;
+}
+
 /** Return a routerinfo for this OR, rebuilding a fresh one if
  * necessary.  Return NULL on error, or if called on an OP. */
 routerinfo_t *
@@ -711,23 +727,19 @@ router_rebuild_descriptor(int force)
   routerinfo_t *ri;
   uint32_t addr;
   char platform[256];
-  struct in_addr in;
   int hibernating = we_are_hibernating();
   or_options_t *options = get_options();
-  char addrbuf[INET_NTOA_BUF_LEN];
 
-  if (!desc_is_dirty && !force)
+  if (desc_clean_since && !force)
     return 0;
 
-  if (resolve_my_address(options, &addr) < 0) {
+  if (resolve_my_address(options, &addr, NULL) < 0) {
     log_fn(LOG_WARN,"options->Address didn't resolve into an IP.");
     return -1;
   }
 
   ri = tor_malloc_zero(sizeof(routerinfo_t));
-  in.s_addr = htonl(addr);
-  tor_inet_ntoa(&in, addrbuf, sizeof(addrbuf));
-  ri->address = tor_strdup(addrbuf);
+  ri->address = tor_dup_addr(addr);
   ri->nickname = tor_strdup(options->Nickname);
   ri->addr = addr;
   ri->or_port = options->ORPort;
@@ -749,11 +761,13 @@ router_rebuild_descriptor(int force)
   if (options->BandwidthRate > options->MaxAdvertisedBandwidth)
     ri->bandwidthrate = (int)options->MaxAdvertisedBandwidth;
 
-  config_parse_addr_policy(get_options()->ExitPolicy, &ri->exit_policy);
+  config_parse_addr_policy(get_options()->ExitPolicy, &ri->exit_policy, -1);
   options_append_default_exit_policy(&ri->exit_policy);
 
   if (desc_routerinfo) /* inherit values */
     ri->is_verified = desc_routerinfo->is_verified;
+  if (authdir_mode(options))
+    ri->is_verified = 1; /* believe in yourself */
   if (options->MyFamily) {
     ri->declared_family = smartlist_create();
     smartlist_split_string(ri->declared_family, options->MyFamily, ",",
@@ -765,21 +779,56 @@ router_rebuild_descriptor(int force)
     log_fn(LOG_WARN, "Couldn't dump router to string.");
     return -1;
   }
+  ri->signed_descriptor_len = strlen(ri->signed_descriptor);
+  crypto_digest(ri->signed_descriptor_digest,
+                ri->signed_descriptor, ri->signed_descriptor_len);
 
   if (desc_routerinfo)
     routerinfo_free(desc_routerinfo);
   desc_routerinfo = ri;
 
-  desc_is_dirty = 0;
+  desc_clean_since = time(NULL);
   desc_needs_upload = 1;
   return 0;
 }
 
+/** Mark descriptor out of date if it's older than <b>when</b> */
+void
+mark_my_descriptor_dirty_if_older_than(time_t when)
+{
+  if (desc_clean_since < when)
+    mark_my_descriptor_dirty();
+}
+
 /** Call when the current descriptor is out of date. */
 void
 mark_my_descriptor_dirty(void)
 {
-  desc_is_dirty = 1;
+  desc_clean_since = 0;
+}
+
+#define MAX_BANDWIDTH_CHANGE_FREQ 20*60
+/** Check whether bandwidth has changed a lot since the last time we announced
+ * bandwidth.  If so, mark our descriptor dirty.*/
+void
+check_descriptor_bandwidth_changed(time_t now)
+{
+  static time_t last_changed = 0;
+  uint64_t prev, cur;
+  if (!desc_routerinfo)
+    return;
+
+  prev = desc_routerinfo->bandwidthcapacity;
+  cur = we_are_hibernating() ? 0 : rep_hist_bandwidth_assess();
+  if ((prev != cur && (!prev || !cur)) ||
+      cur > prev*2 ||
+      cur < prev/2) {
+    if (last_changed+MAX_BANDWIDTH_CHANGE_FREQ < now) {
+      log_fn(LOG_INFO,"Measured bandwidth has changed; rebuilding descriptor.");
+      mark_my_descriptor_dirty();
+      last_changed = now;
+    }
+  }
 }
 
 /** Set <b>platform</b> (max length <b>len</b>) to a NUL-terminated short
@@ -811,9 +860,8 @@ router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
 {
   char *onion_pkey; /* Onion key, PEM-encoded. */
   char *identity_pkey; /* Identity key, PEM-encoded. */
-  char digest[20];
-  char signature[128];
-  char published[32];
+  char digest[DIGEST_LEN];
+  char published[ISO_TIME_LEN+1];
   char fingerprint[FINGERPRINT_LEN+1];
   struct in_addr in;
   char addrbuf[INET_NTOA_BUF_LEN];
@@ -971,18 +1019,11 @@ router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
   if (router_get_router_hash(s, digest) < 0)
     return -1;
 
-  if (crypto_pk_private_sign(ident_key, signature, digest, 20) < 0) {
-    log_fn(LOG_WARN, "Error signing digest");
+  if (router_append_dirobj_signature(s+written,maxlen-written,
+                                     digest,ident_key)<0) {
+    log_fn(LOG_WARN, "Couldn't sign router descriptor");
     return -1;
   }
-  strlcat(s+written, "-----BEGIN SIGNATURE-----\n", maxlen-written);
-  written += strlen(s+written);
-  if (base64_encode(s+written, maxlen-written, signature, 128) < 0) {
-    log_fn(LOG_WARN, "Couldn't base64-encode signature");
-    return -1;
-  }
-  written += strlen(s+written);
-  strlcat(s+written, "-----END SIGNATURE-----\n", maxlen-written);
   written += strlen(s+written);
 
   if (written+2 > maxlen)
