@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerlist.c,v 1.246 2005/08/07 21:24:00 nickm Exp $ */
-const char routerlist_c_id[] = "$Id: routerlist.c,v 1.246 2005/08/07 21:24:00 nickm Exp $";
+/* $Id: routerlist.c,v 1.280 2005/09/08 21:39:39 nickm Exp $ */
+const char routerlist_c_id[] = "$Id: routerlist.c,v 1.280 2005/09/08 21:39:39 nickm Exp $";
 
 /**
  * \file routerlist.c
@@ -21,14 +21,14 @@ static smartlist_t *trusted_dir_servers = NULL;
 /* static function prototypes */
 static routerinfo_t *
 router_pick_directory_server_impl(int requireother, int fascistfirewall,
-                                  int for_runningrouters);
+                                  int for_v2_directory);
 static trusted_dir_server_t *
-router_pick_trusteddirserver_impl(int requireother, int fascistfirewall);
+router_pick_trusteddirserver_impl(int need_v1_support,
+                                  int requireother, int fascistfirewall);
 static void mark_all_trusteddirservers_up(void);
 static int router_nickname_is_in_list(routerinfo_t *router, const char *list);
 static int router_nickname_matches(routerinfo_t *router, const char *nickname);
-static int router_resolve(routerinfo_t *router);
-static int router_resolve_routerlist(routerlist_t *dir);
+static void router_normalize_routerlist(routerlist_t *rl);
 
 /****************************************************************************/
 
@@ -38,11 +38,15 @@ static int router_resolve_routerlist(routerlist_t *dir);
  * descriptors.)
  ****/
 
-/** Global list of all of the routers that we, as an OR or OP, know about. */
+/** Global list of all of the routers that we know about. */
 static routerlist_t *routerlist = NULL;
 
 extern int has_fetched_directory; /**< from main.c */
 
+/** Global list of all of the current network_status documents that we know
+ * about. */
+static smartlist_t *networkstatus_list = NULL;
+
 /**
  * Reload the most recent cached directory (if present).
  */
@@ -75,6 +79,44 @@ router_reload_router_list(void)
   return 0;
 }
 
+/** DOCDOC */
+int
+router_reload_networkstatus(void)
+{
+  char filename[512];
+  struct stat st;
+  smartlist_t *entries, *bad_names;
+  char *s;
+  tor_assert(get_options()->DataDirectory);
+  if (!networkstatus_list)
+    networkstatus_list = smartlist_create();
+
+  tor_snprintf(filename,sizeof(filename),"%s/cached-status",
+               get_options()->DataDirectory);
+  entries = tor_listdir(filename);
+  bad_names = smartlist_create();
+  SMARTLIST_FOREACH(entries, const char *, fn, {
+      char buf[DIGEST_LEN];
+      if (strlen(fn) != HEX_DIGEST_LEN ||
+          base16_decode(buf, sizeof(buf), fn, strlen(fn))) {
+        log_fn(LOG_INFO,
+               "Skipping cached-status file with unexpected name \"%s\"",fn);
+        continue;
+      }
+      tor_snprintf(filename,sizeof(filename),"%s/cached-status/%s",
+                   get_options()->DataDirectory, fn);
+      s = read_file_to_str(filename, 0);
+      if (s) {
+        stat(filename, &st);
+        if (router_set_networkstatus(s, st.st_mtime, NS_FROM_CACHE, NULL)<0) {
+          log_fn(LOG_WARN, "Couldn't load networkstatus from \"%s\"",filename);
+        }
+        tor_free(s);
+      }
+    });
+  return 0;
+}
+
 /* Set *<b>outp</b> to a smartlist containing a list of
  * trusted_dir_server_t * for all known trusted dirservers.  Callers
  * must not modify the list or its contents.
@@ -100,7 +142,7 @@ router_get_trusted_dir_servers(smartlist_t **outp)
 routerinfo_t *
 router_pick_directory_server(int requireother,
                              int fascistfirewall,
-                             int for_runningrouters,
+                             int for_v2_directory,
                              int retry_if_no_servers)
 {
   routerinfo_t *choice;
@@ -109,7 +151,7 @@ router_pick_directory_server(int requireother,
     return NULL;
 
   choice = router_pick_directory_server_impl(requireother, fascistfirewall,
-                                             for_runningrouters);
+                                             for_v2_directory);
   if (choice || !retry_if_no_servers)
     return choice;
 
@@ -118,41 +160,61 @@ router_pick_directory_server(int requireother,
   mark_all_trusteddirservers_up();
   /* try again */
   choice = router_pick_directory_server_impl(requireother, fascistfirewall,
-                                             for_runningrouters);
+                                             for_v2_directory);
   if (choice)
     return choice;
 
   log_fn(LOG_INFO,"Still no %s router entries. Reloading and trying again.",
-         get_options()->FascistFirewall ? "reachable" : "known");
+         firewall_is_fascist() ? "reachable" : "known");
   has_fetched_directory=0; /* reset it */
   if (router_reload_router_list()) {
     return NULL;
   }
   /* give it one last try */
   choice = router_pick_directory_server_impl(requireother, 0,
-                                             for_runningrouters);
+                                             for_v2_directory);
   return choice;
 }
 
+trusted_dir_server_t *
+router_get_trusteddirserver_by_digest(const char *digest)
+{
+  if (!trusted_dir_servers)
+    return NULL;
+
+  SMARTLIST_FOREACH(trusted_dir_servers, trusted_dir_server_t *, ds,
+     {
+       if (!memcmp(ds->digest, digest, DIGEST_LEN))
+         return ds;
+     });
+
+  return NULL;
+}
+
 /** Try to find a running trusted dirserver. If there are no running
  * trusted dirservers and <b>retry_if_no_servers</b> is non-zero,
  * set them all as running again, and try again.
  * Other args are as in router_pick_trusteddirserver_impl().
+ *
+ * DOCDOC need_v1_support
  */
 trusted_dir_server_t *
-router_pick_trusteddirserver(int requireother,
+router_pick_trusteddirserver(int need_v1_support,
+                             int requireother,
                              int fascistfirewall,
                              int retry_if_no_servers)
 {
   trusted_dir_server_t *choice;
 
-  choice = router_pick_trusteddirserver_impl(requireother, fascistfirewall);
+  choice = router_pick_trusteddirserver_impl(need_v1_support,
+                                             requireother, fascistfirewall);
   if (choice || !retry_if_no_servers)
     return choice;
 
   log_fn(LOG_INFO,"No trusted dirservers are reachable. Trying them all again.");
   mark_all_trusteddirservers_up();
-  return router_pick_trusteddirserver_impl(requireother, fascistfirewall);
+  return router_pick_trusteddirserver_impl(need_v1_support,
+                                           requireother, fascistfirewall);
 }
 
 /** Pick a random running verified directory server/mirror from our
@@ -160,13 +222,12 @@ router_pick_trusteddirserver(int requireother,
  * If <b>fascistfirewall</b> and we're not using a proxy,
  * make sure the port we pick is allowed by options-\>firewallports.
  * If <b>requireother</b>, it cannot be us.
- * If <b>for_runningrouters</b>, make sure we pick a dirserver that
- * can answer queries for running-routers (this option will become obsolete
- * once 0.0.9-rc5 is dead).
+ *
+ * DOCDOC need_v1_support, for_v2_directory
  */
 static routerinfo_t *
 router_pick_directory_server_impl(int requireother, int fascistfirewall,
-                                  int for_runningrouters)
+                                  int for_v2_directory)
 {
   int i;
   routerinfo_t *router;
@@ -187,13 +248,12 @@ router_pick_directory_server_impl(int requireother, int fascistfirewall,
     if (requireother && router_is_me(router))
       continue;
     if (fascistfirewall) {
-      if (!fascist_firewall_allows_address(get_options(),router->addr,
-                                           router->dir_port))
+      if (!fascist_firewall_allows_address(router->addr, router->dir_port))
         continue;
     }
-    /* before 0.0.9rc5-cvs, only trusted dirservers served status info. */
-    if (for_runningrouters &&
-        !(tor_version_as_new_as(router->platform,"0.0.9rc5-cvs") ||
+    /* before 0.1.1.6-alpha, only trusted dirservers served status info. */
+    if (for_v2_directory &&
+        !(tor_version_as_new_as(router->platform,"0.1.1.6-alpha") ||
           router_digest_is_trusted_dir(router->identity_digest)))
       continue;
     smartlist_add(sl, router);
@@ -210,7 +270,8 @@ router_pick_directory_server_impl(int requireother, int fascistfirewall,
  * If <b>requireother</b>, it cannot be us.
  */
 static trusted_dir_server_t *
-router_pick_trusteddirserver_impl(int requireother, int fascistfirewall)
+router_pick_trusteddirserver_impl(int need_v1_support,
+                                  int requireother, int fascistfirewall)
 {
   smartlist_t *sl;
   routerinfo_t *me;
@@ -227,11 +288,13 @@ router_pick_trusteddirserver_impl(int requireother, int fascistfirewall)
   SMARTLIST_FOREACH(trusted_dir_servers, trusted_dir_server_t *, d,
     {
       if (!d->is_running) continue;
+      if (need_v1_support && !d->supports_v1_protocol)
+        continue;
       if (requireother && me &&
           !memcmp(me->identity_digest, d->digest, DIGEST_LEN))
         continue;
       if (fascistfirewall) {
-        if (!fascist_firewall_allows_address(get_options(),d->addr,d->dir_port))
+        if (!fascist_firewall_allows_address(d->addr, d->dir_port))
           continue;
       }
       smartlist_add(sl, d);
@@ -434,6 +497,37 @@ routerlist_find_my_routerinfo(void)
   return NULL;
 }
 
+/** Find a router that's up, that has this IP address, and
+ * that allows exit to this address:port, or return NULL if there
+ * isn't a good one.
+ */
+routerinfo_t *
+router_find_exact_exit_enclave(const char *address, uint16_t port) {
+  int i;
+  routerinfo_t *router;
+  uint32_t addr;
+  struct in_addr in;
+
+  if (!tor_inet_aton(address, &in))
+    return NULL; /* it's not an IP already */
+  addr = ntohl(in.s_addr);
+
+  for (i=0;i < smartlist_len(routerlist->routers); i++) {
+    router = smartlist_get(routerlist->routers, i);
+    log_fn(LOG_DEBUG,"Considering %s: %d, %u==%u, %d.",
+           router->nickname,
+           router->is_running,
+           router->addr, addr,
+           router_compare_addr_to_addr_policy(addr, port, router->exit_policy));
+    if (router->is_running &&
+        router->addr == addr &&
+        router_compare_addr_to_addr_policy(addr, port, router->exit_policy) ==
+          ADDR_POLICY_ACCEPTED)
+      return router;
+  }
+  return NULL;
+}
+
 /** Return 1 if <b>router</b> is not suitable for these parameters, else 0.
  * If <b>need_uptime</b> is non-zero, we require a minimum uptime.
  * If <b>need_capacity</b> is non-zero, we require a minimum advertised
@@ -459,8 +553,8 @@ routerlist_sl_remove_unreliable_routers(smartlist_t *sl)
   for (i = 0; i < smartlist_len(sl); ++i) {
     router = smartlist_get(sl, i);
     if (router_is_unreliable(router, 1, 0)) {
-      log(LOG_DEBUG, "Router '%s' has insufficient uptime; deleting.",
-          router->nickname);
+//      log(LOG_DEBUG, "Router '%s' has insufficient uptime; deleting.",
+ //         router->nickname);
       smartlist_del(sl, i--);
     }
   }
@@ -789,6 +883,30 @@ routerlist_free_current(void)
   }
 }
 
+/** Free all storage held by the routerstatus object <b>rs</b>. */
+void
+routerstatus_free(routerstatus_t *rs)
+{
+  tor_free(rs);
+}
+
+/** Free all storage held by the networkstatus object <b>ns</b>. */
+void
+networkstatus_free(networkstatus_t *ns)
+{
+  tor_free(ns->source_address);
+  tor_free(ns->contact);
+  if (ns->signing_key)
+    crypto_free_pk_env(ns->signing_key);
+  tor_free(ns->client_versions);
+  tor_free(ns->server_versions);
+  if (ns->entries) {
+    SMARTLIST_FOREACH(ns->entries, routerstatus_t *, rs, routerstatus_free(rs));
+    smartlist_free(ns->entries);
+  }
+  tor_free(ns);
+}
+
 /** Free all entries in the list of trusted directory servers. */
 void
 free_trusted_dir_servers(void)
@@ -816,7 +934,7 @@ router_mark_as_down(const char *digest)
   if (!router) /* we don't seem to know about him in the first place */
     return;
   log_fn(LOG_DEBUG,"Marking router '%s' as down.",router->nickname);
-  if (router_is_me(router))
+  if (router_is_me(router) && !we_are_hibernating())
     log_fn(LOG_WARN, "We just marked ourself as down. Are your external addresses reachable?");
   router->is_running = 0;
   router->status_set_at = time(NULL);
@@ -825,50 +943,100 @@ router_mark_as_down(const char *digest)
 /** Add <b>router</b> to the routerlist, if we don't already have it.  Replace
  * older entries (if any) with the same name.  Note: Callers should not hold
  * their pointers to <b>router</b> if this function fails; <b>router</b>
- * will either be inserted into the routerlist or freed.  Returns 0 if the
- * router was added; -1 if it was not.
+ * will either be inserted into the routerlist or freed.
  *
- * If we're returning -1 and <b>msg</b> is not NULL, then assign to
- * *<b>msg</b> a static string describing the reason for refusing the
- * routerinfo.
+ * Returns >= 0 if the router was added; less than 0 if it was not.
+ *
+ * If we're returning non-zero, then assign to *<b>msg</b> a static string
+ * describing the reason for not liking the routerinfo.
+ *
+ * If the return value is less than -1, there was a problem with the
+ * routerinfo. If the return value is equal to -1, then the routerinfo was
+ * fine, but out-of-date. If the return value is equal to 1, the
+ * routerinfo was accepted, but we should notify the generator of the
+ * descriptor using the message *<b>msg</b>.
  */
-static int
+int
 router_add_to_routerlist(routerinfo_t *router, const char **msg)
 {
   int i;
-  routerinfo_t *r;
   char id_digest[DIGEST_LEN];
+  int authdir = get_options()->AuthoritativeDir;
+  int authdir_verified = 0;
+
+  tor_assert(msg);
+
+  if (!routerlist) {
+    routerlist = tor_malloc_zero(sizeof(routerlist_t));
+    routerlist->routers = smartlist_create();
+  }
 
-  tor_assert(routerlist);
   crypto_pk_get_digest(router->identity_pkey, id_digest);
 
+  if (authdir) {
+    if (dirserv_wants_to_reject_router(router, &authdir_verified, msg)) {
+      tor_assert(*msg);
+      routerinfo_free(router);
+      return -2;
+    }
+    router->is_verified = authdir_verified;
+    if (tor_version_as_new_as(router->platform,"0.1.0.2-rc"))
+      router->is_verified = 1;
+  }
+
   /* If we have a router with this name, and the identity key is the same,
    * choose the newer one. If the identity key has changed, drop the router.
    */
   for (i = 0; i < smartlist_len(routerlist->routers); ++i) {
-    r = smartlist_get(routerlist->routers, i);
-
-    if (!crypto_pk_cmp_keys(router->identity_pkey, r->identity_pkey)) {
-      if (router->published_on > r->published_on) {
-        log_fn(LOG_DEBUG, "Replacing entry for router '%s/%s' [%s]",
-               router->nickname, r->nickname, hex_str(id_digest,DIGEST_LEN));
-//XXXRD        /* Remember whether we trust this router as a dirserver. */
-        /* If the address hasn't changed; no need to re-resolve. */
-        if (!strcasecmp(r->address, router->address))
-          router->addr = r->addr;
-        routerinfo_free(r);
-        smartlist_set(routerlist->routers, i, router);
-        return 0;
-      } else {
+    routerinfo_t *old_router = smartlist_get(routerlist->routers, i);
+    if (!crypto_pk_cmp_keys(router->identity_pkey,old_router->identity_pkey)) {
+      if (router->published_on <= old_router->published_on) {
         log_fn(LOG_DEBUG, "Skipping not-new descriptor for router '%s'",
                router->nickname);
-        /* Update the is_running status to whatever we were told. */
-        r->is_running = router->is_running;
+        if (authdir) {
+          /* Update the is_verified status based on our lookup. */
+          old_router->is_verified = router->is_verified;
+        } else {
+          /* Update the is_running status to whatever we were told. */
+          old_router->is_running = router->is_running;
+        }
         routerinfo_free(router);
-        if (msg) *msg = "Router descriptor was not new.";
+        *msg = "Router descriptor was not new.";
         return -1;
+      } else {
+        int unreachable = 0;
+        log_fn(LOG_DEBUG, "Replacing entry for router '%s/%s' [%s]",
+               router->nickname, old_router->nickname,
+               hex_str(id_digest,DIGEST_LEN));
+        if (router->addr == old_router->addr &&
+            router->or_port == old_router->or_port) {
+          /* these carry over when the address and orport are unchanged.*/
+          router->last_reachable = old_router->last_reachable;
+          router->testing_since = old_router->testing_since;
+          router->num_unreachable_notifications =
+             old_router->num_unreachable_notifications;
+        }
+        if (authdir &&
+            dirserv_thinks_router_is_blatantly_unreachable(router, time(NULL))) {
+          if (router->num_unreachable_notifications >= 3) {
+            unreachable = 1;
+            log_fn(LOG_WARN, "Notifying server '%s' that it's unreachable. (ContactInfo '%s', platform '%s').",
+              router->nickname, router->contact_info ? router->contact_info : "",
+              router->platform ? router->platform : "");
+          } else {
+            log_fn(LOG_NOTICE,"'%s' may be unreachable -- the %d previous descriptors were thought to be unreachable.", router->nickname, router->num_unreachable_notifications);
+            router->num_unreachable_notifications++;
+          }
+        }
+        routerinfo_free(old_router);
+        smartlist_set(routerlist->routers, i, router);
+        directory_set_dirty();
+        *msg = unreachable ? "Dirserver believes your ORPort is unreachable" :
+               authdir_verified ? "Verified server updated" :
+               "Unverified server updated. (Have you sent us your key fingerprint?)";
+        return unreachable ? 1 : 0;
       }
-    } else if (!strcasecmp(router->nickname, r->nickname)) {
+    } else if (!strcasecmp(router->nickname, old_router->nickname)) {
       /* nicknames match, keys don't. */
       if (router->is_verified) {
         /* The new verified router replaces the old one; remove the
@@ -879,36 +1047,34 @@ router_add_to_routerlist(routerinfo_t *router, const char **msg)
          * make new ones with the new key.
          */
         connection_t *conn;
-        while ((conn = connection_get_by_identity_digest(r->identity_digest,
-                                                         CONN_TYPE_OR))) {
-          log_fn(LOG_INFO,"Closing conn to obsolete router '%s'", r->nickname);
+        while ((conn = connection_get_by_identity_digest(
+                                old_router->identity_digest, CONN_TYPE_OR))) {
+          log_fn(LOG_INFO,"Closing conn to obsolete router '%s'",
+                 old_router->nickname);
           connection_mark_for_close(conn);
         }
-        routerinfo_free(r);
+        routerinfo_free(old_router);
         smartlist_del_keeporder(routerlist->routers, i--);
-      } else if (r->is_verified) {
+      } else if (old_router->is_verified) {
         /* Can't replace a verified router with an unverified one. */
         log_fn(LOG_DEBUG, "Skipping unverified entry for verified router '%s'",
                router->nickname);
         routerinfo_free(router);
-        if (msg) *msg = "Already have verified router with same nickname and different key.";
-        return -1;
+        *msg = "Already have verified router with same nickname and different key.";
+        return -2;
       }
     }
   }
   /* We haven't seen a router with this name before.  Add it to the end of
    * the list. */
   smartlist_add(routerlist->routers, router);
+  directory_set_dirty();
   return 0;
 }
 
 /** Remove any routers from the routerlist that are more than <b>age</b>
  * seconds old.
- *
- * (This function is just like dirserv_remove_old_servers. One day we should
- * merge them.)
  */
-//XXXRD
 void
 routerlist_remove_old_routers(int age)
 {
@@ -930,7 +1096,7 @@ routerlist_remove_old_routers(int age)
   }
 }
 
-/*
+/**
  * Code to parse a single router descriptor and insert it into the
  * routerlist.  Return -1 if the descriptor was ill-formed; 0 if the
  * descriptor was well-formed but could not be added; and 1 if the
@@ -939,6 +1105,8 @@ routerlist_remove_old_routers(int age)
  * If we don't add it and <b>msg</b> is not NULL, then assign to
  * *<b>msg</b> a static string describing the reason for refusing the
  * descriptor.
+ *
+ * This is used only by the controller.
  */
 int
 router_load_single_router(const char *s, const char **msg)
@@ -958,12 +1126,6 @@ router_load_single_router(const char *s, const char **msg)
     routerinfo_free(ri);
     return 0;
   }
-  if (router_resolve(ri)<0) {
-    log_fn(LOG_WARN, "Couldn't resolve router address '%s'; dropping.", ri->address);
-    *msg = "Couldn't resolve router address.";
-    routerinfo_free(ri);
-    return 0;
-  }
   if (routerlist && routerlist->running_routers) {
     running_routers_t *rr = routerlist->running_routers;
     router_update_status_from_smartlist(ri,
@@ -1016,7 +1178,8 @@ router_load_routerlist_from_directory(const char *s,
     smartlist_t *changed = smartlist_create();
     SMARTLIST_FOREACH(new_list->routers, routerinfo_t *, r,
     {
-      if (router_add_to_routerlist(r,NULL)==0)
+      const char *msg;
+      if (router_add_to_routerlist(r,&msg)>=0)
         smartlist_add(changed, r);
     });
     smartlist_clear(new_list->routers);
@@ -1031,79 +1194,247 @@ router_load_routerlist_from_directory(const char *s,
     routerlist = new_list;
     control_event_descriptors_changed(routerlist->routers);
   }
-  if (router_resolve_routerlist(routerlist)) {
-    log_fn(LOG_WARN, "Error resolving routerlist");
+  router_normalize_routerlist(routerlist);
+  return 0;
+}
+
+/** How far in the future do we allow a network-status to get? (seconds) */
+#define NETWORKSTATUS_ALLOW_SKEW (48*60*60)
+/** DOCDOC returns 0 on no problems, -1 on problems.
+ * requested fingerprints must be upcased.
+ */
+int
+router_set_networkstatus(const char *s, time_t arrived_at,
+            networkstatus_source_t source, smartlist_t *requested_fingerprints)
+{
+  networkstatus_t *ns;
+  int i, found;
+  time_t now;
+  char fp[HEX_DIGEST_LEN+1];
+  int skewed = 0;
+
+  ns = networkstatus_parse_from_string(s);
+  if (!ns) {
+    log_fn(LOG_WARN, "Couldn't parse network status.");
     return -1;
   }
-  if (get_options()->AuthoritativeDir) {
-    /* Learn about the descriptors in the directory. */
-    dirserv_load_from_directory_string(s);
-//XXXRD
+  if (!router_digest_is_trusted_dir(ns->identity_digest)) {
+    log_fn(LOG_INFO, "Network status was signed, but not by an authoritative directory we recognize.");
+    networkstatus_free(ns);
+    return -1;
   }
+  now = time(NULL);
+  if (arrived_at > now)
+    arrived_at = now;
+
+  ns->received_on = arrived_at;
+
+  if (ns->published_on > now + NETWORKSTATUS_ALLOW_SKEW) {
+    log_fn(LOG_WARN, "Network status was published in the future (?). Somebody is skewed here: check your clock. Not caching.");
+    skewed = 1;
+  }
+
+  if (!networkstatus_list)
+    networkstatus_list = smartlist_create();
+
+  if (source == NS_FROM_DIR && router_digest_is_me(ns->identity_digest)) {
+    /* Drop our own networkstatus when we get it from somebody else. */
+    networkstatus_free(ns);
+    return 0;
+  }
+
+  base16_encode(fp, HEX_DIGEST_LEN+1, ns->identity_digest, DIGEST_LEN);
+
+  if (requested_fingerprints &&
+      !smartlist_string_isin(requested_fingerprints, fp)) {
+    char *requested = smartlist_join_strings(requested_fingerprints," ",0,NULL);
+    log_fn(LOG_WARN, "We received a network status with a fingerprint (%s) that we never requested. (%s) Dropping.", fp, requested);
+    tor_free(requested);
+    return 0;
+  }
+
+  found = 0;
+  for (i=0; i < smartlist_len(networkstatus_list); ++i) {
+    networkstatus_t *old_ns = smartlist_get(networkstatus_list, i);
+
+    if (!memcmp(old_ns->identity_digest, ns->identity_digest, DIGEST_LEN)) {
+      if (!memcmp(old_ns->networkstatus_digest,
+                  ns->networkstatus_digest, DIGEST_LEN)) {
+        /* Same one we had before. */
+        networkstatus_free(ns);
+        if (old_ns->received_on < arrived_at)
+          old_ns->received_on = arrived_at;
+        return 0;
+      } else if (old_ns->published_on >= ns->published_on) {
+        log_fn(LOG_INFO, "Dropping network-status; we have a newer one for this authority.");
+        networkstatus_free(ns);
+        return 0;
+      } else {
+        networkstatus_free(old_ns);
+        smartlist_set(networkstatus_list, i, ns);
+        found = 1;
+        break;
+      }
+    }
+  }
+
+  if (!found)
+    smartlist_add(networkstatus_list, ns);
+
+  if (source != NS_FROM_CACHE && !skewed) {
+    const char *datadir = get_options()->DataDirectory;
+    size_t len = strlen(datadir)+64;
+    char *fn = tor_malloc(len+1);
+    tor_snprintf(fn, len, "%s/cached-status/%s",datadir,fp);
+    if (write_str_to_file(fn, s, 0)<0) {
+      log_fn(LOG_NOTICE, "Couldn't write cached network status to \"%s\"", fn);
+    }
+    tor_free(fn);
+  }
+
+  if (get_options()->DirPort && !skewed)
+    dirserv_set_cached_networkstatus_v2(s, fp, ns->published_on);
+
   return 0;
 }
 
-/** Helper function: resolve the hostname for <b>router</b>. */
-static int
-router_resolve(routerinfo_t *router)
+/* XXXX These should be configurable, perhaps? NM */
+#define AUTHORITY_NS_CACHE_INTERVAL 10*60
+#define NONAUTHORITY_NS_CACHE_INTERVAL 15*60
+/* DOCDOC*/
+void
+update_networkstatus_cache_downloads(time_t now)
 {
-  if (authdir_mode(get_options())) {
-    /* don't let authdirservers do resolves; this is an easy DoS avenue */
-    struct in_addr iaddr;
-    if (!tor_inet_aton(router->address, &iaddr)) { /* not an IP */
-      log_fn(LOG_WARN,"Refusing to resolve non-IP address '%s' for router '%s'",
-             router->address, router->nickname);
-      return -1;
-    }
-    memcpy((void *)&router->addr, &iaddr.s_addr, 4);
+  static time_t last_downloaded = 0;
+  int authority = authdir_mode(get_options());
+  int interval =
+    authority ? AUTHORITY_NS_CACHE_INTERVAL : NONAUTHORITY_NS_CACHE_INTERVAL;
+
+  if (last_downloaded + interval >= now)
+    return;
+  if (!trusted_dir_servers)
+    return;
+
+  last_downloaded = now;
+
+  if (authority) {
+    /* An authority launches a separate connection for everybody. */
+    SMARTLIST_FOREACH(trusted_dir_servers, trusted_dir_server_t *, ds,
+       {
+         char resource[HEX_DIGEST_LEN+6];
+         if (router_digest_is_me(ds->digest))
+           continue;
+         strlcpy(resource, "fp/", sizeof(resource));
+         base16_encode(resource+3, sizeof(resource)-3, ds->digest, DIGEST_LEN);
+         strlcat(resource, ".z", sizeof(resource));
+         directory_get_from_dirserver(DIR_PURPOSE_FETCH_NETWORKSTATUS,resource,1);
+       });
   } else {
-    if (tor_lookup_hostname(router->address, &router->addr) != 0
-        || !router->addr) {
-      log_fn(LOG_WARN,"Could not resolve address '%s' for router '%s'",
-             router->address, router->nickname);
-      return -1;
-    }
+    /* A non-authority cache launches one connection to a random authority. */
+    directory_get_from_dirserver(DIR_PURPOSE_FETCH_NETWORKSTATUS,"all.z",1);
   }
-  router->addr = ntohl(router->addr); /* get it back into host order */
-  return 0;
 }
 
-/** Helper function: resolve every router in rl, and ensure that our own
- * routerinfo is at the front.
- */
-static int
-router_resolve_routerlist(routerlist_t *rl)
-{
-  int i, remove;
-  routerinfo_t *r;
-  if (!rl)
-    rl = routerlist;
+/*XXXX Should these be configurable? NM*/
+/** How old (in seconds) can a network-status be before we stop believing it? */
+#define NETWORKSTATUS_MAX_VALIDITY (48*60*60)
+/** How long (in seconds) does a client wait after getting a network status
+ * before downloading the next in sequence? */
+#define NETWORKSTATUS_CLIENT_DL_INTERVAL (30*60)
 
-  i = 0;
+/** DOCDOC */
+void
+update_networkstatus_client_downloads(time_t now)
+{
+  int n_live = 0, needed = 0, n_dirservers, i;
+  int most_recent_idx = -1;
+  trusted_dir_server_t *most_recent = NULL;
+  time_t most_recent_received = 0;
+  char *resource, *cp;
+  size_t resource_len;
+
+  /* This is a little tricky.  We want to download enough network-status
+   * objects so that we have at least half of them under NETWORKSTATUS_MAX_VALIDITY
+   * publication time.  We want to download a new *one* if the most recent
+   * one's publication time is under NETWORKSTATUS_CLIENT_DL_INTERVAL.
+   */
+  if (!trusted_dir_servers || !smartlist_len(trusted_dir_servers))
+    return;
+  n_dirservers = smartlist_len(trusted_dir_servers);
+  SMARTLIST_FOREACH(trusted_dir_servers, trusted_dir_server_t *, ds,
+     {
+       networkstatus_t *ns = networkstatus_get_by_digest(ds->digest);
+       if (!ns)
+         continue;
+       if (ns->published_on > now-NETWORKSTATUS_MAX_VALIDITY)
+         ++n_live;
+       if (!most_recent || ns->received_on > most_recent_received) {
+         most_recent_idx = ds_sl_idx; /* magic variable from FOREACH*/
+         most_recent = ds;
+         most_recent_received = ns->received_on;
+       }
+     });
+
+  /* Download enough so we have at least half live, but no more than all the
+   * trusted dirservers we know.
+   */
+  if (n_live < (n_dirservers/2)+1)
+    needed = (n_dirservers/2)+1-n_live;
+  if (needed > n_dirservers)
+    needed = n_dirservers;
+  /* Also, download at least 1 every NETWORKSTATUS_CLIENT_DL_INTERVAL. */
+  if (most_recent_received < now-NETWORKSTATUS_CLIENT_DL_INTERVAL && needed < 1)
+    needed = 1;
+
+  if (!needed)
+    return;
+
+  /* If no networkstatus was found, choose a dirserver at random as "most
+   * recent". */
+  if (most_recent_idx<0)
+    most_recent_idx = crypto_pseudo_rand_int(n_dirservers);
+
+  /* Build a request string for all the resources we want. */
+  resource_len = needed * (HEX_DIGEST_LEN+1) + 6;
+  resource = tor_malloc(resource_len);
+  memcpy(resource, "fp/", 3);
+  cp = resource+3;
+  for (i = most_recent_idx+1; needed; ++i) {
+    trusted_dir_server_t *ds;
+    if (i >= n_dirservers)
+      i = 0;
+    ds = smartlist_get(trusted_dir_servers, i);
+    base16_encode(cp, HEX_DIGEST_LEN+1, ds->digest, DIGEST_LEN);
+    cp += HEX_DIGEST_LEN;
+    --needed;
+    if (needed)
+      *cp++ = '+';
+  }
+  memcpy(cp, ".z", 3);
+  directory_get_from_dirserver(DIR_PURPOSE_FETCH_NETWORKSTATUS, resource, 1);
+  tor_free(resource);
+}
+
+/** Ensure that our own routerinfo is at the front, and remove duplicates
+ * of our routerinfo.
+ */
+static void
+router_normalize_routerlist(routerlist_t *rl)
+{
+  int i=0;
+  routerinfo_t *r;
   if ((r = router_get_my_routerinfo())) {
     smartlist_insert(rl->routers, 0, routerinfo_copy(r));
     ++i;
   }
 
   for ( ; i < smartlist_len(rl->routers); ++i) {
-    remove = 0;
     r = smartlist_get(rl->routers,i);
     if (router_is_me(r)) {
-      remove = 1;
-    } else if (r->addr) {
-      /* already resolved. */
-    } else if (router_resolve(r)) {
-      log_fn(LOG_WARN, "Couldn't resolve address '%s' for router '%s'; not using",
-             r->address, r->nickname);
-      remove = 1;
-    }
-    if (remove) {
       routerinfo_free(r);
       smartlist_del_keeporder(rl->routers, i--);
     }
   }
-
-  return 0;
 }
 
 /** Decide whether a given addr:port is definitely accepted,
@@ -1374,6 +1705,7 @@ routers_update_status_from_entry(smartlist_t *routers,
                                  time_t list_time,
                                  const char *s)
 {
+  int authdir = get_options()->AuthoritativeDir;
   int is_running = 1;
   int is_verified = 0;
   int hex_digest_set = 0;
@@ -1445,13 +1777,22 @@ routers_update_status_from_entry(smartlist_t *routers,
   {
     int nickname_matches = is_verified && !strcasecmp(r->nickname, nickname);
     int digest_matches = !memcmp(digest, r->identity_digest, DIGEST_LEN);
-    if (nickname_matches && digest_matches)
-      r->is_verified = 1;
-    else if (digest_matches)
-      r->is_verified = 0;
+    if (!authdir) {
+      /* If we're not an authoritative directory, update verified status.
+       */
+      if (nickname_matches && digest_matches)
+        r->is_verified = 1;
+      else if (digest_matches)
+        r->is_verified = 0;
+    }
     if (digest_matches)
       if (r->status_set_at < list_time) {
-        r->is_running = is_running;
+        if (!authdir || is_running)
+          /* If we're an authoritative directory, only believe that servers
+           * are down when we hear it ourselves.  Otherwise, believe
+           * what we're told.
+           */
+          r->is_running = is_running;
         r->status_set_at = time(NULL);
       }
   });
@@ -1476,26 +1817,39 @@ router_update_status_from_smartlist(routerinfo_t *router,
 }
 
 /** Add to the list of authorized directory servers one at
- * <b>address</b>:<b>port</b>, with identity key <b>digest</b>. */
+ * <b>address</b>:<b>port</b>, with identity key <b>digest</b>.  If
+ * <b>address</b> is NULL, add ourself. */
 void
-add_trusted_dir_server(const char *address, uint16_t port, const char *digest)
+add_trusted_dir_server(const char *address, uint16_t port, const char *digest,
+                       int supports_v1)
 {
   trusted_dir_server_t *ent;
   uint32_t a;
+  char *hostname = NULL;
   if (!trusted_dir_servers)
     trusted_dir_servers = smartlist_create();
 
-  if (tor_lookup_hostname(address, &a)) {
-    log_fn(LOG_WARN, "Unable to lookup address for directory server at %s",
-           address);
-    return;
+  if (!address) { /* The address is us; we should guess. */
+    if (resolve_my_address(get_options(), &a, &hostname) < 0) {
+      log_fn(LOG_WARN, "Couldn't find a suitable address. Returning.");
+      return;
+    }
+  } else {
+    if (tor_lookup_hostname(address, &a)) {
+      log_fn(LOG_WARN, "Unable to lookup address for directory server at %s",
+             address);
+      return;
+    }
+    hostname = tor_strdup(address);
+    a = ntohl(a);
   }
 
   ent = tor_malloc(sizeof(trusted_dir_server_t));
-  ent->address = tor_strdup(address);
-  ent->addr = ntohl(a);
+  ent->address = hostname;
+  ent->addr = a;
   ent->dir_port = port;
   ent->is_running = 1;
+  ent->supports_v1_protocol = supports_v1;
   memcpy(ent->digest, digest, DIGEST_LEN);
   smartlist_add(trusted_dir_servers, ent);
 }
@@ -1513,3 +1867,14 @@ clear_trusted_dir_servers(void)
   }
 }
 
+networkstatus_t *
+networkstatus_get_by_digest(const char *digest)
+{
+  SMARTLIST_FOREACH(networkstatus_list, networkstatus_t *, ns,
+    {
+      if (!memcmp(ns->identity_digest, digest, DIGEST_LEN))
+        return ns;
+    });
+  return NULL;
+}
+
