@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerparse.c,v 1.115 2005/06/24 23:42:15 nickm Exp $ */
-const char routerparse_c_id[] = "$Id: routerparse.c,v 1.115 2005/06/24 23:42:15 nickm Exp $";
+/* $Id: routerparse.c,v 1.133 2005/09/08 21:01:24 nickm Exp $ */
+const char routerparse_c_id[] = "$Id: routerparse.c,v 1.133 2005/09/08 21:01:24 nickm Exp $";
 
 /**
  * \file routerparse.c
@@ -46,6 +46,13 @@ typedef enum {
   K_HIBERNATING,
   K_READ_HISTORY,
   K_WRITE_HISTORY,
+  K_NETWORK_STATUS_VERSION,
+  K_DIR_SOURCE,
+  K_DIR_OPTIONS,
+  K_CLIENT_VERSIONS,
+  K_SERVER_VERSIONS,
+  K_R,
+  K_S,
   _UNRECOGNIZED,
   _ERR,
   _EOF,
@@ -91,40 +98,53 @@ typedef enum {
 
 /** Rules for where a keyword can appear. */
 typedef enum {
-  ANY = 0,    /**< Appears in router descriptor or in directory sections. */
-  DIR_ONLY,   /**< Appears only in directory. */
-  RTR_ONLY,   /**< Appears only in router descriptor or runningrouters */
+  DIR = 1,   /**< Appears only in directory. */
+  RTR = 2,   /**< Appears only in router descriptor or runningrouters */
+  NETSTATUS = 4,  /**< v2 or later ("versioned") network status. */
+  RTRSTATUS = 8,
+  ANY = 15,    /**< Appears in router descriptor or in directory sections. */
 } where_syntax;
 
 /** Table mapping keywords to token value and to argument rules. */
 static struct {
-  const char *t; int v; arg_syntax s; obj_syntax os; where_syntax ws;
+  const char *t; int v; arg_syntax s; obj_syntax os; int ws;
 } token_table[] = {
-  { "accept",              K_ACCEPT,              ARGS,    NO_OBJ,  RTR_ONLY },
-  { "directory-signature", K_DIRECTORY_SIGNATURE, ARGS,    NEED_OBJ,DIR_ONLY},
-  { "reject",              K_REJECT,              ARGS,    NO_OBJ,  RTR_ONLY },
-  { "router",              K_ROUTER,              ARGS,    NO_OBJ,  RTR_ONLY },
-  { "recommended-software",K_RECOMMENDED_SOFTWARE,ARGS,    NO_OBJ,  DIR_ONLY },
-  { "signed-directory",    K_SIGNED_DIRECTORY,    NO_ARGS, NO_OBJ,  DIR_ONLY },
-  { "signing-key",         K_SIGNING_KEY,         NO_ARGS, NEED_KEY,RTR_ONLY },
-  { "onion-key",           K_ONION_KEY,           NO_ARGS, NEED_KEY,RTR_ONLY },
-  { "router-signature",    K_ROUTER_SIGNATURE,    NO_ARGS, NEED_OBJ,RTR_ONLY },
-  { "running-routers",     K_RUNNING_ROUTERS,     ARGS,    NO_OBJ,  DIR_ONLY },
-  { "router-status",       K_ROUTER_STATUS,       ARGS,    NO_OBJ,  DIR_ONLY },
-  { "ports",               K_PORTS,               ARGS,    NO_OBJ,  RTR_ONLY },
-  { "bandwidth",           K_BANDWIDTH,           ARGS,    NO_OBJ,  RTR_ONLY },
-  { "platform",            K_PLATFORM,        CONCAT_ARGS, NO_OBJ,  RTR_ONLY },
+  { "accept",              K_ACCEPT,              ARGS,    NO_OBJ,  RTR },
+  { "directory-signature", K_DIRECTORY_SIGNATURE, ARGS,    NEED_OBJ,
+    DIR|NETSTATUS},
+  { "r",                   K_R,                   ARGS,    NO_OBJ,  RTRSTATUS },
+  { "s",                   K_S,                   ARGS,    NO_OBJ,  RTRSTATUS },
+  { "reject",              K_REJECT,              ARGS,    NO_OBJ,  RTR },
+  { "router",              K_ROUTER,              ARGS,    NO_OBJ,  RTR },
+  { "recommended-software",K_RECOMMENDED_SOFTWARE,ARGS,    NO_OBJ,  DIR },
+  { "signed-directory",    K_SIGNED_DIRECTORY,    NO_ARGS, NO_OBJ,  DIR },
+  { "signing-key",         K_SIGNING_KEY,         NO_ARGS, NEED_KEY,RTR },
+  { "onion-key",           K_ONION_KEY,           NO_ARGS, NEED_KEY,RTR },
+  { "router-signature",    K_ROUTER_SIGNATURE,    NO_ARGS, NEED_OBJ,RTR },
+  { "running-routers",     K_RUNNING_ROUTERS,     ARGS,    NO_OBJ,  DIR },
+  { "router-status",       K_ROUTER_STATUS,       ARGS,    NO_OBJ,  DIR },
+  { "ports",               K_PORTS,               ARGS,    NO_OBJ,  RTR },
+  { "bandwidth",           K_BANDWIDTH,           ARGS,    NO_OBJ,  RTR },
+  { "platform",            K_PLATFORM,        CONCAT_ARGS, NO_OBJ,  RTR },
   { "published",           K_PUBLISHED,       CONCAT_ARGS, NO_OBJ,  ANY },
   { "opt",                 K_OPT,             CONCAT_ARGS, OBJ_OK,  ANY },
   { "contact",             K_CONTACT,         CONCAT_ARGS, NO_OBJ,  ANY },
-  { "network-status",      K_NETWORK_STATUS,      NO_ARGS, NO_OBJ,  DIR_ONLY },
-  { "uptime",              K_UPTIME,              ARGS,    NO_OBJ,  RTR_ONLY },
-  { "dir-signing-key",     K_DIR_SIGNING_KEY,     ARGS,    OBJ_OK,  DIR_ONLY },
-  { "family",              K_FAMILY,              ARGS,    NO_OBJ,  RTR_ONLY },
+  { "network-status",      K_NETWORK_STATUS,      NO_ARGS, NO_OBJ,  DIR },
+  { "uptime",              K_UPTIME,              ARGS,    NO_OBJ,  RTR },
+  { "dir-signing-key",     K_DIR_SIGNING_KEY,     ARGS,    OBJ_OK, DIR|NETSTATUS},
+  { "family",              K_FAMILY,              ARGS,    NO_OBJ,  RTR },
   { "fingerprint",         K_FINGERPRINT,         ARGS,    NO_OBJ,  ANY },
-  { "hibernating",         K_HIBERNATING,         ARGS,    NO_OBJ,  RTR_ONLY },
-  { "read-history",        K_READ_HISTORY,        ARGS,    NO_OBJ,  RTR_ONLY },
-  { "write-history",       K_WRITE_HISTORY,       ARGS,    NO_OBJ,  RTR_ONLY },
+  /* XXXX NM obsolete; remove once tor26 upgrades. */
+  { "dir-fingerprint",     K_FINGERPRINT,         ARGS,    NO_OBJ,  NETSTATUS },
+  { "hibernating",         K_HIBERNATING,         ARGS,    NO_OBJ,  RTR },
+  { "read-history",        K_READ_HISTORY,        ARGS,    NO_OBJ,  RTR },
+  { "write-history",       K_WRITE_HISTORY,       ARGS,    NO_OBJ,  RTR },
+  { "network-status-version", K_NETWORK_STATUS_VERSION,
+                                                  ARGS,    NO_OBJ, NETSTATUS },
+  { "dir-source",          K_DIR_SOURCE,          ARGS,    NO_OBJ, NETSTATUS },
+  { "dir-options",         K_DIR_OPTIONS,         ARGS,    NO_OBJ, NETSTATUS },
+  { "client-versions",     K_CLIENT_VERSIONS,     ARGS,    NO_OBJ, NETSTATUS },
+  { "server-versions",     K_SERVER_VERSIONS,     ARGS,    NO_OBJ, NETSTATUS },
   { NULL, -1, NO_ARGS, NO_OBJ, ANY }
 };
 
@@ -138,12 +158,13 @@ static smartlist_t *find_all_exitpolicy(smartlist_t *s);
 static directory_token_t *find_first_by_keyword(smartlist_t *s,
                                                 directory_keyword keyword);
 static int tokenize_string(const char *start, const char *end,
-                           smartlist_t *out, int is_dir);
+                           smartlist_t *out, where_syntax where);
 static directory_token_t *get_next_token(const char **s, where_syntax where);
 static int check_directory_signature(const char *digest,
                                      directory_token_t *tok,
                                      crypto_pk_env_t *pkey,
-                                     crypto_pk_env_t *declared_key);
+                                     crypto_pk_env_t *declared_key,
+                                     int check_authority);
 static crypto_pk_env_t *find_dir_signing_key(const char *str);
 /* static */ int is_obsolete_version(const char *myversion,
                                      const char *versionlist);
@@ -179,6 +200,52 @@ router_get_runningrouters_hash(const char *s, char *digest)
                               "network-status","\ndirectory-signature");
 }
 
+/** DOCDOC */
+int
+router_get_networkstatus_v2_hash(const char *s, char *digest)
+{
+  return router_get_hash_impl(s,digest,
+                              "network-status-version","\ndirectory-signature");
+}
+
+/** Helper: used to generate signatures for routers, directories and
+ * network-status objects.  Given a digest in <b>digest</b> and a secret
+ * <b>private_key</b>, generate an PKCS1-padded signature, BASE64-encode it,
+ * surround it with -----BEGIN/END----- pairs, and write it to the
+ * <b>buf_len</b>-byte buffer at <b>buf</b>.  Return 0 on success, -1 on
+ * failure.
+ */
+int
+router_append_dirobj_signature(char *buf, size_t buf_len, const char *digest,
+                               crypto_pk_env_t *private_key)
+{
+  char signature[PK_BYTES];
+  int i;
+
+  if (crypto_pk_private_sign(private_key, signature, digest, DIGEST_LEN) < 0) {
+
+    log_fn(LOG_WARN,"Couldn't sign digest.");
+    return -1;
+  }
+  if (strlcat(buf, "-----BEGIN SIGNATURE-----\n", buf_len) >= buf_len)
+    goto truncated;
+
+  i = strlen(buf);
+  if (base64_encode(buf+i, buf_len-i, signature, 128) < 0) {
+    log_fn(LOG_WARN,"couldn't base64-encode signature");
+    tor_free(buf);
+    return -1;
+  }
+
+  if (strlcat(buf, "-----END SIGNATURE-----\n", buf_len) >= buf_len)
+    goto truncated;
+
+  return 0;
+ truncated:
+  log_fn(LOG_WARN,"tried to exceed string length.");
+  return -1;
+}
+
 /**
  * Find the first instance of "recommended-software ...\n" at the start of
  * a line; return a newly allocated string containing the "..." portion.
@@ -226,7 +293,6 @@ get_recommended_software_from_directory(const char *str)
     r, ret, same;
   static int warned_too_new=0;
   smartlist_t *version_sl;
-  int XXXpath;
 
   vl = versionlist;
 
@@ -273,18 +339,15 @@ get_recommended_software_from_directory(const char *str)
         warned_too_new = 1;
       }
       ret = 0;
-      XXXpath = 1;
     } else {
       /* We found a newer one in the same series; we're obsolete. */
       ret = 1;
-      XXXpath = 2;
     }
   } else {
     if (found_newer) {
       /* We belong to a series with no recommended members, and
        * a newer series is recommended. We're obsolete. */
       ret = 1;
-      XXXpath = 3;
     } else {
       /* We belong to a series with no recommended members, and it's
        * newer than any recommended series. We're probably okay. */
@@ -294,7 +357,6 @@ get_recommended_software_from_directory(const char *str)
         warned_too_new = 1;
       }
       ret = 0;
-      XXXpath = 4;
     }
   }
   /*
@@ -313,8 +375,7 @@ get_recommended_software_from_directory(const char *str)
 /* Return 0 if myversion is supported; else log a message and return
  * -1 (or exit if ignoreversions is false) */
 int
-check_software_version_against_directory(const char *directory,
-                                         int ignoreversion)
+check_software_version_against_directory(const char *directory)
 {
   char *v;
   v = get_recommended_software_from_directory(directory);
@@ -326,21 +387,12 @@ check_software_version_against_directory(const char *directory,
     tor_free(v);
     return 0;
   }
-  log(ignoreversion ? LOG_WARN : LOG_ERR,
+  log(LOG_WARN,
      "You are running Tor version %s, which will not work with this network.\n"
      "Please use %s%s.",
       VERSION, strchr(v,',') ? "one of " : "", v);
   tor_free(v);
-
-  if (ignoreversion) {
-    log(LOG_WARN, "IgnoreVersion is set. If it breaks, we told you so.");
-    return -1;
-  } else {
-    fflush(0);
-    tor_cleanup();
-    exit(0);
-    return -1; /* never reached */
-  }
+  return 0;
 }
 
 /** Parse a directory from <b>str</b> and, when done, store the
@@ -390,7 +442,7 @@ router_parse_routerlist_from_directory(const char *str,
   }
   ++cp;
   tokens = smartlist_create();
-  if (tokenize_string(cp,strchr(cp,'\0'),tokens,1)) {
+  if (tokenize_string(cp,strchr(cp,'\0'),tokens,DIR)) {
     log_fn(LOG_WARN, "Error tokenizing directory signature"); goto err;
   }
   if (smartlist_len(tokens) != 1) {
@@ -401,7 +453,7 @@ router_parse_routerlist_from_directory(const char *str,
     log_fn(LOG_WARN,"Expected a single directory signature"); goto err;
   }
   declared_key = find_dir_signing_key(str);
-  if (check_directory_signature(digest, tok, pkey, declared_key)<0)
+  if (check_directory_signature(digest, tok, pkey, declared_key, 1)<0)
     goto err;
 
   /* now we know tok->n_args == 1, so it's safe to access tok->args[0] */
@@ -417,7 +469,7 @@ router_parse_routerlist_from_directory(const char *str,
 
   /* Now that we know the signature is okay, check the version. */
   if (check_version)
-    check_software_version_against_directory(str, get_options()->IgnoreVersion);
+    check_software_version_against_directory(str);
 
   /* Now try to parse the first part of the directory. */
   if ((end = strstr(str,"\nrouter "))) {
@@ -429,7 +481,7 @@ router_parse_routerlist_from_directory(const char *str,
   }
 
   tokens = smartlist_create();
-  if (tokenize_string(str,end,tokens,1)) {
+  if (tokenize_string(str,end,tokens,DIR)) {
     log_fn(LOG_WARN, "Error tokenizing directory"); goto err;
   }
   if (smartlist_len(tokens) < 1) {
@@ -563,7 +615,7 @@ router_parse_runningrouters(const char *str, int write_to_cache)
     goto err;
   }
   tokens = smartlist_create();
-  if (tokenize_string(str,str+strlen(str),tokens,1)) {
+  if (tokenize_string(str,str+strlen(str),tokens,DIR)) {
     log_fn(LOG_WARN, "Error tokenizing directory"); goto err;
   }
   if ((tok = find_first_by_keyword(tokens, _UNRECOGNIZED))) {
@@ -612,7 +664,7 @@ router_parse_runningrouters(const char *str, int write_to_cache)
     goto err;
   }
   declared_key = find_dir_signing_key(str);
-  if (check_directory_signature(digest, tok, NULL, declared_key) < 0)
+  if (check_directory_signature(digest, tok, NULL, declared_key, 1) < 0)
     goto err;
 
   goto done;
@@ -648,7 +700,7 @@ find_dir_signing_key(const char *str)
     return NULL;
   ++cp; /* Now cp points to the start of the token. */
 
-  tok = get_next_token(&cp, DIR_ONLY);
+  tok = get_next_token(&cp, DIR);
   if (!tok) {
     log_fn(LOG_WARN, "Unparseable dir-signing-key token");
     return NULL;
@@ -701,7 +753,7 @@ dir_signing_key_is_trusted(crypto_pk_env_t *key)
  *
  * If <b>declared_key</b> is set, the directory has declared what key
  * was used to sign it, so we will use that key only if it is an
- * authoritative directory signing key.
+ * authoritative directory signing key or if check_authority is 0.
  *
  * Otherwise, if pkey is provided, try to use it.
  *
@@ -712,7 +764,8 @@ static int
 check_directory_signature(const char *digest,
                           directory_token_t *tok,
                           crypto_pk_env_t *pkey,
-                          crypto_pk_env_t *declared_key)
+                          crypto_pk_env_t *declared_key,
+                          int check_authority)
 {
   char signed_digest[PK_BYTES];
   crypto_pk_env_t *_pkey = NULL;
@@ -723,7 +776,7 @@ check_directory_signature(const char *digest,
   }
 
   if (declared_key) {
-    if (dir_signing_key_is_trusted(declared_key))
+    if (!check_authority || dir_signing_key_is_trusted(declared_key))
       _pkey = declared_key;
   }
   if (!_pkey && pkey) {
@@ -748,7 +801,7 @@ check_directory_signature(const char *digest,
     return -1;
   }
   log_fn(LOG_DEBUG,"Signed directory hash starts %s", hex_str(signed_digest,4));
-  if (memcmp(digest, signed_digest, 20)) {
+  if (memcmp(digest, signed_digest, DIGEST_LEN)) {
     log_fn(LOG_WARN, "Error reading directory: signature does not match.");
     return -1;
   }
@@ -833,6 +886,7 @@ router_parse_entry_from_string(const char *s, const char *end)
   directory_token_t *tok;
   int t;
   int ports_set, bw_set;
+  struct in_addr in;
 
   if (!end) {
     end = s + strlen(s);
@@ -843,7 +897,7 @@ router_parse_entry_from_string(const char *s, const char *end)
     return NULL;
   }
   tokens = smartlist_create();
-  if (tokenize_string(s,end,tokens,0)) {
+  if (tokenize_string(s,end,tokens,RTR)) {
     log_fn(LOG_WARN, "Error tokeninzing router descriptor.");
     goto err;
   }
@@ -866,6 +920,8 @@ router_parse_entry_from_string(const char *s, const char *end)
 
   router = tor_malloc_zero(sizeof(routerinfo_t));
   router->signed_descriptor = tor_strndup(s, end-s);
+  router->signed_descriptor_len = end-s;
+  crypto_digest(router->signed_descriptor_digest, s, end-s);
   ports_set = bw_set = 0;
 
   if (tok->n_args == 2 || tok->n_args == 5 || tok->n_args == 6) {
@@ -875,7 +931,11 @@ router_parse_entry_from_string(const char *s, const char *end)
       goto err;
     }
     router->address = tor_strdup(tok->args[1]);
-    router->addr = 0;
+    if (!tor_inet_aton(router->address, &in)) {
+      log_fn(LOG_WARN,"Router address is not an IP.");
+      goto err;
+    }
+    router->addr = ntohl(in.s_addr);
 
     if (tok->n_args >= 5) {
       router->or_port = (uint16_t) tor_parse_long(tok->args[2],10,0,65535,NULL,NULL);
@@ -925,6 +985,15 @@ router_parse_entry_from_string(const char *s, const char *end)
     }
   }
 
+  if ((tok = find_first_by_keyword(tokens, K_HIBERNATING))) {
+    if (tok->n_args < 1) {
+      log_fn(LOG_WARN, "Too few args on 'hibernating' keyword. Skipping.");
+    } else {
+      router->is_hibernating
+        = (tor_parse_long(tok->args[0],10,0,LONG_MAX,NULL,NULL) != 0);
+    }
+  }
+
   if (!(tok = find_first_by_keyword(tokens, K_PUBLISHED))) {
     log_fn(LOG_WARN, "Missing published time"); goto err;
   }
@@ -997,7 +1066,7 @@ router_parse_entry_from_string(const char *s, const char *end)
     log_fn(LOG_WARN, "Invalid signature %d",t);
     goto err;
   }
-  if (memcmp(digest, signed_digest, 20)) {
+  if (memcmp(digest, signed_digest, DIGEST_LEN)) {
     log_fn(LOG_WARN, "Mismatched signature");
     goto err;
   }
@@ -1023,7 +1092,6 @@ router_parse_entry_from_string(const char *s, const char *end)
   }
 
   goto done;
-  return router;
 
  err:
   routerinfo_free(router);
@@ -1039,10 +1107,325 @@ router_parse_entry_from_string(const char *s, const char *end)
   return router;
 }
 
-/** Parse the exit policy in the string <b>s</b> and return it.
+/** Helper: given a string <b>s</b>, return the start of the next router-status
+ * object (starting with "r " at the start of a line).  If none is found,
+ * return the start of the next directory signature.  If none is found, return
+ * the end of the string. */
+static INLINE const char *
+find_start_of_next_routerstatus(const char *s)
+{
+  const char *eos = strstr(s, "\nr ");
+  if (!eos)
+    eos = strstr(s, "\ndirectory-signature");
+  if (eos)
+    return eos+1;
+  else
+    return s + strlen(s);
+}
+
+/** Given a string at *<b>s</b>, containing a routerstatus object, and an
+ * empty smartlist at <b>tokens</b>, parse and return the first router status
+ * object in the string, and advance *<b>s</b> to just after the end of the
+ * router status.  Return NULL and advance *<b>s</b> on error. */
+static routerstatus_t *
+routerstatus_parse_entry_from_string(const char **s, smartlist_t *tokens)
+{
+#define BASE64_DIGEST_LEN 27
+  const char *eos;
+  routerstatus_t *rs = NULL;
+  directory_token_t *tok;
+  char base64buf_in[BASE64_DIGEST_LEN+3];
+  char base64buf_out[256];
+  char timebuf[ISO_TIME_LEN+1];
+  struct in_addr in;
+
+  tor_assert(tokens);
+
+  eos = find_start_of_next_routerstatus(*s);
+
+  if (tokenize_string(*s, eos, tokens, RTRSTATUS)) {
+    log_fn(LOG_WARN, "Error tokenizing router status");
+    goto err;
+  }
+  if (smartlist_len(tokens) < 1) {
+    log_fn(LOG_WARN, "Impossibly short router status");
+    goto err;
+  }
+  if ((tok = find_first_by_keyword(tokens, _UNRECOGNIZED))) {
+    log_fn(LOG_WARN, "Unrecognized keyword \"%s\" in router status; skipping.",
+           tok->args[0]);
+    goto err;
+  }
+  if (!(tok = find_first_by_keyword(tokens, K_R))) {
+    log_fn(LOG_WARN, "Missing 'r' keywork in router status; skipping.");
+    goto err;
+  }
+  if (tok->n_args < 8) {
+    log_fn(LOG_WARN,
+           "Too few arguments to 'r' keywork in router status; skipping.");
+  }
+  rs = tor_malloc_zero(sizeof(routerstatus_t));
+
+  if (!is_legal_nickname(tok->args[0])) {
+    log_fn(LOG_WARN,
+           "Invalid nickname '%s' in router status; skipping.", tok->args[0]);
+    goto err;
+  }
+  strlcpy(rs->nickname, tok->args[0], sizeof(rs->nickname));
+
+  if (strlen(tok->args[1]) != BASE64_DIGEST_LEN) {
+    log_fn(LOG_WARN, "Digest '%s' is wrong length in router status; skipping.",
+           tok->args[1]);
+    goto err;
+  }
+  memcpy(base64buf_in, tok->args[1], BASE64_DIGEST_LEN);
+  memcpy(base64buf_in+BASE64_DIGEST_LEN, "=\n\0", 3);
+  if (base64_decode(base64buf_out, sizeof(base64buf_out),
+                    base64buf_in, sizeof(base64buf_in)-1) != DIGEST_LEN) {
+    log_fn(LOG_WARN, "Error decoding digest '%s'", tok->args[1]);
+    goto err;
+  }
+  memcpy(rs->identity_digest, base64buf_out, DIGEST_LEN);
+
+  if (strlen(tok->args[2]) != BASE64_DIGEST_LEN) {
+    log_fn(LOG_WARN, "Digest '%s' is wrong length in router status; skipping.",
+           tok->args[2]);
+    goto err;
+  }
+  memcpy(base64buf_in, tok->args[2], BASE64_DIGEST_LEN);
+  memcpy(base64buf_in+BASE64_DIGEST_LEN, "=\n\0", 3);
+  if (base64_decode(base64buf_out, sizeof(base64buf_out),
+                    base64buf_in, sizeof(base64buf_in)-1) != DIGEST_LEN) {
+    log_fn(LOG_WARN, "Error decoding digest '%s'", tok->args[2]);
+    goto err;
+  }
+  memcpy(rs->descriptor_digest, base64buf_out, DIGEST_LEN);
+
+  if (tor_snprintf(timebuf, sizeof(timebuf), "%s %s",
+                   tok->args[3], tok->args[4]) < 0 ||
+      parse_iso_time(timebuf, &rs->published_on)<0) {
+    log_fn(LOG_WARN, "Error parsing time '%s %s'", tok->args[3], tok->args[4]);
+    goto err;
+  }
+
+  if (tor_inet_aton(tok->args[5], &in) == 0) {
+    log_fn(LOG_WARN, "Error parsing address '%s'", tok->args[5]);
+    goto err;
+  }
+  rs->addr = ntohl(in.s_addr);
+
+  rs->or_port =(uint16_t) tor_parse_long(tok->args[6],10,0,65535,NULL,NULL);
+  rs->dir_port = (uint16_t) tor_parse_long(tok->args[7],10,0,65535,NULL,NULL);
+
+  if ((tok = find_first_by_keyword(tokens, K_S))) {
+    int i;
+    for (i=0; i < tok->n_args; ++i) {
+      if (!strcmp(tok->args[i], "Exit"))
+        rs->is_exit = 1;
+      else if (!strcmp(tok->args[i], "Stable"))
+        rs->is_stable = 1;
+      else if (!strcmp(tok->args[i], "Fast"))
+        rs->is_fast = 1;
+      else if (!strcmp(tok->args[i], "Running"))
+        rs->is_running = 1;
+      else if (!strcmp(tok->args[i], "Named"))
+        rs->is_named = 1;
+      else if (!strcmp(tok->args[i], "Valid"))
+        rs->is_valid = 1;
+    }
+  }
+
+  goto done;
+ err:
+  if (rs)
+    routerstatus_free(rs);
+  rs = NULL;
+ done:
+  SMARTLIST_FOREACH(tokens, directory_token_t *, t, token_free(t));
+  smartlist_clear(tokens);
+  *s = eos;
+
+  return rs;
+}
+
+/** Given a versioned (v2 or later) network-status object in <b>s</b>, try to
+ * parse it and return the result.  Return NULL on failure.  Check the
+ * signature of the network status, but do not (yet) check the signing key for
+ * authority.
+ */
+networkstatus_t *
+networkstatus_parse_from_string(const char *s)
+{
+  const char *eos;
+  smartlist_t *tokens = smartlist_create();
+  networkstatus_t *ns = NULL;
+  char ns_digest[DIGEST_LEN];
+  char tmp_digest[DIGEST_LEN];
+  struct in_addr in;
+  directory_token_t *tok;
+
+  if (router_get_networkstatus_v2_hash(s, ns_digest)) {
+    log_fn(LOG_WARN, "Unable to compute digest of network-status");
+    goto err;
+  }
+
+  eos = find_start_of_next_routerstatus(s);
+  if (tokenize_string(s, eos, tokens, NETSTATUS)) {
+    log_fn(LOG_WARN, "Error tokenizing network-status header.");
+    goto err;
+  }
+  if ((tok = find_first_by_keyword(tokens, _UNRECOGNIZED))) {
+    log_fn(LOG_WARN, "Unrecognized keyword '%s'; can't parse network-status",
+           tok->args[0]);
+    goto err;
+  }
+  ns = tor_malloc_zero(sizeof(networkstatus_t));
+  memcpy(ns->networkstatus_digest, ns_digest, DIGEST_LEN);
+
+  if (!(tok = find_first_by_keyword(tokens, K_NETWORK_STATUS_VERSION))) {
+    log_fn(LOG_WARN, "Couldn't find network-status-version keyword");
+    goto err;
+  }
+  /* XXXX do something with the version? NM */
+
+  if (!(tok = find_first_by_keyword(tokens, K_DIR_SOURCE))) {
+    log_fn(LOG_WARN, "Couldn't find dir-source keyword");
+    goto err;
+  }
+  if (tok->n_args < 3) {
+    log_fn(LOG_WARN, "Too few arguments to dir-source keyword");
+    goto err;
+  }
+  ns->source_address = tok->args[0]; tok->args[0] = NULL;
+  if (tor_inet_aton(tok->args[1], &in) == 0) {
+    log_fn(LOG_WARN, "Error parsing address '%s'", tok->args[1]);
+    goto err;
+  }
+  ns->source_addr = ntohl(in.s_addr);
+  ns->source_dirport =
+    (uint16_t) tor_parse_long(tok->args[2],10,0,65535,NULL,NULL);
+  if (ns->source_dirport == 0) {
+    log_fn(LOG_WARN, "Directory source without dirport; skipping.");
+    goto err;
+  }
+
+  if (!(tok = find_first_by_keyword(tokens, K_FINGERPRINT))) {
+    log_fn(LOG_WARN, "Couldn't find fingerprint keyword");
+    goto err;
+  }
+  if (tok->n_args < 1) {
+    log_fn(LOG_WARN, "Too few arguments to fingerprint");
+    goto err;
+  }
+  if (base16_decode(ns->identity_digest, DIGEST_LEN, tok->args[0],
+                    strlen(tok->args[0]))) {
+    log_fn(LOG_WARN, "Couldn't decode fingerprint '%s'", tok->args[0]);
+    goto err;
+  }
+
+  if ((tok = find_first_by_keyword(tokens, K_CONTACT)) && tok->n_args) {
+    ns->contact = tok->args[0];
+    tok->args[0] = NULL;
+  }
+
+  if (!(tok = find_first_by_keyword(tokens, K_DIR_SIGNING_KEY)) || !tok->key) {
+    log_fn(LOG_WARN, "Missing dir-signing-key");
+    goto err;
+  }
+  ns->signing_key = tok->key;
+  tok->key = NULL;
+
+  if (crypto_pk_get_digest(ns->signing_key, tmp_digest)<0) {
+    log_fn(LOG_WARN, "Couldn't compute signing key digest");
+    goto err;
+  }
+  if (memcmp(tmp_digest, ns->identity_digest, DIGEST_LEN)) {
+    log_fn(LOG_WARN, "network-status fingerprint did not match dir-signing-key");
+    goto err;
+  }
+
+  if (!(tok = find_first_by_keyword(tokens, K_CLIENT_VERSIONS)) || tok->n_args<1) {
+    log_fn(LOG_WARN, "Missing client-versions");
+    goto err;
+  }
+  ns->client_versions = tok->args[0];
+
+  if (!(tok = find_first_by_keyword(tokens, K_CLIENT_VERSIONS)) || tok->n_args<1) {
+    log_fn(LOG_WARN, "Missing client-versions");
+    goto err;
+  }
+  ns->client_versions = tok->args[0];
+  tok->args[0] = NULL;
+
+  if (!(tok = find_first_by_keyword(tokens, K_SERVER_VERSIONS)) || tok->n_args<1) {
+    log_fn(LOG_WARN, "Missing server-versions");
+    goto err;
+  }
+  ns->server_versions = tok->args[0];
+  tok->args[0] = NULL;
+
+  if (!(tok = find_first_by_keyword(tokens, K_PUBLISHED))) {
+    log_fn(LOG_WARN, "Missing published time on directory.");
+    goto err;
+  }
+  tor_assert(tok->n_args == 1);
+  if (parse_iso_time(tok->args[0], &ns->published_on) < 0) {
+     goto err;
+  }
+
+  if ((tok = find_first_by_keyword(tokens, K_DIR_OPTIONS))) {
+    int i;
+    for (i=0; i < tok->n_args; ++i) {
+      if (!strcmp(tok->args[i], "Names"))
+        ns->binds_names = 1;
+    }
+  }
+
+  ns->entries = smartlist_create();
+  s = eos;
+  SMARTLIST_FOREACH(tokens, directory_token_t *, t, token_free(t));
+  smartlist_clear(tokens);
+  while (!strcmpstart(s, "r ")) {
+    routerstatus_t *rs;
+    if ((rs = routerstatus_parse_entry_from_string(&s, tokens)))
+      smartlist_add(ns->entries, rs);
+  }
+
+  if (tokenize_string(s, NULL, tokens, NETSTATUS)) {
+    log_fn(LOG_WARN, "Error tokenizing network-status footer.");
+    goto err;
+  }
+  if (smartlist_len(tokens) < 1) {
+    log_fn(LOG_WARN, "Too few items in network-status footer.");
+    goto err;
+  }
+  tok = smartlist_get(tokens, smartlist_len(tokens)-1);
+  if (tok->tp != K_DIRECTORY_SIGNATURE) {
+    log_fn(LOG_WARN, "Expected network-status footer to end with a signature.");
+    goto err;
+  }
+
+  if (check_directory_signature(ns_digest, tok, NULL, ns->signing_key, 0))
+    goto err;
+
+  goto done;
+ err:
+  if (ns)
+    networkstatus_free(ns);
+  ns = NULL;
+ done:
+  SMARTLIST_FOREACH(tokens, directory_token_t *, t, token_free(t));
+  smartlist_free(tokens);
+
+  return ns;
+}
+
+/** Parse the exit policy in the string <b>s</b> and return it.  If
+ * assume_action is nonnegative, then insert its action (ADDR_POLICY_ACCEPT or
+ * ADDR_POLICY_REJECT) for items that specify no action.
  */
 addr_policy_t *
-router_parse_addr_policy_from_string(const char *s)
+router_parse_addr_policy_from_string(const char *s, int assume_action)
 {
   directory_token_t *tok = NULL;
   const char *cp;
@@ -1058,7 +1441,16 @@ router_parse_addr_policy_from_string(const char *s)
   }
   tmp[len]='\n';
   tmp[len+1]='\0';
-  tok = get_next_token(&cp, RTR_ONLY);
+  while (TOR_ISSPACE(*cp))
+    ++cp;
+  if ((*cp == '*' || TOR_ISDIGIT(*cp)) && assume_action >= 0) {
+    char *new_str = tor_malloc(len+10);
+    tor_snprintf(new_str, len+10, "%s %s\n",
+                 assume_action == ADDR_POLICY_ACCEPT?"accept":"reject", cp);
+    tor_free(tmp);
+    cp = tmp = new_str;
+  }
+  tok = get_next_token(&cp, RTR);
   if (tok->tp == _ERR) {
     log_fn(LOG_WARN, "Error reading exit policy: %s", tok->error);
     goto err;
@@ -1084,7 +1476,7 @@ int
 router_add_exit_policy_from_string(routerinfo_t *router, const char *s)
 {
   addr_policy_t *newe, *tmpe;
-  newe = router_parse_addr_policy_from_string(s);
+  newe = router_parse_addr_policy_from_string(s, -1);
   if (!newe)
     return -1;
   for (tmpe = router->exit_policy; tmpe; tmpe=tmpe->next)
@@ -1167,7 +1559,7 @@ assert_addr_policy_ok(addr_policy_t *t)
     tor_assert(t->policy_type == ADDR_POLICY_REJECT ||
                t->policy_type == ADDR_POLICY_ACCEPT);
     tor_assert(t->prt_min <= t->prt_max);
-    t2 = router_parse_addr_policy_from_string(t->string);
+    t2 = router_parse_addr_policy_from_string(t->string, -1);
     tor_assert(t2);
     tor_assert(t2->policy_type == t->policy_type);
     tor_assert(t2->addr == t->addr);
@@ -1207,8 +1599,8 @@ token_free(directory_token_t *tok)
 }
 
 /** Helper function: read the next token from *s, advance *s to the end
- * of the token, and return the parsed token.  If 'where' is DIR_ONLY
- * or RTR_ONLY, reject all tokens of the wrong type.
+ * of the token, and return the parsed token.  If 'where' is DIR
+ * or RTR, reject all tokens of the wrong type.
  */
 static directory_token_t *
 get_next_token(const char **s, where_syntax where)
@@ -1255,11 +1647,15 @@ get_next_token(const char **s, where_syntax where)
       tok->tp = token_table[i].v;
       a_syn = token_table[i].s;
       o_syn = token_table[i].os;
-      if (token_table[i].ws != ANY && token_table[i].ws != where) {
-        if (where == DIR_ONLY) {
-          RET_ERR("Found a router-only token in a directory section");
+      if (!(token_table[i].ws & where)) {
+        if (where == DIR) {
+          RET_ERR("Found an out-of-place token in a directory section");
+        } else if (where == RTR) {
+          RET_ERR("Found an out-of-place token in a router descriptor");
+        } else if (where == NETSTATUS) {
+          RET_ERR("Found an out-of-place token in a network-status header");
         } else {
-          RET_ERR("Found a directory-only token in a router descriptor");
+          RET_ERR("Found an out-of-place token in a router status body");
         }
       }
       if (a_syn == ARGS) {
@@ -1399,12 +1795,13 @@ get_next_token(const char **s, where_syntax where)
  */
 static int
 tokenize_string(const char *start, const char *end, smartlist_t *out,
-                int is_dir)
+                where_syntax where)
 {
   const char **s;
   directory_token_t *tok = NULL;
-  where_syntax where = is_dir ? DIR_ONLY : RTR_ONLY;
   s = &start;
+  if (!end)
+    end = start+strlen(start);
   while (*s < end && (!tok || tok->tp != _EOF)) {
     tok = get_next_token(s, where);
     if (tok->tp == _ERR) {
