@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: test.c,v 1.188 2005/07/23 01:58:05 nickm Exp $ */
-const char test_c_id[] = "$Id: test.c,v 1.188 2005/07/23 01:58:05 nickm Exp $";
+/* $Id: test.c,v 1.200 2005/09/10 01:42:42 nickm Exp $ */
+const char test_c_id[] = "$Id: test.c,v 1.200 2005/09/10 01:42:42 nickm Exp $";
 
 /**
  * \file test.c
@@ -35,6 +35,7 @@ void get_platform_str(char *platform, size_t len);
 int is_obsolete_version(const char *myversion, const char *start);
 size_t read_escaped_data(const char *data, size_t len, int translate_newlines,
                          char **out);
+or_options_t *options_new(void);
 
 static char temp_dir[256];
 
@@ -73,55 +74,21 @@ get_fname(const char *name)
 static void
 remove_directory(void)
 {
-#ifdef MS_WINDOWS
-  char *pattern;
-  HANDLE handle;
-  WIN32_FIND_DATA findData;
-
-  setup_directory();
-  pattern = tor_malloc(strlen(temp_dir)+16);
-  tor_snprintf(pattern, strlen(temp_dir)+16, "%s\\*", temp_dir);
-  handle = FindFirstFile(pattern, &findData);
-  if (handle == INVALID_HANDLE_VALUE) {
-    perror("Can't remove");
-    return;
+  smartlist_t *elements = tor_listdir(temp_dir);
+  if (elements) {
+    SMARTLIST_FOREACH(elements, const char *, cp,
+       {
+         size_t len = strlen(cp)+strlen(temp_dir)+16;
+         char *tmp = tor_malloc(len);
+         tor_snprintf(tmp, len, "%s"PATH_SEPARATOR"%s", temp_dir, cp);
+         unlink(tmp);
+         tor_free(tmp);
+       });
+    SMARTLIST_FOREACH(elements, char *, cp, tor_free(cp));
+    smartlist_free(elements);
   }
-  while (1) {
-    size_t dlen = strlen(findData.cFileName)+strlen(temp_dir)+16;
-    char *deleteable = tor_malloc(dlen);
-    tor_snprintf(deleteable, dlen, "%s\\%s", temp_dir, findData.cFileName);
-    unlink(deleteable);
-    tor_free(deleteable);
-    if (!FindNextFile(handle, &findData)) {
-      if (GetLastError() != ERROR_NO_MORE_FILES) {
-        perror("error reading dir");
-      }
-      break;
-    }
-  }
-  FindClose(handle);
-  tor_free(pattern);
-#else
-  DIR *dirp;
-  struct dirent *de;
-  setup_directory();
-  if (!(dirp = opendir(temp_dir))) {
-    perror("Can't open temporary directory to remove files");
-    return;
-  }
-  while ((de = readdir(dirp)) != NULL) {
-    /* Only "." and ".." start with ., since we don't create any dotfiles. */
-    if (de->d_name[0] == '.') continue;
-    if (unlink(get_fname(de->d_name))) {
-      perror("Error removing file");
-    }
-#if 0
-    printf("==%s\n", de->d_name);
-#endif
-  }
-  closedir(dirp);
-#endif
   rmdir(temp_dir);
+  
 }
 
 static void
@@ -602,6 +569,20 @@ test_crypto_s2k(void)
   test_memeq(buf, buf2, 29);
 }
 
+static int
+_compare_strs(const void **a, const void **b)
+{
+  const char *s1 = *a, *s2 = *b;
+  return strcmp(s1, s2);
+}
+
+static int
+_compare_without_first_ch(const void *a, const void **b)
+{
+  const char *s1 = a, *s2 = *b;
+  return strcasecmp(s1+1, s2);
+}
+
 static void
 test_util(void)
 {
@@ -675,12 +656,12 @@ test_util(void)
   smartlist_insert(sl, 1, (void*)22);
   smartlist_insert(sl, 0, (void*)0);
   smartlist_insert(sl, 5, (void*)555);
-  test_eq((void*)0,   smartlist_get(sl,0));
-  test_eq((void*)1,   smartlist_get(sl,1));
-  test_eq((void*)22,  smartlist_get(sl,2));
-  test_eq((void*)3,   smartlist_get(sl,3));
-  test_eq((void*)4,   smartlist_get(sl,4));
-  test_eq((void*)555, smartlist_get(sl,5));
+  test_eq_ptr((void*)0,   smartlist_get(sl,0));
+  test_eq_ptr((void*)1,   smartlist_get(sl,1));
+  test_eq_ptr((void*)22,  smartlist_get(sl,2));
+  test_eq_ptr((void*)3,   smartlist_get(sl,3));
+  test_eq_ptr((void*)4,   smartlist_get(sl,4));
+  test_eq_ptr((void*)555, smartlist_get(sl,5));
 
   smartlist_clear(sl);
   smartlist_split_string(sl, "abc", ":", 0, 0);
@@ -774,6 +755,21 @@ test_util(void)
   test_eq(2, smartlist_len(sl));
   test_streq("efgh", smartlist_get(sl, 1));
 
+  SMARTLIST_FOREACH(sl, char *, cp, tor_free(cp));
+  smartlist_clear(sl);
+
+  /* Test smartlist sorting. */
+  smartlist_split_string(sl, "the,onion,router,by,arma,and,nickm", ",", 0, 0);
+  test_eq(7, smartlist_len(sl));
+  smartlist_sort(sl, _compare_strs);
+  cp = smartlist_join_strings(sl, ",", 0, NULL);
+  test_streq(cp,"and,arma,by,nickm,onion,router,the");
+  tor_free(cp);
+
+  test_streq("nickm", smartlist_bsearch(sl, "zNicKM", _compare_without_first_ch));
+  test_streq("and", smartlist_bsearch(sl, " AND", _compare_without_first_ch));
+  test_eq_ptr(NULL, smartlist_bsearch(sl, " ANz", _compare_without_first_ch));
+
   /* Test tor_strstrip() */
   strcpy(buf, "Testing 1 2 3");
   test_eq(0, tor_strstrip(buf, ",!"));
@@ -906,7 +902,7 @@ test_gzip(void)
   char *buf1, *buf2=NULL, *buf3=NULL;
   size_t len1, len2;
 
-  buf1 = tor_strdup("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA");
+  buf1 = tor_strdup("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAZAAAAAAAAAAAAAAAAAAAZ");
   test_eq(detect_compression_method(buf1, strlen(buf1)), 0);
   if (is_gzip_supported()) {
     test_assert(!tor_gzip_compress(&buf2, &len1, buf1, strlen(buf1)+1,
@@ -933,6 +929,16 @@ test_gzip(void)
   test_assert(buf3);
   test_streq(buf1,buf3);
 
+  tor_free(buf3);
+  buf2 = tor_realloc(buf2, len1*2);
+  memcpy(buf2+len1, buf2, len1);
+  test_assert(!tor_gzip_uncompress(&buf3, &len2, buf2, len1*2, ZLIB_METHOD));
+  test_eq(len2, (strlen(buf1)+1)*2);
+  test_memeq(buf3,
+             "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAZAAAAAAAAAAAAAAAAAAAZ\0"
+             "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAZAAAAAAAAAAAAAAAAAAAZ\0",
+             (strlen(buf1)+1)*2);
+
   tor_free(buf2);
   tor_free(buf3);
   tor_free(buf1);
@@ -967,14 +973,14 @@ test_strmap(void)
   test_eq(v, NULL);
   v = strmap_set(map, "K1", (void*)100);
   test_eq(v, (void*)99);
-  test_eq(strmap_get(map,"K1"), (void*)100);
-  test_eq(strmap_get(map,"K2"), (void*)101);
-  test_eq(strmap_get(map,"K-not-there"), NULL);
+  test_eq_ptr(strmap_get(map,"K1"), (void*)100);
+  test_eq_ptr(strmap_get(map,"K2"), (void*)101);
+  test_eq_ptr(strmap_get(map,"K-not-there"), NULL);
 
   v = strmap_remove(map,"K2");
-  test_eq(v, (void*)101);
-  test_eq(strmap_get(map,"K2"), NULL);
-  test_eq(strmap_remove(map,"K2"), NULL);
+  test_eq_ptr(v, (void*)101);
+  test_eq_ptr(strmap_get(map,"K2"), NULL);
+  test_eq_ptr(strmap_remove(map,"K2"), NULL);
 
   strmap_set(map, "K2", (void*)101);
   strmap_set(map, "K3", (void*)102);
@@ -985,9 +991,9 @@ test_strmap(void)
   count = 0;
   strmap_foreach(map, _squareAndRemoveK4, &count);
   test_eq(count, 1);
-  test_eq(strmap_get(map, "K4"), NULL);
-  test_eq(strmap_get(map, "K1"), (void*)10000);
-  test_eq(strmap_get(map, "K6"), (void*)11025);
+  test_eq_ptr(strmap_get(map, "K4"), NULL);
+  test_eq_ptr(strmap_get(map, "K1"), (void*)10000);
+  test_eq_ptr(strmap_get(map, "K6"), (void*)11025);
 
   iter = strmap_iter_init(map);
   strmap_iter_get(iter,&k,&v);
@@ -1009,8 +1015,8 @@ test_strmap(void)
   test_assert(strmap_iter_done(iter));
 
   /* Make sure we removed K2, but not the others. */
-  test_eq(strmap_get(map, "K2"), NULL);
-  test_eq(strmap_get(map, "K5"), (void*)10816);
+  test_eq_ptr(strmap_get(map, "K2"), NULL);
+  test_eq_ptr(strmap_get(map, "K5"), (void*)10816);
 
   /* Clean up after ourselves. */
   strmap_free(map, NULL);
@@ -1018,11 +1024,11 @@ test_strmap(void)
   /* Now try some lc functions. */
   map = strmap_new();
   strmap_set_lc(map,"Ab.C", (void*)1);
-  test_eq(strmap_get(map,"ab.c"), (void*)1);
-  test_eq(strmap_get_lc(map,"AB.C"), (void*)1);
-  test_eq(strmap_get(map,"AB.C"), NULL);
-  test_eq(strmap_remove_lc(map,"aB.C"), (void*)1);
-  test_eq(strmap_get_lc(map,"AB.C"), NULL);
+  test_eq_ptr(strmap_get(map,"ab.c"), (void*)1);
+  test_eq_ptr(strmap_get_lc(map,"AB.C"), (void*)1);
+  test_eq_ptr(strmap_get(map,"AB.C"), NULL);
+  test_eq_ptr(strmap_remove_lc(map,"aB.C"), (void*)1);
+  test_eq_ptr(strmap_get_lc(map,"AB.C"), NULL);
   strmap_free(map,NULL);
 }
 
@@ -1282,11 +1288,9 @@ test_dir_format(void)
   r1.published_on = time(NULL);
   r2.published_on = time(NULL)-3*60*60;
   test_assert(router_dump_router_to_string(buf, 2048, &r1, pk2)>0);
-  cp = buf;
-  test_eq(dirserv_add_descriptor((const char**)&cp,&m), 1);
+  test_eq(dirserv_add_descriptor(buf,&m), 2);
   test_assert(router_dump_router_to_string(buf, 2048, &r2, pk1)>0);
-  cp = buf;
-  test_eq(dirserv_add_descriptor((const char**)&cp,&m), 1);
+  test_eq(dirserv_add_descriptor(buf,&m), 2);
   get_options()->Nickname = tor_strdup("DirServer");
   test_assert(!dirserv_dump_directory_to_string(&cp,pk3));
   test_assert(!router_parse_routerlist_from_directory(cp, &dir1, pk3, 1, 0));
@@ -1385,7 +1389,7 @@ test_exit_policies(void)
 {
   addr_policy_t *policy;
 
-  policy = router_parse_addr_policy_from_string("reject 192.168.0.0/16:*");
+  policy = router_parse_addr_policy_from_string("reject 192.168.0.0/16:*",-1);
   test_eq(NULL, policy->next);
   test_eq(ADDR_POLICY_REJECT, policy->policy_type);
   test_eq(0xc0a80000u, policy->addr);
@@ -1395,11 +1399,11 @@ test_exit_policies(void)
   test_streq("reject 192.168.0.0/16:*", policy->string);
 
   test_assert(exit_policy_implicitly_allows_local_networks(policy, 0));
-  test_eq(ADDR_POLICY_ACCEPTED,
+  test_assert(ADDR_POLICY_ACCEPTED ==
           router_compare_addr_to_addr_policy(0x01020304u, 2, policy));
-  test_eq(ADDR_POLICY_PROBABLY_ACCEPTED,
+  test_assert(ADDR_POLICY_PROBABLY_ACCEPTED ==
           router_compare_addr_to_addr_policy(0, 2, policy));
-  test_eq(ADDR_POLICY_REJECTED,
+  test_assert(ADDR_POLICY_REJECTED ==
           router_compare_addr_to_addr_policy(0xc0a80102, 2, policy));
 
   addr_policy_free(policy);
@@ -1508,10 +1512,10 @@ test_rend_fns(void)
   test_memeq(d2->intro_point_extend_info[1]->identity_digest,
              d1->intro_point_extend_info[1]->identity_digest, DIGEST_LEN);
 
-  test_eq(BAD_HOSTNAME, parse_extended_hostname(address1));
-  test_eq(ONION_HOSTNAME, parse_extended_hostname(address2));
-  test_eq(EXIT_HOSTNAME, parse_extended_hostname(address3));
-  test_eq(NORMAL_HOSTNAME, parse_extended_hostname(address4));
+  test_assert(BAD_HOSTNAME == parse_extended_hostname(address1));
+  test_assert(ONION_HOSTNAME == parse_extended_hostname(address2));
+  test_assert(EXIT_HOSTNAME == parse_extended_hostname(address3));
+  test_assert(NORMAL_HOSTNAME == parse_extended_hostname(address4));
 
   rend_service_descriptor_free(d1);
   rend_service_descriptor_free(d2);
@@ -1522,13 +1526,16 @@ test_rend_fns(void)
 int
 main(int c, char**v)
 {
-  or_options_t *options = tor_malloc_zero(sizeof(or_options_t));
+  or_options_t *options = options_new();
+  options->command = CMD_RUN_UNITTESTS;
   network_init();
+  setup_directory();
   options_init(options);
+  options->DataDirectory = tor_strdup(temp_dir);
   set_options(options);
 
   crypto_seed_rng();
-  setup_directory();
+
   rep_hist_init();
   atexit(remove_directory);
 
